@@ -16,8 +16,7 @@
 #include <gnome.h>
 #include <gdk/gdkx.h>
 
-#include "applet-lib.h"
-#include "applet-widget.h"
+#include <applet-widget.h>
 
 #include "battery.h"
 #include "session.h"
@@ -32,9 +31,7 @@ main(int argc, char ** argv)
   bindtextdomain (PACKAGE, GNOMELOCALEDIR);
   textdomain (PACKAGE);
 
-  /* Initialize the applet */
-  applet_widget_init("battery_applet", &parser, argc, argv, 0, NULL, argv[0],
-		     TRUE, 0, applet_start_new_applet, NULL);
+  applet_widget_init("battery_applet", VERSION, argc, argv, NULL, 0, NULL);
 
   /* Create the battery applet widget */
   make_new_battery_applet();
@@ -392,7 +389,7 @@ make_new_battery_applet(void)
 
   bat = g_new(BatteryData, 1);
 
-  bat->applet = applet_widget_new_with_param(param);
+  bat->applet = applet_widget_new("battery_applet");
 
   if (bat->applet == NULL)
     g_error(_("Can't create applet!\n"));
@@ -666,33 +663,7 @@ battery_setup_colors(BatteryData * bat)
 
 /* When we get a command to start a new widget. */
 void
-applet_start_new_applet(const gchar *param, gpointer data)
+applet_start_new_applet(const gchar *goad_id, gpointer data)
 {
   make_new_battery_applet();
 } /* applet_start_new_applet */
-
-
-error_t
-battery_parse_an_arg (int key, char *arg, struct argp_state *state)
-{
-	return 0;
-} /* parse_an_arg */
-
-/* These are the arguments that our application supports.  */
-struct argp_option arguments[] =
-{
-  { NULL, 0, NULL, 0, NULL, 0 }
-};
-
-struct argp parser =
-{
-  arguments,			/* Options.  */
-  battery_parse_an_arg,		/* The parser function.  */
-  NULL,				/* Some docs.  */
-  NULL,				/* Some more docs.  */
-  NULL,				/* Child arguments -- gnome_init fills
-				   this in for us.  */
-  NULL,				/* Help filter.  */
-  NULL				/* Translation domain; for the app it
-				   can always be NULL.  */
-};
