@@ -20,7 +20,6 @@
 #include <config.h>
 #include <gnome.h>
 #include <gdk/gdkx.h>
-#include <applet-lib.h>
 #include <applet-widget.h>
 
 #include "led.h"
@@ -112,6 +111,8 @@ cdplayer_play_pause(GtkWidget * w, gpointer data)
 			break;
 		}
 	}
+
+	return 0;
 }
 
 static int 
@@ -289,10 +290,10 @@ main(int argc, char **argv)
 {
 	GtkWidget *cdplayer;
 
-	applet_widget_init_defaults("cdplayer_applet", NULL, argc, argv, 0,
-				    NULL, argv[0]);
+	applet_widget_init("cdplayer_applet", VERSION, argc, argv,
+				    NULL, 0, NULL);
 
-	applet = applet_widget_new();
+	applet = applet_widget_new("cdplayer_applet");
 	if (!applet)
 		g_error("Can't create applet!\n");
 
@@ -301,7 +302,7 @@ main(int argc, char **argv)
 					   APPLET_WIDGET(applet)->globcfgpath);
 
 	if(cdplayer == NULL) {
-		gnome_panel_applet_abort_id(APPLET_WIDGET(applet)->applet_id);
+		applet_widget_abort_load(APPLET_WIDGET(applet));
 		return 1;
 	}
 
