@@ -673,10 +673,10 @@ int main(int argc, char **argv)
         bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 
-        applet_widget_init_defaults("diskusage_applet", NULL, argc, argv, 0,
-				    NULL,argv[0]);
+        applet_widget_init("diskusage_applet", NULL, argc, argv,
+				    NULL, 0, NULL);
 
-	applet = applet_widget_new();
+	applet = applet_widget_new("diskusage_applet");
 	if (!applet)
 		g_error("Can't create applet!\n");
 
