@@ -1,5 +1,5 @@
 /*#####################################################*/
-/*##           drivemount applet 0.2.0               ##*/
+/*##           drivemount applet 0.2.1               ##*/
 /*#####################################################*/
 
 #include "drivemount.h"
@@ -22,6 +22,7 @@
 #include "harddisk_h_out.xpm"
 
 static void about_cb (AppletWidget *widget, gpointer data);
+static gint device_is_in_mountlist(DriveData *dd);
 static dev_t get_device(gchar *file);
 static gint device_is_mounted(DriveData *dd);
 static void update_pixmap(DriveData *dd, gint t);
@@ -33,7 +34,7 @@ static void applet_change_orient(GtkWidget *w, PanelOrientType o, gpointer data)
 static gint applet_save_session(GtkWidget *widget, char *privcfgpath, char *globcfgpath, gpointer data);
 static void destroy_drive_widget(GtkWidget *widget, gpointer data);
 static DriveData * create_drive_widget(GtkWidget *applet);
-static void applet_start_new_applet(const gchar *param, gpointer data);
+static GtkWidget * applet_start_new_applet(const gchar *goad_id, const char **params, int nparams);
 
 static void about_cb (AppletWidget *widget, gpointer data)
 {
@@ -44,7 +45,7 @@ static void about_cb (AppletWidget *widget, gpointer data)
 	sprintf(version,"%d.%d.%d",DRIVEMOUNT_APPLET_VERSION_MAJ,
 		DRIVEMOUNT_APPLET_VERSION_MIN, DRIVEMOUNT_APPLET_VERSION_REV);
 
-	authors[0] = "John Ellis (gqview@geocities.com)";
+	authors[0] = "John Ellis <johne@bellatlantic.net>";
 	authors[1] = NULL;
 
         about = gnome_about_new ( _("Drive Mount Applet"), version,
@@ -57,6 +58,33 @@ static void about_cb (AppletWidget *widget, gpointer data)
 	gtk_widget_show (about);
 }
 
+static gint device_is_in_mountlist(DriveData *dd)
+{
+	FILE *fp;
+	gchar *command_line = "mount";
+	gchar buf[201];
+	gint found = FALSE;
+
+	buf[201] = '\0';
+
+	fp = popen(command_line, "r");
+
+	if (!fp)
+		{
+		printf("unable to run command: %s\n", command_line);
+		return FALSE;
+		}
+
+	while (fgets(buf, 200, fp) != NULL)
+		{
+		if (strstr(buf, dd->mount_point) != 0) found = TRUE;
+		}
+
+	pclose (fp);
+
+	return found;
+}
+
 static dev_t get_device(gchar *file)
 {
 	struct stat file_info;
@@ -72,10 +100,20 @@ static dev_t get_device(gchar *file)
 
 static gint device_is_mounted(DriveData *dd)
 {
-	if (get_device(dd->mount_base) == get_device(dd->mount_point))
-		return FALSE;
+	if (!dd->autofs_friendly)
+		{
+		if (get_device(dd->mount_base) == get_device(dd->mount_point))
+			return FALSE;
+		else
+			return TRUE;
+		}
 	else
-		return TRUE;
+		{
+		if (device_is_in_mountlist(dd))
+			return TRUE;
+		else
+			return FALSE;
+		}
 }
 
 static void update_pixmap(DriveData *dd, gint t)
@@ -139,8 +177,6 @@ static int mount_cb(GtkWidget *widget, gpointer data)
 	else
 		sprintf(command_line, "umount %s 2>&1", dd->mount_point);
 
-	system (command_line);
-
 	fp = popen(command_line, "r");
 
 	if (!fp)
@@ -320,7 +356,7 @@ void create_pixmaps(DriveData *dd)
 
 void redraw_pixmap(DriveData *dd)
 {
-	if (get_device(dd->mount_base) == get_device(dd->mount_point))
+	if (!device_is_mounted(dd))
 		{
 		update_pixmap(dd, FALSE);
 		dd->mounted = FALSE;
@@ -379,6 +415,7 @@ static DriveData * create_drive_widget(GtkWidget *applet)
 	dd->mount_point = NULL;
 	dd->propwindow = NULL;
 	dd->mount_base = g_strdup("/mnt");
+	dd->autofs_friendly = FALSE;
 
 	property_load(APPLET_WIDGET(applet)->privcfgpath, dd);
 
@@ -436,12 +473,12 @@ static DriveData * create_drive_widget(GtkWidget *applet)
 	return dd;
 }
 
-static void applet_start_new_applet(const gchar *param, gpointer data)
+static GtkWidget * applet_start_new_applet(const gchar *goad_id, const char **params, int nparams)
 {
 	DriveData *dd;
 	GtkWidget *applet;
 
-	applet = applet_widget_new_with_param(param);
+	applet = applet_widget_new(goad_id);
 		if (!applet)
 			g_error("Can't create applet!\n");
 
@@ -449,6 +486,8 @@ static void applet_start_new_applet(const gchar *param, gpointer data)
 
 	applet_widget_add(APPLET_WIDGET(applet), dd->button);
 	gtk_widget_show(applet);
+	
+	return applet;
 }
 
 int main (int argc, char *argv[])
@@ -460,10 +499,11 @@ int main (int argc, char *argv[])
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 
-	applet_widget_init("drivemount_applet", NULL, argc, argv, 0, NULL,
-				argv[0], TRUE, TRUE, applet_start_new_applet, NULL);
+	applet_widget_init("drivemount_applet", VERSION, argc, argv,
+			   NULL, 0, NULL);
+	applet_factory_new("drivemount_applet_factory", NULL, applet_start_new_applet);
 
-	applet = applet_widget_new();
+	applet = applet_widget_new("drivemount_applet");
 	if (!applet)
 		g_error("Can't create applet!\n");
 
