@@ -8,7 +8,7 @@
  */
 
 #include <config.h>
-#include <applet-lib.h>
+#include <applet-widget.h>
 
 
 #define PIECE_SIZE 11
@@ -26,13 +26,16 @@ free_stuff (GtkObject *object, gpointer data)
 static void
 test_win (GnomeCanvasItem **board)
 {
+        GtkWidget *dlg;
 	int i;
 
 	for (i = 0; i < 15; i++)
 		if (!board[i] || (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (board[i]), "piece_num")) != i))
 			return;
 
-	gnome_dialog_run_modal (GNOME_DIALOG (gnome_ok_dialog (_("You win!"))));
+        dlg = gnome_ok_dialog (_("You win!"));
+	gtk_window_set_modal(GTK_WINDOW(dlg),TRUE);
+	gnome_dialog_run (GNOME_DIALOG (dlg));
 }
 
 static char *
@@ -141,7 +144,7 @@ create_fifteen (void)
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
 
 	canvas = gnome_canvas_new ();
-	gnome_canvas_set_size (GNOME_CANVAS (canvas), PIECE_SIZE * 4, PIECE_SIZE * 4);
+	gtk_widget_set_usize (canvas, PIECE_SIZE * 4, PIECE_SIZE * 4);
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas), 0, 0, PIECE_SIZE * 4, PIECE_SIZE * 4);
 	gtk_container_add (GTK_CONTAINER (frame), canvas);
 	gtk_widget_show (canvas);
@@ -281,9 +284,10 @@ main (int argc, char **argv)
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 
-	applet_widget_init_defaults ("fifteen_applet", NULL, argc, argv, 0, NULL, argv[0]);
+	applet_widget_init ("fifteen_applet", VERSION, argc,
+				     argv, NULL, 0, NULL);
 
-	applet = applet_widget_new ();
+	applet = applet_widget_new ("fifteen_applet");
 	if (!applet)
 		g_error (_("Can't create fifteen applet!"));
 
