@@ -7,8 +7,7 @@
 #include <string.h>
 #include <gnome.h>
 #include <gdk_imlib.h>
-#include "applet-lib.h"
-#include "applet-widget.h"
+#include <applet-widget.h>
 
 typedef struct _fish_properties fish_properties;
 
@@ -23,7 +22,7 @@ static fish_properties defaults = {
 	"Wanda",
 	NULL,
 	3,
-	0.3
+	1.0
 };
 
 static fish_properties properties = { NULL };
@@ -167,7 +166,7 @@ apply_cb(GnomePropertyBox * pb, int page, gpointer data)
 	apply_properties();
 }
 
-static void
+static gint
 close_cb(GnomePropertyBox * pb, gpointer data)
 {
 	GtkWidget *name = gtk_object_get_data(GTK_OBJECT(pb),
@@ -176,6 +175,8 @@ close_cb(GnomePropertyBox * pb, gpointer data)
 					       "image");
 	gtk_signal_disconnect_by_data(GTK_OBJECT(name),pb);
 	gtk_signal_disconnect_by_data(GTK_OBJECT(image),pb);
+
+  return FALSE;
 }
 
 static void 
@@ -193,7 +194,7 @@ properties_dialog(AppletWidget *aw, gpointer data)
 	gtk_window_set_title(GTK_WINDOW(pb), _("GNOME Fish Properties"));
 
 	vbox = gtk_vbox_new(FALSE, GNOME_PAD);
-	gtk_container_border_width(GTK_CONTAINER(vbox), GNOME_PAD);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), GNOME_PAD);
 
 	hbox = gtk_hbox_new(FALSE, GNOME_PAD);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
@@ -367,7 +368,7 @@ create_fish_widget(GtkWidget *window)
 }
 
 /*the most important dialog in the whole application*/
-void
+static void
 about_cb (AppletWidget *widget, gpointer data)
 {
 	GtkWidget *about;
@@ -414,53 +415,86 @@ applet_save_session(GtkWidget *w,
 	return FALSE;
 }
 
+static CORBA_Object
+wanda_activator(PortableServer_POA poa,
+		const char *goad_id,
+		const char **params,
+		gpointer *impl_ptr,
+		CORBA_Environment *ev)
+{
+  GtkWidget *fish, *applet;
+  
+  applet = applet_widget_new(goad_id);
+
+  load_properties(APPLET_WIDGET(applet)->privcfgpath);
+
+  /*gtk_widget_realize(applet);*/
+  fish = create_fish_widget(applet);
+  gtk_widget_show(fish);
+  applet_widget_add(APPLET_WIDGET(applet), fish);
+  gtk_widget_show(applet);
+
+  gtk_signal_connect(GTK_OBJECT(applet),"save_session",
+		     GTK_SIGNAL_FUNC(applet_save_session),
+		     NULL);
+
+  applet_widget_register_stock_callback(APPLET_WIDGET(applet),
+					"about",
+					GNOME_STOCK_MENU_ABOUT,
+					_("About..."),
+					about_cb,
+					NULL);
 
 
+  applet_widget_register_stock_callback(APPLET_WIDGET(applet),
+					"properties",
+					GNOME_STOCK_MENU_PROP,
+					_("Properties..."),
+					properties_dialog,
+					NULL);
+
+  return applet_widget_corba_activate(applet, poa, goad_id, params,
+				      impl_ptr, ev);
+}
+
+static void
+wanda_deactivator(PortableServer_POA poa,
+		  const char *goad_id,
+		  gpointer impl_ptr,
+		  CORBA_Environment *ev)
+{
+  applet_widget_corba_deactivate(poa, goad_id, impl_ptr, ev);
+}
+
+#if 1 || defined(SHLIB_APPLETS)
+static const char *repo_id[]={"IDL:GNOME/Applet:1.0", NULL};
+static GnomePluginObject applets_list[] = {
+  {repo_id, "fish_applet", NULL, "Wanda the Magnificient",
+   &wanda_activator, &wanda_deactivator},
+  {NULL}
+};
+
+GnomePlugin GNOME_Plugin_info = {
+  applets_list, NULL
+};
+#else
 int
 main(int argc, char *argv[])
 {
-	GtkWidget *fish;
+        gpointer wanda_impl;
 
 	/* Initialize the i18n stuff */
         bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 
-	applet_widget_init_defaults("fish_applet", NULL, argc, argv, 0, NULL,
-				    argv[0]);
-
-	applet = applet_widget_new();
-	if (!applet)
-		g_error(_("Can't create applet!\n"));
-
-	load_properties(APPLET_WIDGET(applet)->privcfgpath);
-
-	gtk_widget_realize(applet);
-	fish = create_fish_widget(applet);
-	gtk_widget_show(fish);
-
-	applet_widget_add(APPLET_WIDGET(applet), fish);
-	gtk_widget_show(applet);
-	gtk_signal_connect(GTK_OBJECT(applet),"save_session",
-			   GTK_SIGNAL_FUNC(applet_save_session),
-			   NULL);
-
-	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
-					      "about",
-					      GNOME_STOCK_MENU_ABOUT,
-					      _("About..."),
-					      about_cb,
-					      NULL);
-
-
-	applet_widget_register_stock_callback(APPLET_WIDGET(applet),
-					      "properties",
-					      GNOME_STOCK_MENU_PROP,
-					      _("Properties..."),
-					      properties_dialog,
-					      NULL);
+	applet_widget_init("fish_applet", VERSION, argc, argv, NULL, 0, NULL);
 
+	APPLET_ACTIVATE(wanda_activator, "fish_applet", &wanda_impl);
 	
 	applet_widget_gtk_main();
 
+	APPLET_DEACTIVATE(wanda_deactivator, "fish_applet", wanda_impl);
+
 	return 0;
 }
+#endif
