@@ -17,8 +17,7 @@
 #include <gdk/gdkx.h>
 #include <gdk/gdkprivate.h>
 #include "WinMgr.h"
-#include "applet-lib.h"
-#include "applet-widget.h"
+#include <applet-widget.h>
 
 void   setup();
 void   switch_cb(GtkWidget *widget, gpointer data);
@@ -40,10 +39,8 @@ Atom _XA_WIN_STATE;
 
 int main(int argc, char *argv[])
 {
-
-    panel_corba_register_arguments();
-
-    applet_widget_init_defaults("wmpager_applet", NULL, argc, argv, 0, NULL, argv[0]);
+    applet_widget_init("wmpager_applet", VERSION, argc, argv,
+				NULL, 0, NULL);
 
     /* Get the Atom for making a window sticky, so that the detached pager appears on all screens */
     _XA_WIN_STATE = XInternAtom(GDK_DISPLAY(), XA_WIN_STATE, False);
@@ -57,7 +54,7 @@ int main(int argc, char *argv[])
         exit(1);
     }
 
-    applet = applet_widget_new();
+    applet = applet_widget_new("wmpager_applet");
     if (!applet)
         g_error("Can't create applet!\n");
 
@@ -174,7 +171,7 @@ void setup()
         button = gtk_toggle_button_new_with_label(ws_name);
         button_list = g_list_append(button_list, button);
         gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                           GTK_SIGNAL_FUNC(switch_cb), (int*)i);
+                           GTK_SIGNAL_FUNC(switch_cb), GINT_TO_POINTER (i));
         gtk_box_pack_start(GTK_BOX(hb), button, TRUE, TRUE, 0);
         gtk_widget_show(button);
     }
@@ -190,7 +187,7 @@ void setup()
         button = gtk_toggle_button_new_with_label((gchar*)(workspace_list->data));
         button_list = g_list_append(button_list, button);
         gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                           GTK_SIGNAL_FUNC(switch_cb), (int*)i);
+                           GTK_SIGNAL_FUNC(switch_cb), GINT_TO_POINTER (i));
         gtk_box_pack_start(GTK_BOX(hb), button, TRUE, TRUE, 0);
         gtk_widget_show(button);
     }
@@ -201,7 +198,7 @@ void setup()
         gtk_box_pack_start(GTK_BOX(hb), label, TRUE, TRUE, 0);
         button_list = g_list_append(button_list, button);
         gtk_signal_connect(GTK_OBJECT(button), "clicked",
-                           GTK_SIGNAL_FUNC(switch_cb), (int*)i);
+                           GTK_SIGNAL_FUNC(switch_cb), GINT_TO_POINTER (i));
         gtk_widget_show(label);
     }
 
@@ -215,7 +212,7 @@ void setup()
 void about_cb (AppletWidget *widget, gpointer data)
 {
     GtkWidget *about;
-    gchar *authors[] = {"M.Watson", NULL};
+    static const gchar *authors[] = {"M.Watson", NULL};
 
     about = gnome_about_new ( _("Desktop Pager Applet"), "0.0",
                               _("Copyright (C)1998 M.Watson"),
@@ -240,7 +237,7 @@ void switch_cb(GtkWidget *widget, gpointer data)
     gint tmp_ws;
 
     tmp_ws = get_current_workspace();
-    button_num = (int)data;
+    button_num = GPOINTER_TO_INT (data);
     if(GTK_TOGGLE_BUTTON(widget)->active)
     {
         if(button_num != tmp_ws)
