@@ -10,6 +10,9 @@
 #include "button_off.xpm"
 #include "button_on.xpm"
 
+#include <errno.h>
+#include <ctype.h>
+
 #ifdef __linux__
 #include <linux/isdn.h>
 #include <sys/ioctl.h>
@@ -78,14 +81,15 @@ static int get_stats(int *in, int *out);
 static void about_cb (AppletWidget *widget, gpointer data)
 {
 	GtkWidget *about;
-	const gchar *authors[2];
+	const gchar *authors[8];
 	gchar version[32];
 
 	sprintf(version,"%d.%d.%d",MODEMLIGHTS_APPLET_VERSION_MAJ,
 		MODEMLIGHTS_APPLET_VERSION_MIN, MODEMLIGHTS_APPLET_VERSION_REV);
 
-	authors[0] = "John Ellis (gqview@geocities.com)";
-	authors[1] = NULL;
+	authors[0] = "John Ellis <johne@bellatlantic.net>";
+	authors[1] = "Martin Baulig <martin@home-of-linux.org> - ISDN";
+	authors[2] = NULL;
 
         about = gnome_about_new ( _("Modem Lights Applet"), version,
 			"(C) 1998",
@@ -534,7 +538,7 @@ static gint update_display()
 }
 
 /* start or change the update callback timeout interval */
-void start_callback_update()
+void start_callback_update(void)
 {
 	gint delay;
 	delay = 1000 / UPDATE_DELAY;
@@ -642,8 +646,8 @@ int main (int argc, char *argv[])
 		load_hist_tx[i] = 0;
 		}
 
-	applet_widget_init_defaults("modemlights_applet", NULL, argc, argv, 0,
-				    NULL,argv[0]);
+	applet_widget_init("modemlights_applet", VERSION, argc, argv,
+				    NULL, 0, NULL);
 
 	lock_file = g_strdup("/var/lock/LCK..modem");
 	device_name = g_strdup("ppp0");
@@ -659,7 +663,7 @@ int main (int argc, char *argv[])
 		}
 
 
-	applet = applet_widget_new();
+	applet = applet_widget_new("modemlights_applet");
 	if (!applet)
 		g_error("Can't create applet!\n");
 
