@@ -14,8 +14,7 @@
 #include <net/ppp_defs.h>
 
 #include <gnome.h>
-#include "applet-lib.h"
-#include "applet-widget.h"
+#include <applet-widget.h>
 
 #define MODEMLIGHTS_APPLET_VERSION_MAJ 0
 #define MODEMLIGHTS_APPLET_VERSION_MIN 3
@@ -31,7 +30,7 @@ extern gint use_ISDN;
 
 extern GtkWidget *applet;
 
-void start_callback_update();
+void start_callback_update(void);
 
 void property_load(char *path);
 void property_save(char *path);
