@@ -16,7 +16,7 @@
 #include <config.h>
 #include <gnome.h>
 #include <gdk/gdkx.h>
-#include "applet-lib.h"
+#include <applet-widget.h>
 
 #include "linux-proc.h"
 #include "properties.h"
@@ -56,11 +56,11 @@ int draw(void)
 		data = (unsigned long int *)calloc(width, sizeof(unsigned long int));
 	}
 	
-	bytes = GetTraffic(1, props.device);
+	bytes = GetTraffic(props.device);
 
 	if (! init){
 		init = 1;
-		old_bytes = GetTraffic(1, props.device);
+		old_bytes = GetTraffic(props.device);
 	}
 
 	/* The MAX is in case the stats get reset or wrap. It'll get one wrong sample. */
@@ -241,7 +241,7 @@ static gint applet_save_session(GtkWidget *widget, char *privcfgpath, char *glob
 void
 error_close_cb(GtkWidget *widget, void *data)
 {
-	applet_widget_remove_from_panel(APPLET_WIDGET(widget));
+  applet_widget_remove(APPLET_WIDGET(widget));
 }
 
 /*
@@ -270,7 +270,7 @@ error_dialog(char *message, int fatal)
 		TRUE, TRUE, GNOME_PAD);
 
 	gnome_less_fixed_font(GNOME_LESS(less));
-	gnome_dialog_set_modal(GNOME_DIALOG(error));
+	gtk_window_set_modal(GTK_WINDOW(error),TRUE);
 
 	if (fatal)
 		gtk_signal_connect( GTK_OBJECT(error),
@@ -312,10 +312,10 @@ int main(int argc, char **argv)
         bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
 
-	applet_widget_init_defaults("netload_applet", NULL, argc, argv, 0,
-				    NULL,argv[0]);
+	applet_widget_init("netload_applet", VERSION, argc,
+				    argv, NULL, 0, NULL);
 
-	applet = applet_widget_new();
+	applet = applet_widget_new("netload_applet");
 	if (!applet)
 		g_error(_("Can't create applet!\n"));
 
