@@ -23,18 +23,19 @@
 #include <gnome.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <fcntl.h>
 #include <unistd.h>
+#include <stdio.h>
+#include <orb/orbit.h>
+#include <libgnorba/gnorba.h>
+#include <errno.h>
 
 #include "window.h"
 #include "history.h"
 #include "bookmarks.h"
 #include "toc2.h"
 #include "cache.h"
-
-#define UGLY_LE_HACK 1
-#ifdef UGLY_LE_HACK
-#include "check-running.h"
-#endif
+#include "help-browser.h"
 
 extern char *program_invocation_name;
 
@@ -55,15 +56,21 @@ void messageHandler(gchar *s);
 void warningHandler(gchar *s);
 void errorHandler(gchar *s);
 void setErrorHandlers(void);
-static HelpWindow makeHelpWindow(gint x, gint y, gint w, gint h);
+HelpWindow makeHelpWindow(gint x, gint y, gint w, gint h);
 static void initConfig(void);
 static void saveConfig(void);
 static GnomeClient *newGnomeClient(void);
-static error_t parseAnArg (int key, char *arg, struct argp_state *state);
 
 static void configApply(GtkWidget *w, int page, GtkWidget *window);
 static int configCancel(GtkWidget *w, GtkWidget *window);
 
+void Exception( CORBA_Environment*);
+help_browser_simple_browser 
+  impl_help_browser_simple_browser__create(PortableServer_POA poa,
+					   HelpWindow window, 
+					   CORBA_Environment * ev);
+void destroy_server(HelpWindow win);
+
 /* MANPATH should probably come from somewhere */
 #define DEFAULT_MANPATH "/usr/man:/usr/local/man:/usr/X11R6/man"
 #define DEFAULT_INFOPATH "/usr/info:/usr/local/info"
@@ -79,28 +86,6 @@ static int configCancel(GtkWidget *w, GtkWidget *window);
 #define DEFAULT_CACHEFILE "cache"
 #define DEFAULT_BOOKMARKFILE "bookmarks"
 
-/* Argument parsing.  */
-static struct argp_option options[] =
-{
-	{ NULL, 'x', N_("X"), 0, N_("X position of window"), 1 },
-	{ NULL, 'y', N_("Y"), 0, N_("Y position of window"), 1 },
-	{ NULL, 'w', N_("WIDTH"), 0, N_("Width of window"), 1 },
-	{ NULL, 'h', N_("HEIGHT"), 0, N_("Height of window"), 1 },
-	{ "debug", 'd', NULL, 0, N_("Debug level"), 0 },
-	{ NULL, 0, NULL, 0, NULL, 0 }
-};
-
-static struct argp parser =
-{
-    options,
-    parseAnArg,
-    N_("[URL]"),
-    NULL,
-    NULL,
-    NULL,
-    NULL
-};
-
 /* Config data */			  
 static gchar *manPath;			  
 static gchar *infoPath;			  
@@ -130,9 +115,18 @@ static gint defposy=0;
 static gint defwidth =0;
 static gint defheight=0;
 
-#ifdef UGLY_LE_HACK
+/* Argument parsing.  */
+static struct poptOption options[] = 
+{
+  {NULL, 'x', POPT_ARG_INT, &defposx, 0, N_("X position of window"), N_("X")},
+  {NULL, 'y', POPT_ARG_INT, &defposy, 0, N_("Y position of window"), N_("Y")},
+  {NULL, 'w', POPT_ARG_INT, &defwidth, 0, N_("Width of window"), N_("WIDTH")},
+  {NULL, 'h', POPT_ARG_INT, &defheight, 0, N_("Height of window"), N_("HEIGHT")},
+  {"debug", 'd', POPT_ARG_INT, &debugLevel, 0, N_("Debug level"), NULL},
+  {NULL, '\0', 0, NULL, 0}
+};
 
-HelpWindow ahelpwindow;
+#ifdef UGLY_LE_HACK
 
 void
 show_requested_url(char *url)
@@ -141,33 +135,67 @@ show_requested_url(char *url)
 }
 #endif
 
+void Exception( CORBA_Environment* ev )
+{
+  switch( ev->_major )
+    {
+    case CORBA_SYSTEM_EXCEPTION:
+      g_log("GNOME Help", G_LOG_LEVEL_DEBUG, "CORBA system exception %s.\n",
+	    CORBA_exception_id(ev));
+      exit ( 1 );
+    case CORBA_USER_EXCEPTION:
+      g_log("GNOME Help", G_LOG_LEVEL_DEBUG, "CORBA user exception: %s.\n",
+	    CORBA_exception_id( ev ) );
+      exit ( 1 );
+    default:
+      break;
+    }
+}
+
 int
 main(int argc, char *argv[])
 {
-    HelpWindow window;
-    gchar buf[BUFSIZ];
-
-    argp_program_version = HELP_VERSION;
-
+    HelpWindow                  window = 0;
+    gchar                       buf[BUFSIZ];
+    CORBA_ORB                   orb;
+    CORBA_Environment           ev;
+    PortableServer_POA          root_poa;
+    PortableServer_POAManager   pm;
+    help_browser_simple_browser browser_object;
+    CORBA_Object                name_service;
+    gchar*                      objref;
+    gchar **leftovers;
+    int            output_fd;
+    poptContext ctx;
+    
+    
+    CORBA_exception_init(&ev);
+    
     /* Initialize the i18n stuff */
     bindtextdomain (PACKAGE, GNOMELOCALEDIR);
     textdomain (PACKAGE);
 
-    gnome_init(NAME, &parser, argc, argv, 0, NULL);
+    orb = gnome_CORBA_init_with_popt_table(NAME, VERSION, &argc, argv,
+					   options, 0, &ctx, GNORBA_INIT_SERVER_FUNC, &ev);
 
-/* enable session management here */
-#if 0
-    smClient = NULL;
-    g_message("Session management was disabled at compile-time");
-#else
-    smClient = newGnomeClient();
-#endif
-
-#ifdef UGLY_LE_HACK
-    if (send_command_to_running(helpURL, show_requested_url)) {
-	exit(0);
+    leftovers = poptGetArgs(ctx);
+    if(leftovers && leftovers[0]) {
+      helpURL = leftovers[0];
+      if(*helpURL == '/')
+	helpURL = g_copy_strings("file:", helpURL, NULL);
+      else
+	helpURL = g_strdup(helpURL);
     }
-#endif
+    poptFreeContext(ctx);
+
+    Exception(&ev);
+    
+    root_poa = (PortableServer_POA) 
+      CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
+    Exception(&ev);
+    
+/* enable session management here */
+    smClient = newGnomeClient();
 
     initConfig();
 
@@ -189,28 +217,58 @@ main(int argc, char *argv[])
     g_snprintf(buf, sizeof(buf), "%s/%s", HELP_BROWSER_RC_DIR, bookmarkFile);
     bookmarkWindow = newBookmarks(bookmarkCallback, NULL, buf);
 
-    window = makeHelpWindow(defposx, defposy, defwidth, defheight );
-#ifdef UGLY_LE_HACK
-    ahelpwindow=window;
-#endif
+    window = makeHelpWindow(defposx, defposy, defwidth, defheight);
+    if (!helpURL)
+      helpURL = "toc:";
+    helpWindowShowURL(window, helpURL, TRUE, TRUE);
+
+    browser_object =  impl_help_browser_simple_browser__create(root_poa, 
+							       window, &ev);
+    Exception(&ev);
+    
+    objref = CORBA_ORB_object_to_string(orb, browser_object, &ev);
+    Exception(&ev);
+
+    pm = PortableServer_POA__get_the_POAManager(root_poa, &ev);
+    Exception(&ev);
+
+    PortableServer_POAManager_activate(pm, &ev);
+    Exception(&ev);
+
+    name_service = gnome_name_service_get();
+    if (!CORBA_Object_is_nil(name_service, &ev))
+      {
+	goad_server_register(name_service, browser_object, "gnome-help-browser", "object", &ev);
+        printf("%s\n", objref); fflush(stdout);
+      }
+    CORBA_free(objref);
+    CORBA_Object_release(name_service, &ev);
+
+    output_fd = open("/tmp/gnome-help-browser.log", O_CREAT | O_WRONLY
+		     | O_APPEND, 0666);
+    setvbuf(stderr, 0, _IOLBF, 0);
+    setvbuf(stdout, 0, _IOLBF, 0);
+    dup2(output_fd, fileno(stdout));
+    dup2(output_fd, fileno(stderr));
+    close(output_fd);
+
+    gtk_widget_show (helpWindowGetAppWindow (window));
 
-    if (helpURL)
-	    helpWindowShowURL(window, helpURL, TRUE, TRUE);
-    else {
-	    helpWindowShowURL(window, "toc:", TRUE, TRUE);
-    }
-	 
-	
     gtk_main();
 
     saveHistory(historyWindow);
     saveBookmarks(bookmarkWindow);
     saveCache(cache);
-	
+
+    if (!CORBA_Object_is_nil(name_service, &ev))
+      goad_server_unregister(name_service, "help-browser", "object", &ev);
+
+    Exception(&ev);
+    
     return 0;
 }
 
-static HelpWindow
+HelpWindow
 makeHelpWindow(gint x, gint y, gint w, gint h)
 {
     HelpWindow window;
@@ -225,60 +283,9 @@ makeHelpWindow(gint x, gint y, gint w, gint h)
     helpWindowSetBookmarks(window, bookmarkWindow);
 
     windowList = g_list_append(windowList, window);
-    
     return window;
 }
 
-static error_t
-parseAnArg (int key, char *arg, struct argp_state *state)
-{
-	gint val;
-	
-	if (key == 'x' || key == 'y' || key == 'w' || key == 'h') {
-		val = strtol(arg, NULL, 10);
-
-		switch (key) {
-		case 'x':
-			defposx = val;
-			break;
-		case 'y':
-			defposy = val;
-			break;
-		case 'w':
-			defwidth  = val;
-			break;
-		case 'h':
-			defheight = val;
-			break;
-		default:
-			break;
-		}
-		return 0;
-	}
-
-	if (key == 'd') {
-	        debugLevel++;
-		return 0;
-	}
-
-  if (key != ARGP_KEY_ARG)
-    return ARGP_ERR_UNKNOWN;
-  if (helpURL)
-    argp_usage (state);
-
-  /* this probably somehow leads to a memory leak the size of the URL */
-  if (arg && *arg == '/') {
-	  helpURL = g_malloc(strlen(arg)+10);
-	  strcpy(helpURL, "file:");
-	  strcat(helpURL, arg);
-  } else {
-	  helpURL = g_strdup(arg);
-  }
-  
-
-  return 0;
-}
-
 /********************************************************************/
 
 /* Callbacks */
@@ -286,20 +293,20 @@ parseAnArg (int key, char *arg, struct argp_state *state)
 static void
 setCurrentCallback(HelpWindow win)
 {
-    windowList = g_list_remove(windowList, win);
-    windowList = g_list_append(windowList, win);
+  windowList = g_list_remove(windowList, win);
+  windowList = g_list_append(windowList, win);
 }
 
 static void
 closeWindowCallback(HelpWindow win)
 {
-    helpWindowClose(win);
-
-    windowList = g_list_remove(windowList, win);
-
-    if (!windowList) {
-	gtk_main_quit();
-    }
+  destroy_server(win);
+  helpWindowClose(win);
+  windowList = g_list_remove(windowList, win);
+  
+  if (!windowList) {
+    gtk_main_quit();
+  }
 }
 
 static void
@@ -309,6 +316,7 @@ newWindowCallback(HelpWindow win)
     
     window = makeHelpWindow(0,0,0,0);
     helpWindowShowURL(window, "toc:", TRUE, TRUE);
+    gtk_widget_show (helpWindowGetAppWindow (window));
 }
 
 static void
@@ -331,7 +339,7 @@ static void
 aboutCallback (HelpWindow win)
 {
 	GtkWidget *about;
-	gchar *authors[] = {
+	const gchar *authors[] = {
 		"Mike Fulbright",
 		"Marc Ewing",
 		NULL
@@ -641,7 +649,7 @@ generateConfigWidgets(GnomePropertyBox *property, struct _config_entry *configs,
     }
 
     table = gtk_table_new(0, 0, FALSE);
-    gtk_container_border_width (GTK_CONTAINER (table), GNOME_PAD);
+    gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD);
     gnome_property_box_append_page (property, table, gtk_label_new (title));
 
     rows = 0;
