@@ -253,7 +253,7 @@ static int hideHistoryInt(GtkWidget *window)
 static void createHistoryWindow(History h, GtkWidget **window,
 				GtkWidget **clist)
 {
-    GtkWidget *box;
+    GtkWidget *box, *sw;
     gchar *titles[3] = { N_("URL"), N_("Last"), N_("Count") };
     static int translated;
 
@@ -273,15 +273,17 @@ static void createHistoryWindow(History h, GtkWidget **window,
     /* Vbox */
     /* Don't need this now but might be used later.  I'll leave it */
     box = gtk_vbox_new(FALSE, 5);
-    gtk_container_border_width (GTK_CONTAINER (box), 5);
+    gtk_container_set_border_width (GTK_CONTAINER (box), 5);
     gtk_container_add(GTK_CONTAINER(*window), box);
     gtk_widget_show(box);
 
     /* The clist */
     *clist = gtk_clist_new_with_titles(3, titles);
     gtk_clist_set_selection_mode(GTK_CLIST(*clist), GTK_SELECTION_SINGLE);
+#ifndef GTK_HAVE_FEATURES_1_1_4
     gtk_clist_set_policy(GTK_CLIST(*clist),
 			 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+#endif
     gtk_clist_column_titles_show(GTK_CLIST(*clist));
     gtk_clist_column_titles_passive(GTK_CLIST(*clist));
     gtk_clist_set_column_justification(GTK_CLIST(*clist), 0,
@@ -294,7 +296,16 @@ static void createHistoryWindow(History h, GtkWidget **window,
     gtk_clist_set_column_width(GTK_CLIST(*clist), 1, 120);
     gtk_clist_set_column_width(GTK_CLIST(*clist), 2, 50);
 
+#ifdef GTK_HAVE_FEATURES_1_1_4
+    sw = gtk_scrolled_window_new (NULL, NULL);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
+				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+    gtk_container_add(GTK_CONTAINER(sw), *clist);
+    gtk_box_pack_start(GTK_BOX(box), sw, TRUE, TRUE, 0);
+    gtk_widget_show(sw);
+#else
     gtk_box_pack_start(GTK_BOX(box), *clist, TRUE, TRUE, 0);
+#endif
     gtk_widget_show(*clist);
 
     /* Set callbacks */
