@@ -16,6 +16,10 @@ static void button_widget_size_request  (GtkWidget         *widget,
 static void button_widget_size_allocate (GtkWidget         *widget,
 					 GtkAllocation     *allocation);
 static void button_widget_realize	(GtkWidget         *widget);
+static void button_widget_unrealize     (GtkWidget         *widget);
+static void button_widget_map           (GtkWidget         *widget);
+static void button_widget_unmap         (GtkWidget         *widget);
+
 static int  button_widget_button_press	(GtkWidget         *widget,
 					 GdkEventButton    *event);
 static int  button_widget_button_release(GtkWidget         *widget,
@@ -34,15 +38,15 @@ typedef void (*VoidSignal) (GtkObject * object,
 static GList *buttons=NULL;
 
 /*the tiles go here*/
-static GdkPixmap *tiles_up[MAX_TILES]={NULL,NULL,NULL,NULL};
-static GdkBitmap *tiles_up_mask[MAX_TILES]={NULL,NULL,NULL,NULL};
-static GdkPixmap *tiles_down[MAX_TILES]={NULL,NULL,NULL,NULL};
-static GdkBitmap *tiles_down_mask[MAX_TILES]={NULL,NULL,NULL,NULL};
-static int tile_border[MAX_TILES]={0,0,0,0};
-static int tile_depth[MAX_TILES]={0,0,0,0};
+static GdkPixmap *tiles_up[LAST_TILE]={NULL,NULL,NULL};
+static GdkBitmap *tiles_up_mask[LAST_TILE]={NULL,NULL,NULL};
+static GdkPixmap *tiles_down[LAST_TILE]={NULL,NULL,NULL};
+static GdkBitmap *tiles_down_mask[LAST_TILE]={NULL,NULL,NULL};
+static int tile_border[LAST_TILE]={0,0,0};
+static int tile_depth[LAST_TILE]={0,0,0};
 
 /*are tiles enabled*/
-static int tiles_enabled = FALSE;
+static int tiles_enabled[LAST_TILE]={FALSE,FALSE,FALSE};
 
 static GtkWidgetClass *parent_class;
 
@@ -135,6 +139,9 @@ button_widget_class_init (ButtonWidgetClass *class)
 	class->unpressed = button_widget_unpressed;
 
 	widget_class->realize = button_widget_realize;
+	widget_class->unrealize = button_widget_unrealize;
+	widget_class->map = button_widget_map;
+	widget_class->unmap = button_widget_unmap;
 	widget_class->size_allocate = button_widget_size_allocate;
 	widget_class->size_request = button_widget_size_request;
 	widget_class->button_press_event = button_widget_button_press;
@@ -149,10 +156,13 @@ button_widget_realize(GtkWidget *widget)
 {
 	GdkWindowAttr attributes;
 	gint attributes_mask;
+	ButtonWidget *button_widget;
 
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (IS_BUTTON_WIDGET (widget));
 
+	button_widget = BUTTON_WIDGET (widget);
+
 	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
 	attributes.window_type = GDK_WINDOW_CHILD;
@@ -170,11 +180,57 @@ button_widget_realize(GtkWidget *widget)
 				 GDK_LEAVE_NOTIFY_MASK);
 	attributes_mask = GDK_WA_X | GDK_WA_Y;
 
-	widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
-					 &attributes, attributes_mask);
-	gdk_window_set_user_data (widget->window, widget);
+	widget->window = gtk_widget_get_parent_window(widget);
+	gdk_window_ref(widget->window);
+      
+	button_widget->event_window = gdk_window_new (gtk_widget_get_parent_window (widget),
+						      &attributes, attributes_mask);
+	gdk_window_set_user_data (button_widget->event_window, widget);
+
+	widget->style = gtk_style_attach (widget->style, widget->window);
 }
 
+static void
+button_widget_unrealize (GtkWidget *widget)
+{
+	ButtonWidget *button_widget;
+  
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (IS_BUTTON_WIDGET (widget));
+
+	button_widget = BUTTON_WIDGET (widget);
+	
+	gdk_window_set_user_data (button_widget->event_window, NULL);
+	gdk_window_destroy (button_widget->event_window);
+	button_widget->event_window = NULL;
+	
+	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
+		(* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
+}
+
+static void
+button_widget_map (GtkWidget *widget)
+{
+  g_return_if_fail (widget != NULL);
+  g_return_if_fail (IS_BUTTON_WIDGET (widget));
+
+  if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_MAPPED (widget))
+      gdk_window_show (BUTTON_WIDGET (widget)->event_window);
+
+  GTK_WIDGET_CLASS (parent_class)->map (widget);
+}
+
+static void
+button_widget_unmap (GtkWidget *widget)
+{
+  g_return_if_fail (widget != NULL);
+  g_return_if_fail (IS_BUTTON_WIDGET (widget));
+
+  if (GTK_WIDGET_MAPPED (widget))
+      gdk_window_hide (BUTTON_WIDGET (widget)->event_window);
+
+  GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+}
 
 static int
 button_widget_destroy(GtkWidget *w, gpointer data)
@@ -244,7 +300,7 @@ button_widget_draw(ButtonWidget *button, GdkPixmap *pixmap)
 	/*offset for pressed buttons*/
 	int off = button->in_button&&button->pressed?tile_depth[button->tile]:0;
 	/*border to not draw when drawing a tile*/
-	int border = tiles_enabled?tile_border[button->tile]:0;
+	int border = tiles_enabled[button->tile]?tile_border[button->tile]:0;
 	int i;
 	 
 	if(!GTK_WIDGET_REALIZED(button))
@@ -263,7 +319,7 @@ button_widget_draw(ButtonWidget *button, GdkPixmap *pixmap)
 		}
 	}
 	
-	if(tiles_enabled) {
+	if(tiles_enabled[button->tile]) {
 		if(button->pressed && button->in_button) {
 			tile = tiles_down[button->tile];
 			tile_mask = tiles_down_mask[button->tile];
@@ -273,7 +329,7 @@ button_widget_draw(ButtonWidget *button, GdkPixmap *pixmap)
 		}
 	}
 
-	if(tiles_enabled) {
+	if(tiles_enabled[button->tile]) {
 		if (tile_mask) {
 			gdk_gc_set_clip_mask (gc, tile_mask);
 			gdk_gc_set_clip_origin (gc, widget->allocation.x,
@@ -327,9 +383,16 @@ button_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 static void
 button_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
+	ButtonWidget *button_widget;
+
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (IS_BUTTON_WIDGET (widget));
+
+	button_widget = BUTTON_WIDGET (widget);
+
 	widget->allocation = *allocation;
 	if (GTK_WIDGET_REALIZED (widget))
-		gdk_window_move_resize (widget->window,
+		gdk_window_move_resize (button_widget->event_window,
 					allocation->x, allocation->y,
 					allocation->width, allocation->height);
 }
@@ -341,6 +404,8 @@ button_widget_init (ButtonWidget *button)
 {
 	buttons = g_list_prepend(buttons,button);
 
+	GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
+
 	button->pixmap = NULL;
 	button->mask = NULL;
 	
@@ -618,11 +683,11 @@ button_widget_load_tile(int tile, char *tile_up, char *tile_down,
 }
 
 void
-button_widget_tile_enable(int enabled)
+button_widget_tile_enable(int i, int enabled)
 {
-	if(tiles_enabled != enabled) {
+	if(tiles_enabled[i] != enabled) {
 		GList *list;
-		tiles_enabled = enabled;
+		tiles_enabled[i] = enabled;
 
 		for(list = buttons;list!=NULL;list=g_list_next(list))
 			panel_widget_draw_icon(PANEL_WIDGET(GTK_WIDGET(list->data)->parent),
