@@ -21,6 +21,8 @@ static void corner_widget_size_request	(GtkWidget         *widget,
 static void corner_widget_size_allocate	(GtkWidget         *widget,
 					 GtkAllocation     *allocation);
 
+static GtkWindowClass *parent_class = NULL;
+
 extern GdkCursor *fleur_cursor;
 
 /*global settings*/
@@ -87,13 +89,85 @@ marshal_signal_int (GtkObject * object,
 		  func_data);
 }
 
+static void
+set_frame_colors(CornerWidget *corner)
+{
+	if(PANEL_WIDGET(corner->panel)->back_type == PANEL_BACK_COLOR) {
+		GdkColor n;
+		GtkStyle *ns;
+
+		ns = gtk_style_copy(corner->panel->style);
+		gtk_style_ref(ns);
+
+		ns->bg[GTK_STATE_NORMAL] =
+			PANEL_WIDGET(corner->panel)->back_color;
+		ns->base[GTK_STATE_NORMAL] =
+			PANEL_WIDGET(corner->panel)->back_color;
+
+		n = PANEL_WIDGET(corner->panel)->back_color;
+		n.red/=3;
+		n.green/=3;
+		n.blue/=3;
+		ns->dark[GTK_STATE_NORMAL] = n;
+
+		n = PANEL_WIDGET(corner->panel)->back_color;
+		n.red/=2;
+		n.green/=2;
+		n.blue/=2;
+		ns->mid[GTK_STATE_NORMAL] = n;
+
+		n = PANEL_WIDGET(corner->panel)->back_color;
+		n.red=MIN(65535,n.red*2);
+		n.green=MIN(65535,n.green*2);
+		n.blue=MIN(65535,n.blue*2);
+		ns->light[GTK_STATE_NORMAL] = n;
+
+		gtk_widget_set_style(corner->frame, ns);
+
+		gtk_style_unref(ns);
+	} else {
+		GtkStyle *ns;
+
+		ns = gtk_rc_get_style(corner->frame);
+		if(!ns) ns = gtk_style_new();
+
+		gtk_style_ref(ns);
+		gtk_widget_set_style(corner->frame, ns);
+		gtk_style_unref(ns);
+	}
+}
+
+static void
+corner_widget_realize(GtkWidget *w)
+{
+	CornerWidget *corner = CORNER_WIDGET(w);
+	GTK_WIDGET_CLASS(parent_class)->realize(w);
+
+	gnome_win_hints_init();
+	if (gnome_win_hints_wm_exists()) {
+		gnome_win_hints_set_hints(w,
+					  WIN_HINTS_SKIP_FOCUS |
+					  WIN_HINTS_SKIP_WINLIST |
+					  WIN_HINTS_SKIP_TASKBAR);
+		gnome_win_hints_set_state(w,
+					  WIN_STATE_STICKY |
+					  WIN_STATE_FIXED_POSITION);
+		gnome_win_hints_set_layer(w, WIN_LAYER_DOCK);
+		gnome_win_hints_set_expanded_size(w, 0, 0, 0, 0);
+		gdk_window_set_decorations(w->window, 0);
+	}
+	
+	set_frame_colors(corner);
+}
+
 static void
 corner_widget_class_init (CornerWidgetClass *class)
 {
 	GtkObjectClass *object_class = (GtkObjectClass*) class;
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) class;
 
-	corner_widget_signals[POS_CHANGE_SIGNAL] =
+        parent_class = gtk_type_class (gtk_window_get_type ());
+        corner_widget_signals[POS_CHANGE_SIGNAL] =
 		gtk_signal_new("pos_change",
 			       GTK_RUN_LAST,
 			       object_class->type,
@@ -122,9 +196,9 @@ corner_widget_class_init (CornerWidgetClass *class)
 	
 	widget_class->size_request = corner_widget_size_request;
 	widget_class->size_allocate = corner_widget_size_allocate;
+	widget_class->realize = corner_widget_realize;
 }
 
-
 /*if this is true the size request will request a 48x48 cube, this is used
   during orientation changes to make no flicker*/
 static int corner_widget_request_cube = FALSE;
@@ -527,19 +601,32 @@ corner_enter_notify(CornerWidget *corner,
 		     GdkEventCrossing *event,
 		     gpointer data)
 {
-	/*FIXME: do we want this autoraise piece?*/
-	gdk_window_raise(GTK_WIDGET(corner)->window);
+  if (!gnome_win_hints_wm_exists())
+    gdk_window_raise(GTK_WIDGET(corner)->window);
 }
 
 static void
 corner_widget_set_hidebuttons(CornerWidget *corner)
 {
-	if(PANEL_WIDGET(corner->panel)->orient == PANEL_HORIZONTAL) {
+	/*hidebuttons are disabled*/
+	if(!corner->hidebuttons_enabled) {
+		gtk_widget_hide(corner->hidebutton_n);
+		gtk_widget_hide(corner->hidebutton_e);
+		gtk_widget_hide(corner->hidebutton_w);
+		gtk_widget_hide(corner->hidebutton_s);
+		/*in case the panel was hidden, show it, since otherwise
+		  we wouldn't see it anymore*/
+		if(is_right(corner))
+			corner_widget_pop_show(corner,FALSE);
+		else
+			corner_widget_pop_show(corner,TRUE);
+	/* horizontal and enabled */
+	} else if(PANEL_WIDGET(corner->panel)->orient == PANEL_HORIZONTAL) {
 		gtk_widget_hide(corner->hidebutton_n);
 		gtk_widget_show(corner->hidebutton_e);
 		gtk_widget_show(corner->hidebutton_w);
 		gtk_widget_hide(corner->hidebutton_s);
-	} else {
+	} else { /*vertical*/
 		gtk_widget_show(corner->hidebutton_n);
 		gtk_widget_hide(corner->hidebutton_e);
 		gtk_widget_hide(corner->hidebutton_w);
@@ -570,6 +657,7 @@ make_hidebutton(CornerWidget *corner,
 	gtk_widget_show(pixmap);
 
 	gtk_container_add(GTK_CONTAINER(w),pixmap);
+	gtk_object_set_user_data(GTK_OBJECT(w), pixmap);
 	gtk_signal_connect(GTK_OBJECT(w), "clicked",
 			   GTK_SIGNAL_FUNC(hidefunc),
 			   corner);
@@ -579,8 +667,12 @@ make_hidebutton(CornerWidget *corner,
 static void
 corner_widget_init (CornerWidget *corner)
 {
-	/*if we set the icewm hints it will have to be changed to TOPLEVEL*/
-	GTK_WINDOW(corner)->type = GTK_WINDOW_POPUP;
+	/*if we set the gnomewm hints it will have to be changed to TOPLEVEL*/
+	gnome_win_hints_init();
+	if (gnome_win_hints_wm_exists())
+		GTK_WINDOW(corner)->type = GTK_WINDOW_TOPLEVEL;
+	else
+		GTK_WINDOW(corner)->type = GTK_WINDOW_POPUP;
 	GTK_WINDOW(corner)->allow_shrink = TRUE;
 	GTK_WINDOW(corner)->allow_grow = TRUE;
 	GTK_WINDOW(corner)->auto_shrink = TRUE;
@@ -633,21 +725,58 @@ corner_widget_init (CornerWidget *corner)
 			   NULL);
 	corner->pos = CORNER_NE;
 	corner->state = CORNER_SHOWN;
+
+	corner->hidebuttons_enabled = TRUE;
+	corner->hidebutton_pixmaps_enabled = TRUE;
 }
 
+static void
+show_hidebutton_pixmap(GtkWidget *hidebutton, int show)
+{
+	GtkWidget *pixmap;
 
+	pixmap = gtk_object_get_user_data(GTK_OBJECT(hidebutton));
+
+	if (!pixmap) return;
+
+	if (show)
+		gtk_widget_show(pixmap);
+	else
+		gtk_widget_hide(pixmap);
+}
+
+static void
+corner_widget_show_hidebutton_pixmaps(CornerWidget *corner)
+{
+	int show = corner->hidebutton_pixmaps_enabled;
+	show_hidebutton_pixmap(corner->hidebutton_n, show);
+	show_hidebutton_pixmap(corner->hidebutton_e, show);
+	show_hidebutton_pixmap(corner->hidebutton_w, show);
+	show_hidebutton_pixmap(corner->hidebutton_s, show);
+}
+
+static void
+c_back_change(PanelWidget *panel,
+	    PanelBackType type,
+	    char *pixmap,
+	    GdkColor *color,
+	    CornerWidget *corner)
+{
+	set_frame_colors(corner);
+}
 
 GtkWidget*
 corner_widget_new (CornerPos pos,
 		   PanelOrientation orient,
 		   CornerState state,
+		   int hidebuttons_enabled,
+		   int hidebutton_pixmaps_enabled,
 		   PanelBackType back_type,
 		   char *back_pixmap,
 		   int fit_pixmap_bg,
 		   GdkColor *back_color)
 {
 	CornerWidget *corner;
-	GtkWidget *frame;
 
 	corner = gtk_type_new(corner_widget_get_type());
 
@@ -657,28 +786,34 @@ corner_widget_new (CornerPos pos,
 					 back_pixmap,
 					 fit_pixmap_bg,
 					 back_color);
+	gtk_signal_connect_after(GTK_OBJECT(corner->panel), "back_change",
+				 GTK_SIGNAL_FUNC(c_back_change),
+				 corner);
 	gtk_object_set_data(GTK_OBJECT(corner->panel),PANEL_PARENT,
 			    corner);
 	PANEL_WIDGET(corner->panel)->drop_widget = GTK_WIDGET(corner);
 
 	gtk_widget_show(corner->panel);
 
-	frame = gtk_frame_new(NULL);
-	gtk_widget_show(frame);
-	gtk_frame_set_shadow_type(GTK_FRAME(frame),GTK_SHADOW_OUT);
-	gtk_container_add(GTK_CONTAINER(frame),corner->panel);
+	corner->frame = gtk_frame_new(NULL);
+	gtk_widget_show(corner->frame);
+	gtk_frame_set_shadow_type(GTK_FRAME(corner->frame),GTK_SHADOW_OUT);
+	gtk_container_add(GTK_CONTAINER(corner->frame),corner->panel);
 
-	gtk_table_attach(GTK_TABLE(corner->table),frame,1,2,1,2,
+	gtk_table_attach(GTK_TABLE(corner->table),corner->frame,1,2,1,2,
 			 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
 			 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
 			 0,0);
 
+	corner->hidebuttons_enabled = hidebuttons_enabled;
+	corner->hidebutton_pixmaps_enabled = hidebutton_pixmaps_enabled;
+
 	corner->pos = pos;
 	if(state != CORNER_MOVING)
 		corner->state = state;
 
 	corner_widget_set_hidebuttons(corner);
-
+/* */	corner_widget_show_hidebutton_pixmaps(corner);
 	corner_widget_set_initial_pos(corner);
 
 	return GTK_WIDGET(corner);
@@ -689,6 +824,8 @@ corner_widget_change_params(CornerWidget *corner,
 			    CornerPos pos,
 			    PanelOrientation orient,
 			    CornerState state,
+			    int hidebuttons_enabled,
+			    int hidebutton_pixmaps_enabled,
 			    PanelBackType back_type,
 			    char *pixmap_name,
 			    int fit_pixmap_bg,
@@ -704,6 +841,8 @@ corner_widget_change_params(CornerWidget *corner,
 	corner->pos = pos;
 	oldstate = corner->state;
 	corner->state = state;
+	corner->hidebuttons_enabled = hidebuttons_enabled;
+	corner->hidebutton_pixmaps_enabled = hidebutton_pixmaps_enabled;
 
 	/*avoid flicker during size_request*/
 	if(PANEL_WIDGET(corner->panel)->orient != orient)
@@ -717,6 +856,7 @@ corner_widget_change_params(CornerWidget *corner,
 				   back_color);
 
 	corner_widget_set_hidebuttons(corner);
+/* */	corner_widget_show_hidebutton_pixmaps(corner);
 
 	if(oldpos != corner->pos)
 	   	gtk_signal_emit(GTK_OBJECT(corner),
@@ -741,6 +881,8 @@ corner_widget_change_pos_orient(CornerWidget *corner,
 				    pos,
 				    orient,
 				    corner->state,
+				    corner->hidebuttons_enabled,
+				    corner->hidebutton_pixmaps_enabled,
 				    panel->back_type,
 				    panel->back_pixmap,
 				    panel->fit_pixmap_bg,
