@@ -11,6 +11,7 @@
 #include "drawer-widget.h"
 #include "panel-util.h"
 #include "gdkextra.h"
+#include "panel-include.h" 
 
 /*there  can universally be only one applet being dragged since we assume
 we only have one mouse :) */
@@ -23,6 +24,8 @@ static void drawer_widget_size_request	(GtkWidget          *widget,
 static void drawer_widget_size_allocate	(GtkWidget          *widget,
 					 GtkAllocation      *allocation);
 
+static GtkWindowClass *parent_class = NULL;
+
 extern GdkCursor *fleur_cursor;
 
 /*global settings*/
@@ -86,13 +89,85 @@ marshal_signal_state (GtkObject * object,
 		  func_data);
 }
 
+static void
+set_frame_colors(DrawerWidget *drawer)
+{
+	if(PANEL_WIDGET(drawer->panel)->back_type == PANEL_BACK_COLOR) {
+		GdkColor n;
+		GtkStyle *ns;
+
+		ns = gtk_style_copy(drawer->panel->style);
+		gtk_style_ref(ns);
+
+		ns->bg[GTK_STATE_NORMAL] =
+			PANEL_WIDGET(drawer->panel)->back_color;
+		ns->base[GTK_STATE_NORMAL] =
+			PANEL_WIDGET(drawer->panel)->back_color;
+
+		n = PANEL_WIDGET(drawer->panel)->back_color;
+		n.red/=3;
+		n.green/=3;
+		n.blue/=3;
+		ns->dark[GTK_STATE_NORMAL] = n;
+
+		n = PANEL_WIDGET(drawer->panel)->back_color;
+		n.red/=2;
+		n.green/=2;
+		n.blue/=2;
+		ns->mid[GTK_STATE_NORMAL] = n;
+
+		n = PANEL_WIDGET(drawer->panel)->back_color;
+		n.red=MIN(65535,n.red*2);
+		n.green=MIN(65535,n.green*2);
+		n.blue=MIN(65535,n.blue*2);
+		ns->light[GTK_STATE_NORMAL] = n;
+
+		gtk_widget_set_style(drawer->frame, ns);
+
+		gtk_style_unref(ns);
+	} else {
+		GtkStyle *ns;
+
+		ns = gtk_rc_get_style(drawer->frame);
+		if(!ns) ns = gtk_style_new();
+
+		gtk_style_ref(ns);
+		gtk_widget_set_style(drawer->frame, ns);
+		gtk_style_unref(ns);
+	}
+}
+
+
+static void
+drawer_widget_realize(GtkWidget *w)
+{
+	DrawerWidget *drawer = DRAWER_WIDGET(w);
+	GTK_WIDGET_CLASS(parent_class)->realize(w);
+
+	gnome_win_hints_init();
+	if (gnome_win_hints_wm_exists()) {
+		gnome_win_hints_set_hints(w,
+					  WIN_HINTS_SKIP_FOCUS |
+					  WIN_HINTS_SKIP_WINLIST |
+					  WIN_HINTS_SKIP_TASKBAR);
+		gnome_win_hints_set_state(w,
+					  WIN_STATE_STICKY |
+					  WIN_STATE_FIXED_POSITION);
+		gnome_win_hints_set_layer(w, WIN_LAYER_DOCK);
+		gnome_win_hints_set_expanded_size(w, 0, 0, 0, 0);
+		gdk_window_set_decorations(w->window, 0);
+	}
+	set_frame_colors(drawer);
+}
+
 static void
 drawer_widget_class_init (DrawerWidgetClass *class)
 {
 	GtkObjectClass *object_class = (GtkObjectClass*) class;
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) class;
 
-	drawer_widget_signals[STATE_CHANGE_SIGNAL] =
+        parent_class = gtk_type_class (gtk_window_get_type ());
+        drawer_widget_signals[STATE_CHANGE_SIGNAL] =
 		gtk_signal_new("state_change",
 			       GTK_RUN_LAST,
 			       object_class->type,
@@ -110,6 +185,7 @@ drawer_widget_class_init (DrawerWidgetClass *class)
 	
 	widget_class->size_request = drawer_widget_size_request;
 	widget_class->size_allocate = drawer_widget_size_allocate;
+	widget_class->realize = drawer_widget_realize;
 }
 
 /*if this is set, the size request and size alloc calls are ignored*/
@@ -151,7 +227,7 @@ drawer_widget_get_pos(DrawerWidget *drawer, gint16 *x, gint16 *y,
 		GtkWidget *ppanel; /*parent panel*/
 
 		/*get the parent of the applet*/
-		ppanel = PANEL_WIDGET(panel->master_widget->parent);
+		ppanel = panel->master_widget->parent;
 		if (GTK_WIDGET_REALIZED (ppanel)) {
 			gdk_window_get_origin (panel->master_widget->window, &bx, &by);
 			if(GTK_WIDGET_NO_WINDOW(panel->master_widget)) {
@@ -163,21 +239,24 @@ drawer_widget_get_pos(DrawerWidget *drawer, gint16 *x, gint16 *y,
 			gdk_window_get_origin (ppanel->window, &px, &py);
 			gdk_window_get_size (ppanel->window, &pw, &ph);
 
+			/*FIXME: THE +/- 2 is a HACK!, the border of a frame
+			  could be different ... we'd better figure out the
+			  GoodSolution(tm) for this*/
 			switch(drawer->orient) {
 			case ORIENT_UP:
 				*x = bx+(bw-width)/2;
-				*y = py - height;
+				*y = py - height - 2;
 				break;
 			case ORIENT_DOWN:
 				*x = bx+(bw-width)/2;
-				*y = py + ph;
+				*y = py + ph + 2;
 				break;
 			case ORIENT_LEFT:
-				*x = px - width;
+				*x = px - width - 2;
 				*y = by+(bh-height)/2;
 				break;
 			case ORIENT_RIGHT:
-				*x = px + pw;
+				*x = px + pw + 2;
 				*y = by+(bh-height)/2;
 				break;
 			}
@@ -486,9 +565,8 @@ drawer_widget_close_drawer(DrawerWidget *drawer)
 static int
 drawer_enter_notify(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
 {
-	/*FIXME: do we want this autoraise piece?*/
-	gdk_window_raise(widget->window);
-
+	if (!gnome_win_hints_wm_exists())
+		gdk_window_raise(widget->window);
 	return FALSE;
 }
 
@@ -497,33 +575,97 @@ static GtkWidget *
 make_handle(char *pixmaphandle, int wi, int he)
 {
 	GtkWidget *w;
+	GtkWidget *box;
 	GtkWidget *pixmap;
 	char *pixmap_name;
 
-	GtkWidget *frame = gtk_frame_new(NULL);
-	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
-	gtk_widget_show(frame);
 	pixmap_name=gnome_unconditional_pixmap_file(pixmaphandle);
 	pixmap = gnome_pixmap_new_from_file(pixmap_name);
 	gtk_widget_show(pixmap);
 	g_free(pixmap_name);
-	gtk_container_add(GTK_CONTAINER(frame),pixmap);
-	w = gtk_event_box_new();
-	gtk_widget_show(w);
-	gtk_container_add(GTK_CONTAINER(w),frame);
-	gtk_widget_set_usize(w,wi,he);
 
-	return w;
+	w = gtk_button_new();
+	GTK_WIDGET_UNSET_FLAGS(w,GTK_CAN_DEFAULT|GTK_CAN_FOCUS);
+	gtk_widget_show(w);
+
+	box = gtk_hbox_new(1,1);
+	gtk_widget_show(box);
+	
+	gtk_container_add(GTK_CONTAINER(w), pixmap);
+       	gtk_container_add(GTK_CONTAINER(box), w);
+	
+	gtk_widget_set_usize(box,wi,he);
+
+	gtk_object_set_data(GTK_OBJECT(box), "button", w);
+	gtk_object_set_data(GTK_OBJECT(box), "pixmap", pixmap);
+	return box;
 }
 
+
+static void
+show_handle_pixmap(GtkWidget *handle, int show_pixmap, int show_button)
+{
+	GtkWidget *pixmap, *button;
+
+	button = gtk_object_get_data(GTK_OBJECT(handle), "button");
+	pixmap = gtk_object_get_data(GTK_OBJECT(handle), "pixmap");
+
+	if (!button) return;	
+	if (show_button)
+	        gtk_widget_show(button);
+	else 
+	        gtk_widget_hide(button);
+
+	if (!pixmap) return;
+	if (show_pixmap)
+		gtk_widget_show(pixmap);
+	else
+		gtk_widget_hide(pixmap);
+}
+
+static void
+drawer_hidebutton_set(DrawerWidget *drawer)
+{
+  int pixmap_enabled = drawer->hidebutton_pixmap_enabled;
+  int button_enabled = drawer->hidebutton_enabled;
+  show_handle_pixmap(drawer->handle_n, pixmap_enabled,
+		     button_enabled);
+  show_handle_pixmap(drawer->handle_s, pixmap_enabled,
+		     button_enabled); 
+  show_handle_pixmap(drawer->handle_w, pixmap_enabled,
+		     button_enabled); 
+  show_handle_pixmap(drawer->handle_e, pixmap_enabled,
+		     button_enabled);
+}
+
+
+static void
+drawer_handle_click(GtkWidget *widget, gpointer data)
+{
+	Drawer *drawer = gtk_object_get_data(GTK_OBJECT(data),
+					     DRAWER_PANEL_KEY);
+	DrawerWidget *drawerw = DRAWER_WIDGET(drawer->drawer);
+	PanelWidget *parent = PANEL_WIDGET(drawer->button->parent);
+	GtkWidget *panelw = gtk_object_get_data(GTK_OBJECT(parent),
+						PANEL_PARENT);
+	drawer_widget_close_drawer(data);
+	if(IS_SNAPPED_WIDGET(panelw))
+		SNAPPED_WIDGET(panelw)->drawers_open--;
+}
+
+
 static void
 drawer_widget_init (DrawerWidget *drawer)
 {
-	GTK_WINDOW(drawer)->type = GTK_WINDOW_POPUP;
+	gnome_win_hints_init();
+	if (gnome_win_hints_wm_exists())
+		GTK_WINDOW(drawer)->type = GTK_WINDOW_TOPLEVEL;
+	else
+		GTK_WINDOW(drawer)->type = GTK_WINDOW_POPUP;
 	GTK_WINDOW(drawer)->allow_shrink = TRUE;
 	GTK_WINDOW(drawer)->allow_grow = TRUE;
 	GTK_WINDOW(drawer)->auto_shrink = TRUE;
-
+	
 	/*this makes the popup "pop down" once the button is released*/
 	gtk_widget_set_events(GTK_WIDGET(drawer),
 			      gtk_widget_get_events(GTK_WIDGET(drawer)) |
@@ -535,37 +677,63 @@ drawer_widget_init (DrawerWidget *drawer)
 
 	/*we add all the handles to the table here*/
 	/*EAST*/
-	drawer->handle_e = make_handle("panel-knob.png",0,40);
+	drawer->handle_e = make_handle("panel-arrow-right.png",0,40);
 	gtk_table_attach(GTK_TABLE(drawer->table),drawer->handle_e,
 			 0,1,1,2,GTK_FILL,GTK_FILL,0,0);
+	gtk_signal_connect(gtk_object_get_data(GTK_OBJECT(drawer->handle_e),
+					       "button"), "clicked",
+			   GTK_SIGNAL_FUNC(drawer_handle_click), drawer);
 	/*NORTH*/
-	drawer->handle_n = make_handle("panel-knob.png",40,0);
+	drawer->handle_n = make_handle("panel-arrow-down.png",40,0);
 	gtk_table_attach(GTK_TABLE(drawer->table),drawer->handle_n,
 			 1,2,0,1,GTK_FILL,GTK_FILL,0,0);
+	gtk_signal_connect(gtk_object_get_data(GTK_OBJECT(drawer->handle_n),
+					       "button"), "clicked",
+			   GTK_SIGNAL_FUNC(drawer_handle_click), drawer);
 	/*WEST*/
-	drawer->handle_w = make_handle( "panel-knob.png",0,40);
+	drawer->handle_w = make_handle( "panel-arrow-left.png",0,40);
 	gtk_table_attach(GTK_TABLE(drawer->table),drawer->handle_w,
 			 2,3,1,2,GTK_FILL,GTK_FILL,0,0);
+	gtk_signal_connect(gtk_object_get_data(GTK_OBJECT(drawer->handle_w),
+					       "button"), "clicked",
+			   GTK_SIGNAL_FUNC(drawer_handle_click), drawer);
 	/*SOUTH*/
-	drawer->handle_s = make_handle("panel-knob.png",40,0);
+	drawer->handle_s = make_handle("panel-arrow-up.png",40,0);
 	gtk_table_attach(GTK_TABLE(drawer->table),drawer->handle_s,
 			 1,2,2,3,GTK_FILL,GTK_FILL,0,0);
+	gtk_signal_connect(gtk_object_get_data(GTK_OBJECT(drawer->handle_s),
+					       "button"), "clicked",
+			   GTK_SIGNAL_FUNC(drawer_handle_click), drawer);
 
 	gtk_signal_connect(GTK_OBJECT(drawer), "enter_notify_event",
 			   GTK_SIGNAL_FUNC(drawer_enter_notify),
 			   NULL);
-
+	drawer->hidebutton_enabled = TRUE;
+	drawer->hidebutton_pixmap_enabled = TRUE;
 	drawer->state = DRAWER_SHOWN;
 }
 
+static void
+d_back_change(PanelWidget *panel,
+	    PanelBackType type,
+	    char *pixmap,
+	    GdkColor *color,
+	    DrawerWidget *drawer)
+{
+	set_frame_colors(drawer);
+}
 
+
+ 
 GtkWidget*
 drawer_widget_new (PanelOrientType orient,
 		   DrawerState state,
 		   PanelBackType back_type,
 		   char *back_pixmap,
 		   int fit_pixmap_bg,
-		   GdkColor *back_color)
+		   GdkColor *back_color,
+		   int hidebutton_pixmap_enabled,
+		   int hidebutton_enabled)
 {
 	DrawerWidget *drawer;
 	GtkWidget *frame;
@@ -588,24 +756,29 @@ drawer_widget_new (PanelOrientType orient,
 					 back_pixmap,
 					 fit_pixmap_bg,
 					 back_color);
+	gtk_signal_connect_after(GTK_OBJECT(drawer->panel), "back_change",
+				 GTK_SIGNAL_FUNC(d_back_change),
+				 drawer);
 	gtk_object_set_data(GTK_OBJECT(drawer->panel),PANEL_PARENT,
 			    drawer);
 	PANEL_WIDGET(drawer->panel)->drop_widget = GTK_WIDGET(drawer);
 
 	gtk_widget_show(drawer->panel);
 
-	frame = gtk_frame_new(NULL);
-	gtk_widget_show(frame);
-	gtk_frame_set_shadow_type(GTK_FRAME(frame),GTK_SHADOW_OUT);
-	gtk_container_add(GTK_CONTAINER(frame),drawer->panel);
+	drawer->frame = gtk_frame_new(NULL);
+	gtk_widget_show(drawer->frame);
+	gtk_frame_set_shadow_type(GTK_FRAME(drawer->frame),GTK_SHADOW_OUT);
+	gtk_container_add(GTK_CONTAINER(drawer->frame),drawer->panel);
 
-	gtk_table_attach(GTK_TABLE(drawer->table),frame,1,2,1,2,
+	gtk_table_attach(GTK_TABLE(drawer->table),drawer->frame,1,2,1,2,
 			 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
 			 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
 			 0,0);
 
 	drawer->state = state;
-
+        drawer->hidebutton_enabled = hidebutton_enabled;
+	drawer->hidebutton_pixmap_enabled = hidebutton_enabled;
+	drawer_hidebutton_set(drawer);
 	gtk_widget_set_uposition(GTK_WIDGET(drawer),-100,-100);
 	drawer_widget_set_drop_zone(drawer);
 
@@ -619,7 +792,9 @@ drawer_widget_change_params(DrawerWidget *drawer,
 			    PanelBackType back_type,
 			    char *pixmap,
 			    int fit_pixmap_bg,
-			    GdkColor *back_color)
+			    GdkColor *back_color,
+			    int hidebutton_pixmap_enabled,
+			    int hidebutton_enabled)
 {
 	DrawerState oldstate;
 	PanelOrientation porient;
@@ -658,6 +833,10 @@ drawer_widget_change_params(DrawerWidget *drawer,
 	   	gtk_signal_emit(GTK_OBJECT(drawer),
 	   			drawer_widget_signals[STATE_CHANGE_SIGNAL],
 	   			drawer->state);
+
+	drawer->hidebutton_enabled = hidebutton_enabled;
+	drawer->hidebutton_pixmap_enabled = hidebutton_pixmap_enabled;
+	drawer_hidebutton_set(drawer);
 }
 
 void
@@ -671,7 +850,9 @@ drawer_widget_change_orient(DrawerWidget *drawer,
 				    panel->back_type,
 				    panel->back_pixmap,
 				    panel->fit_pixmap_bg,
-				    &panel->back_color);
+				    &panel->back_color,
+				    drawer->hidebutton_pixmap_enabled,
+				    drawer->hidebutton_enabled); 
 }
 
 void
