@@ -19,7 +19,11 @@
 
 #define DRAWER_PROPERTIES "drawer_properties"
 
-extern GArray *applets;
+/* Used for all the packing and padding options */
+#define CONFIG_PADDING_SIZE 3
+
+extern GList *applets;
+extern GList *applets_last;
 extern int applet_count;
 extern GlobalConfig global_config;
 
@@ -44,7 +48,7 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 		g_free(drawer->pixmap);
 	if(drawer->tooltip)
 		g_free(drawer->tooltip);
-	s = gtk_entry_get_text(GTK_ENTRY(pixentry));
+	s = gnome_pixmap_entry_get_filename(GNOME_PIXMAP_ENTRY(pixentry));
 	if(!s || !*s) {
 		drawer->pixmap =
 			gnome_unconditional_pixmap_file ("panel-drawer.png");
@@ -60,6 +64,7 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 							    drawer->pixmap);
 		}
 	}
+	g_free(s);
 	s = gtk_entry_get_text(GTK_ENTRY(tipentry));
 	if(!s || !*s)
 		drawer->tooltip = NULL;
@@ -74,26 +79,41 @@ static int
 properties_close_callback(GtkWidget *widget, gpointer data)
 {
 	Drawer *drawer = data;
-	GtkWidget *pixentry = gtk_object_get_data(GTK_OBJECT(widget),"pixmap");
-	GtkWidget *tipentry = gtk_object_get_data(GTK_OBJECT(widget),"tooltip");
+	/*GtkWidget *pixentry = gtk_object_get_data(GTK_OBJECT(widget),"pixmap");
+	GtkWidget *gtkpixentry =
+		gnome_pixmap_entry_gtk_entry(GNOME_PIXMAP_ENTRY(pixentry));
+	GtkWidget *tipentry = gtk_object_get_data(GTK_OBJECT(widget),"tooltip");*/
 	gtk_object_set_data(GTK_OBJECT(drawer->button),
 			    DRAWER_PROPERTIES,NULL);
-	gtk_signal_disconnect_by_data(GTK_OBJECT(pixentry),widget);
-	gtk_signal_disconnect_by_data(GTK_OBJECT(tipentry),widget);
+	/*gtk_signal_disconnect_by_data(GTK_OBJECT(gtkpixentry),widget);
+	gtk_signal_disconnect_by_data(GTK_OBJECT(tipentry),widget);*/
 	return FALSE;
 }
 
-void
-add_drawer_properties_page(GtkWidget *dialog, Drawer *drawer)
+static void
+set_toggle_not (GtkWidget *widget, gpointer data)
 {
-	GtkWidget *table;
+	PerPanelConfig *ppc = gtk_object_get_user_data(GTK_OBJECT(widget));
+	int *the_toggle = data;
+
+	*the_toggle = !(GTK_TOGGLE_BUTTON(widget)->active);
+	if (ppc->register_changes)
+		gnome_property_box_changed (GNOME_PROPERTY_BOX (ppc->config_window));
+}
+
+void
+add_drawer_properties_page(PerPanelConfig *ppc, Drawer *drawer)
+{
+        GtkWidget *dialog = ppc->config_window;
+        GtkWidget *table;
 	GtkWidget *f;
-	GtkWidget *box;
+	GtkWidget *box, *box_in;
 	GtkWidget *nbook;
 	GtkWidget *w;
-
+	GtkWidget *button;
+	
 	table = gtk_table_new(2, 2, FALSE);
-	gtk_container_border_width(GTK_CONTAINER(table), 5);
+	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 6);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 2);
 
@@ -101,17 +121,42 @@ add_drawer_properties_page(GtkWidget *dialog, Drawer *drawer)
 			      drawer->tooltip, dialog);
 	gtk_object_set_data(GTK_OBJECT(dialog),"tooltip",w);
 	
-	w = create_file_entry(table, "icon", 1, _("Icon"), drawer->pixmap,
-			      dialog);
+	w = create_pixmap_entry(table, "icon", 1, _("Icon"), drawer->pixmap,
+				dialog,64,64);
 	gtk_object_set_data(GTK_OBJECT(dialog),"pixmap",w);
 
 	f = gtk_frame_new(_("Applet appearance"));
 	gtk_container_add(GTK_CONTAINER(f),table);
 
 	box = gtk_vbox_new(FALSE,5);
-	gtk_container_border_width(GTK_CONTAINER(box), 5);
+	gtk_container_set_border_width(GTK_CONTAINER(box), 5);
 	gtk_box_pack_start(GTK_BOX(box),f,FALSE,FALSE,0);
 
+	f = gtk_frame_new(_("Drawer handle"));
+	box_in = gtk_vbox_new(FALSE,5);
+	button = gtk_check_button_new_with_label (_("Disable hidebutton"));
+	gtk_object_set_user_data(GTK_OBJECT(button),ppc);
+	if (!ppc->drawer_hidebutton)
+		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
+	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
+			    GTK_SIGNAL_FUNC (set_toggle_not),
+			    &ppc->drawer_hidebutton);
+	gtk_box_pack_start (GTK_BOX (box_in), button, TRUE, FALSE,
+			    CONFIG_PADDING_SIZE);
+
+	button = gtk_check_button_new_with_label (_("Disable hidebutton arrow"));
+	gtk_object_set_user_data(GTK_OBJECT(button),ppc);
+	if (!ppc->drawer_hidebutton_pixmap)
+		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
+	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
+			    GTK_SIGNAL_FUNC (set_toggle_not),
+			    &ppc->drawer_hidebutton_pixmap);
+	gtk_box_pack_start (GTK_BOX (box_in), button, TRUE, TRUE,
+			    CONFIG_PADDING_SIZE);
+	gtk_container_add(GTK_CONTAINER(f),box_in);
+	gtk_box_pack_start (GTK_BOX (box),f,FALSE,FALSE,0);
+
+	
 	nbook = GNOME_PROPERTY_BOX (dialog)->notebook;
 	gtk_notebook_append_page (GTK_NOTEBOOK(nbook),
 				  box, gtk_label_new (_("Drawer")));
@@ -164,10 +209,11 @@ destroy_drawer(GtkWidget *widget, gpointer data)
 static int
 enter_notify_drawer(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
 {
-	Drawer *drawer = data;
-	/*FIXME: do we want this autoraise piece?*/
-	gdk_window_raise(drawer->drawer->window);
-	return TRUE;
+  Drawer *drawer = data;
+
+  if (!gnome_win_hints_wm_exists())
+    gdk_window_raise(drawer->drawer->window);
+  return TRUE;
 }
 
 static Drawer *
@@ -231,7 +277,7 @@ create_empty_drawer_applet(char *tooltip, char *pixmap,
 	return create_drawer_applet(drawer_widget_new(orient,
 						      DRAWER_SHOWN,
 						      PANEL_BACK_NONE, NULL,
-						      TRUE, NULL),
+						      TRUE, NULL, TRUE, TRUE),
 				    tooltip,pixmap,orient);
 }
 
@@ -267,9 +313,7 @@ drawer_realize_cb(GtkWidget *button, Drawer *drawer)
 static void
 drawer_move_foreach(GtkWidget *w, gpointer user_data)
 {
-	int applet_id = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w),
-							    "applet_id"));
-	AppletInfo *info = get_applet_info(applet_id);
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(w), "applet_info");
 	
 	if(info->type == APPLET_DRAWER) {
 		Drawer *drawer = info->data;
@@ -359,7 +403,9 @@ load_drawer_applet(char *params, char *pixmap, char *tooltip,
 				 GTK_SIGNAL_FUNC(drawer_realize_cb),
 				 drawer);
 
-	applet_add_callback(applet_count-1,"properties",
+	g_return_if_fail(applets_last!=NULL);
+
+	applet_add_callback(applets_last->data,"properties",
 			    GNOME_STOCK_MENU_PROP,
 			    _("Properties..."));
 }
