@@ -21,6 +21,9 @@
 #define LAUNCHER_PROPERTIES "launcher_properties"
 
 extern GtkTooltips *panel_tooltips;
+
+extern GList *applets;
+extern GList *applets_last;
 extern int applet_count;
 
 static char *default_app_pixmap=NULL;
@@ -125,8 +128,6 @@ create_launcher (char *parameters, GnomeDesktopEntry *dentry)
 
 	launcher->dentry = dentry;
 
-	launcher->applet_id = -1;
-
 	gtk_object_set_data(GTK_OBJECT(launcher->button),
 			    LAUNCHER_PROPERTIES,NULL);
 
@@ -142,10 +143,20 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 	if (page != -1)
 		return;
 	
+			/* remove the old launcher callback */
+	gtk_signal_disconnect_by_func(GTK_OBJECT(launcher->button),
+			(GtkSignalFunc) launch, launcher->dentry);
+
 	gnome_desktop_entry_free(launcher->dentry);
 	launcher->dentry =
 		gnome_dentry_get_dentry(GNOME_DENTRY_EDIT(launcher->dedit));
 
+			/* and install the new one with the right dentry pointer */
+	gtk_signal_connect (GTK_OBJECT(launcher->button),
+			    "clicked",
+			    (GtkSignalFunc) launch,
+			    launcher->dentry);
+
 	gtk_tooltips_set_tip (panel_tooltips,launcher->button,
 			      launcher->dentry->comment,NULL);
 	
@@ -195,7 +206,7 @@ create_properties_dialog(Launcher *launcher)
 	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
 	
 	launcher->dedit =
-		gnome_dentry_edit_new(
+		gnome_dentry_edit_new_notebook(
 		      GTK_NOTEBOOK(GNOME_PROPERTY_BOX(dialog)->notebook));
 
 	gnome_dentry_edit_set_dentry(GNOME_DENTRY_EDIT(launcher->dedit),
@@ -250,7 +261,7 @@ _load_launcher_applet(char *params, GnomeDesktopEntry *dentry,
 				      launcher->button,
 				      launcher->dentry->comment,NULL);
 
-		applet_add_callback(applet_count-1,"properties",
+		applet_add_callback(applets_last->data,"properties",
 				    GNOME_STOCK_MENU_PROP,
 				    _("Properties..."));
 	}
@@ -285,7 +296,7 @@ ask_about_launcher(char *file, PanelWidget *panel, int pos)
 	notebook = gtk_notebook_new();
 	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox),notebook,
 			   TRUE,TRUE,5);
-	dee = GNOME_DENTRY_EDIT(gnome_dentry_edit_new(GTK_NOTEBOOK(notebook)));
+	dee = GNOME_DENTRY_EDIT(gnome_dentry_edit_new_notebook(GTK_NOTEBOOK(notebook)));
 	
 	if(file)
 		gtk_entry_set_text(GTK_ENTRY(dee->exec_entry), file);
