@@ -11,6 +11,7 @@
 #include <signal.h>
 #include <sys/wait.h>
 #include <gnome.h>
+#include <libgnorba/gnorba.h>
 
 #include "panel-include.h"
 
@@ -20,7 +21,8 @@ extern int panels_to_sync;
 extern int globals_to_sync;
 extern int need_complete_save;
 
-extern GArray *applets;
+extern GList *applets;
+extern GList *applets_last;
 extern int applet_count;
 
 extern char *panel_cfg_path;
@@ -34,37 +36,7 @@ GnomeClient *client = NULL;
 extern GList * children;
 
 /* True if parsing determined that all the work is already done.  */
-int just_exit = 0;
-
-/* The security cookie */
-char *cookie = NULL;
-
-/* These are the arguments that our application supports.  */
-static struct argp_option arguments[] =
-{
-#define DISCARD_KEY -1
-  { "discard-session", DISCARD_KEY, N_("ID"), 0, N_("Discard session"), 1 },
-  { NULL, 0, NULL, 0, NULL, 0 }
-};
-
-/* Forward declaration of the function that gets called when one of
-   our arguments is recognized.  */
-static error_t parse_an_arg (int key, char *arg, struct argp_state *state);
-
-/* This structure defines our parser.  It can be used to specify some
-   options for how our parsing function should be called.  */
-static struct argp parser =
-{
-	arguments,			/* Options.  */
-	parse_an_arg,			/* The parser function.  */
-	NULL,				/* Some docs.  */
-	NULL,				/* Some more docs.  */
-	NULL,				/* Child arguments -- gnome_init fills
-					   this in for us.  */
-	NULL,				/* Help filter.  */
-	NULL				/* Translation domain; for the app it
-					   can always be NULL.  */
-};
+char *just_exit = NULL;
 
 /*I guess this should be called after we load up, but the problem is
   we never know when all the applets are going to finish loading and
@@ -85,21 +57,24 @@ discard_session (char *id)
 	return;
 }
 
-	
-static error_t
-parse_an_arg (int key, char *arg, struct argp_state *state)
+static void
+parse_an_arg (poptContext state,
+              enum poptCallbackReason reason,
+              const struct poptOption *opt,
+              const char *arg, void *data)
 {
-	if (key == DISCARD_KEY) {
-		gnome_client_disable_master_connection ();
-		discard_session (arg);
-		just_exit = 1;
-		return 0;
-	}
-
-	/* We didn't recognize it.  */
-	return ARGP_ERR_UNKNOWN;
+  if(opt->val == 0) {
+    *((char **)data) = (char *)arg;
+  }
 }
 
+static struct poptOption options[] = {
+  { NULL, '\0', POPT_ARG_CALLBACK, parse_an_arg, 0},
+  {"discard-session", '\0', POPT_ARG_STRING, &just_exit,0, N_("Discard session"),
+   N_("ID")},
+  {NULL, '\0', 0, NULL, 0}
+};
+
 static int
 try_config_sync(gpointer data)
 {
@@ -107,12 +82,20 @@ try_config_sync(gpointer data)
 	return TRUE;
 }
 
+static int
+start_extern_timeout(gpointer data)
+{
+	extern_start_next();
+	return FALSE;
+}
+
 
 int
 main(int argc, char **argv)
 {
 	char buf[256];
-	struct sigaction sa;
+	CORBA_ORB orb;
+	CORBA_Environment ev;
 	
 	panel_cfg_path = g_strdup("/panel.d/default/");
 	old_panel_cfg_path = g_strdup("/panel.d/default/");
@@ -120,22 +103,21 @@ main(int argc, char **argv)
 	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
 	textdomain(PACKAGE);
 
-	panel_corba_register_arguments ();
+	CORBA_exception_init(&ev);
+	orb = gnome_CORBA_init_with_popt_table("panel", VERSION,
+					       &argc, argv, options, 0, NULL,
+					       GNORBA_INIT_SERVER_FUNC, &ev);
+	CORBA_exception_free(&ev);
 
-	gnome_init("panel", &parser, argc, argv, 0, NULL);
+	panel_corba_gtk_init(orb);
 
-	/* Setup the cookie */
-	cookie = create_cookie ();
-	g_snprintf(buf,256,"/panel/Secret/cookie-DISPLAY-%s",getenv("DISPLAY"));
-	gnome_config_private_set_string (buf, cookie);
-	gnome_config_sync();
-	
-	panel_corba_gtk_init();
+	if (just_exit) {
+	  gnome_client_disable_master_connection ();
+	  discard_session (just_exit);
+	  return 0;
+	}
 
-	if (just_exit)
-		return 0;
-
-	client= gnome_master_client ();
+	client = gnome_master_client ();
 
 	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
 			    GTK_SIGNAL_FUNC (panel_session_save), NULL);
@@ -169,7 +151,6 @@ main(int argc, char **argv)
 	gnome_client_set_clone_command (client, 1, argv);
 	gnome_client_set_restart_command (client, 1, argv);
 
-	applets = g_array_new(FALSE, FALSE, sizeof(AppletInfo));
 	applet_count = 0;
 
 	panel_tooltips = gtk_tooltips_new();
@@ -191,8 +172,12 @@ main(int argc, char **argv)
 	  indicated though*/
 	config_sync_timeout = gtk_timeout_add(10*1000,try_config_sync,NULL);
 	
+	/*actually start executing the external applets*/
+	/*extern_start_next();*/
+	gtk_timeout_add(500,start_extern_timeout,NULL);
+	
 	/* I use the glue code to avoid making this a C++ file */
-	panel_corba_gtk_main ("IDL:GNOME/Panel:1.0");
+	gtk_main ();
 
 	return 0;
 }
