@@ -31,7 +31,8 @@ static char *gnome_folder = NULL;
 GList *small_icons = NULL;
 GList *dot_buttons = NULL;
 
-extern GArray *applets;
+extern GList *applets;
+extern GList *applets_last;
 extern int applet_count;
 
 extern GlobalConfig global_config;
@@ -74,6 +75,7 @@ about_cb (GtkWidget *widget, gpointer data)
 	  "Federico Mena (quartic@gimp.org)",
 	  "Tom Tromey (tromey@cygnus.com)",
 	  "Ian Main (slow@intergate.bc.ca)",
+	  "Elliot Lee (sopwith@redhat.com)",
 	  "and finally, The Knights Who Say ... NI!",
 	  NULL
 	  };
@@ -93,13 +95,7 @@ about_cb (GtkWidget *widget, gpointer data)
 static void
 about_gnome_cb(GtkObject *object, char *program_path)
 {
-  if(!fork()) {
-    int i;
-    for(i = 0; i < 255; i++)
-      close(i);
-    execlp(program_path, program_path, NULL);
-    _exit(1);
-  }
+  gnome_execute_async(NULL, 1, &program_path);
 }
 
 static void
@@ -203,8 +199,9 @@ add_drawers_from_dir(char *dirname, char *name, int pos, PanelWidget *panel)
 	load_drawer_applet(NULL,pixmap_name,subdir_name,
 			   panel,pos);
 	
-	info = get_applet_info(applet_count -1);
-	g_return_if_fail(info);
+	g_return_if_fail(applets_last!=NULL);
+	info = applets_last->data;
+	g_return_if_fail(info!=NULL);
 	
 	drawer = info->data;
 	g_return_if_fail(drawer);
@@ -329,7 +326,7 @@ edit_dentry(GtkWidget *widget, GnomeDesktopEntry *dentry)
 	gtk_window_set_title(GTK_WINDOW(dialog), _("Desktop entry properties"));
 	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
 	
-	o = gnome_dentry_edit_new(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(dialog)->notebook));
+	o = gnome_dentry_edit_new_notebook(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(dialog)->notebook));
 	gtk_object_set_data(o,"location",dentry->location);
 
 	gnome_dentry_edit_set_dentry(GNOME_DENTRY_EDIT(o),dentry);
@@ -364,7 +361,7 @@ edit_direntry(GtkWidget *widget, MenuFinfo *mf)
 	gtk_window_set_title(GTK_WINDOW(dialog), _("Desktop entry properties"));
 	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
 	
-	o = gnome_dentry_edit_new(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(dialog)->notebook));
+	o = gnome_dentry_edit_new_notebook(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(dialog)->notebook));
 
 	dentry = gnome_desktop_entry_load_unconditional(dirfile);
 	if (dentry) {
@@ -526,6 +523,86 @@ destroy_item_menu(GtkWidget *w, gpointer data)
 	return FALSE;
 }
 
+/* This is a _horrible_ hack to have this here. This needs to be added to the
+ * GTK+ menuing code in some manner.
+ */
+static void  
+drag_end_menu_cb (GtkWidget *widget, GdkDragContext     *context)
+{
+  GtkWidget *xgrab_shell;
+  GtkWidget *parent;
+  GdkEvent *current_event;
+  GtkMenuShell *menu_shell;
+
+  /* Find the last viewable ancestor, and make an X grab on it
+   */
+  parent = widget->parent;
+  xgrab_shell = NULL;
+  while (parent)
+    {
+      gboolean viewable = TRUE;
+      GtkWidget *tmp = parent;
+      
+      while (tmp)
+	{
+	  if (!GTK_WIDGET_MAPPED (tmp))
+	    {
+	      viewable = FALSE;
+	      break;
+	    }
+	  tmp = tmp->parent;
+	}
+      
+      if (viewable)
+	xgrab_shell = parent;
+      
+      parent = GTK_MENU_SHELL (parent)->parent_menu_shell;
+    }
+  
+  if (xgrab_shell)
+    {
+      GdkCursor *cursor = gdk_cursor_new (GDK_ARROW);
+
+      if ((gdk_pointer_grab (xgrab_shell->window, TRUE,
+			     GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+			     GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
+			     GDK_POINTER_MOTION_MASK,
+			     NULL, cursor, GDK_CURRENT_TIME) == 0))
+	{
+	  if (gdk_keyboard_grab (xgrab_shell->window, TRUE,
+				 GDK_CURRENT_TIME) == 0)
+	    GTK_MENU_SHELL (xgrab_shell)->have_xgrab = TRUE;
+	  else
+	    {
+	      gdk_pointer_ungrab (GDK_CURRENT_TIME);
+	    }
+	}
+
+      gdk_cursor_destroy (cursor);
+    }
+}
+
+static void  
+drag_data_get_menu_cb (GtkWidget *widget, GdkDragContext     *context,
+		       GtkSelectionData   *selection_data, guint info,
+		       guint time, GnomeDesktopEntry *dentry)
+{
+	gchar *uri_list = g_copy_strings ("file:", dentry->location, NULL);
+	gtk_selection_data_set (selection_data,
+				selection_data->target, 8, uri_list,
+				strlen(uri_list));
+	g_free(uri_list);
+}
+
+static void  
+drag_data_get_dir_cb (GtkWidget *widget, GdkDragContext     *context,
+		      GtkSelectionData   *selection_data, guint info,
+		      guint time, char *directory)
+{
+	gtk_selection_data_set (selection_data,
+				selection_data->target, 8, directory,
+				strlen(directory));
+}
 
 static void
 setup_title_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *title,
@@ -542,7 +619,7 @@ setup_title_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *title,
 	
 	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
 	gtk_widget_show (align);
-	gtk_container_border_width (GTK_CONTAINER (align), 1);
+	gtk_container_set_border_width (GTK_CONTAINER (align), 1);
 
 	if (pixmap) {
 		gtk_container_add (GTK_CONTAINER (align), pixmap);
@@ -597,6 +674,10 @@ static void
 setup_full_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *title,
 		     GnomeDesktopEntry *dentry)
 {
+        static GtkTargetEntry menu_item_targets[] = {
+		{ "text/uri-list", 0, 0 }
+	};
+
 	GtkWidget *label, *hbox, *align;
 
 	label = gtk_label_new (title);
@@ -608,7 +689,7 @@ setup_full_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *title,
 	
 	align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
 	gtk_widget_show (align);
-	gtk_container_border_width (GTK_CONTAINER (align), 1);
+	gtk_container_set_border_width (GTK_CONTAINER (align), 1);
 
 	if (pixmap) {
 		gtk_container_add (GTK_CONTAINER (align), pixmap);
@@ -648,6 +729,16 @@ setup_full_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *title,
 		/*this is not really a problem for large fonts but it
 		  makes the button smaller*/
 		gtk_widget_set_usize(w,0,16);
+
+		gtk_drag_source_set(menuitem,
+				    GDK_BUTTON1_MASK,
+				    menu_item_targets, 1,
+				    GDK_ACTION_COPY);
+
+		gtk_signal_connect(GTK_OBJECT(menuitem), "drag_data_get",
+				   drag_data_get_menu_cb, dentry);
+		gtk_signal_connect(GTK_OBJECT(menuitem), "drag_end",
+				   drag_end_menu_cb, NULL);
 	}
 	gtk_container_add (GTK_CONTAINER (menuitem), hbox);
 
@@ -660,6 +751,72 @@ setup_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *title)
 	setup_full_menuitem(menuitem,pixmap,title,NULL);
 }
 
+static void
+setup_directory_drag (GtkWidget *menuitem, char *directory)
+{
+        static GtkTargetEntry menu_item_targets[] = {
+		{ "application/x-panel-directory", 0, 0 }
+	};
+
+	gtk_drag_source_set(menuitem,
+			    GDK_BUTTON1_MASK,
+			    menu_item_targets, 1,
+			    GDK_ACTION_COPY);
+	
+	gtk_signal_connect_full(GTK_OBJECT(menuitem), "drag_data_get",
+			   GTK_SIGNAL_FUNC (drag_data_get_dir_cb), NULL,
+			   g_strdup (directory), (GtkDestroyNotify)g_free,
+			   FALSE, FALSE);
+	gtk_signal_connect(GTK_OBJECT(menuitem), "drag_end",
+			   GTK_SIGNAL_FUNC (drag_end_menu_cb), NULL);
+}
+
+static char *
+get_applet_goad_id_from_dentry(GnomeDesktopEntry *ii)
+{
+	int i;
+	int constantlen = strlen("--activate-goad-server");
+	char *goad_id=NULL;
+	/*FIXME:
+	  this is a horrible horrible hack and should be taken out
+	  and shot, once we add proper way to do this*/
+	for(i=1;ii->exec[i];i++) {
+		if(strncmp("--activate-goad-server",
+			   ii->exec[i],constantlen)==0) {
+			if(strlen(ii->exec[i])>constantlen)
+				goad_id = g_strdup(&ii->exec[i][constantlen+1]);
+			else
+				goad_id = g_strdup(ii->exec[i+1]);
+		}
+	}
+	return goad_id;
+}
+
+
+static void
+setup_applet_drag (GtkWidget *menuitem, GnomeDesktopEntry *ii)
+{
+        static GtkTargetEntry menu_item_targets[] = {
+		{ "application/x-panel-applet", 0, 0 }
+	};
+	char *goad_id = get_applet_goad_id_from_dentry(ii);
+	
+	if(!goad_id)
+		return;
+	
+	gtk_drag_source_set(menuitem,
+			    GDK_BUTTON1_MASK,
+			    menu_item_targets, 1,
+			    GDK_ACTION_COPY);
+	
+	gtk_signal_connect_full(GTK_OBJECT(menuitem), "drag_data_get",
+			   GTK_SIGNAL_FUNC (drag_data_get_dir_cb), NULL,
+			   goad_id, (GtkDestroyNotify)g_free, FALSE, FALSE);
+	gtk_signal_connect(GTK_OBJECT(menuitem), "drag_end",
+			   GTK_SIGNAL_FUNC (drag_end_menu_cb), NULL);
+
+}
+
 static void
 free_app_def (GtkWidget *widget, void *data)
 {
@@ -698,21 +855,17 @@ static int
 add_applet (GtkWidget *w, gpointer data)
 {
 	GnomeDesktopEntry *ii = data;
-	char *path;
-	char *param;
 
-	path = ii->exec[0];
-
-	g_return_val_if_fail(path!=NULL,FALSE);
+	char *goad_id = get_applet_goad_id_from_dentry(ii);
 	
-	if((ii->exec[1])!=NULL)
-		param = gnome_string_joinv (" ", ii->exec + 1);
-	else
-		param = NULL;
+	if(!goad_id) {
+		g_warning(_("Can't get goad_id from desktop entry!"));
+		return TRUE;
+	}
+	load_extern_applet(goad_id,NULL,current_panel,0);
 
-	load_extern_applet(path,param,NULL,current_panel,0);
+	g_free(goad_id);
 
-	if(param) g_free(param);
 	return TRUE;
 }
 
@@ -724,22 +877,28 @@ check_finfo_list(GList *finfo)
 
 	for(;finfo!=NULL;finfo=g_list_next(finfo)) {
 		fi = finfo->data;
-		if (stat (fi->name, &s) == -1)
-			continue;
-		if(fi->mtime != s.st_mtime)
+		if (stat (fi->name, &s) == -1 ||
+		    fi->mtime != s.st_mtime)
 			return FALSE;
 	}
 	return TRUE;
 }
 
 static FileInfo *
-make_finfo(char *name)
+make_finfo(char *name, int force)
 {
 	struct stat s;
 	FileInfo *fi;
 
-	if (stat (name, &s) == -1)
+	if (stat (name, &s) == -1) {
+		if(force) {
+			fi = g_new(FileInfo,1);
+			fi->name = g_strdup(name);
+			fi->mtime = 0;
+			return fi;
+		}
 		return NULL;
+	}
 
 	fi = g_new(FileInfo,1);
 	fi->name = g_strdup(name);
@@ -792,7 +951,8 @@ static void add_menu_widget (Menu *menu, GList *menudirl,
 static GtkWidget * create_menu_at (GtkWidget *menu,
 				   char *menudir, 
 				   int applets, char *dir_name,
-				   char *pixmap_name, int fake_submenus);
+				   char *pixmap_name, int fake_submenus,
+				   int force);
 
 /*if menu is NULL that means just reread the menu don't do anything with the
   applet*/
@@ -878,7 +1038,8 @@ check_and_reread(GtkWidget *menuw,Menu *menu,int main_menu)
 						       mf->applets,
 						       mf->dir_name,
 						       mf->pixmap_name,
-						       TRUE);
+						       TRUE,
+						       FALSE);
 				old_menu = menuw;
 			}
 			/*free up stuff that won't be freed somewhere else*/
@@ -938,7 +1099,8 @@ create_menu_at (GtkWidget *menu,
 		int applets,
 		char *dir_name,
 		char *pixmap_name,
-		int fake_submenus)
+		int fake_submenus,
+		int force)
 {	
 	GnomeDesktopEntry *item_info=NULL;
 	GnomeDesktopEntry *dir_info=NULL;
@@ -956,6 +1118,9 @@ create_menu_at (GtkWidget *menu,
 	
 	MenuFinfo *mf = NULL;
 	
+	if(!force && !g_file_exists(menudir))
+		return menu;
+	
 	/*try getting the dir_name and pixmap if we haven't gotten them*/
 	if(!dir_name || !pixmap_name) {
 		char *dentry_name;
@@ -967,7 +1132,7 @@ create_menu_at (GtkWidget *menu,
 		/*add the .directory file to the checked files list,
 		  but only if we can stat it (if we can't it probably
 		  doesn't exist)*/
-		fi = make_finfo(dentry_name);
+		fi = make_finfo(dentry_name,FALSE);
 		if(fi)
 			finfo = g_list_prepend(finfo,fi);
 		g_free (dentry_name);
@@ -980,7 +1145,7 @@ create_menu_at (GtkWidget *menu,
 
 	
 	/*add dir to the checked files list*/
-	fi = make_finfo(menudir);
+	fi = make_finfo(menudir,force);
 	if(!fi)
 		g_warning("Something is wrong, directory %s can't be stated",
 			  menudir);
@@ -991,7 +1156,7 @@ create_menu_at (GtkWidget *menu,
 	  but only if we can stat it (if we can't it probably doesn't
 	  exist)*/
 	filename = g_concat_dir_and_file(menudir,".order");
-	fi = make_finfo(filename);
+	fi = make_finfo(filename,FALSE);
 	if(fi)
 		finfo = g_list_prepend(finfo,fi);
 	g_free(filename);
@@ -1036,7 +1201,7 @@ create_menu_at (GtkWidget *menu,
 			/*add the .directory file to the checked files list,
 			  but only if we can stat it (if we can't it probably
 			  doesn't exist)*/
-			fi = make_finfo(dentry_name);
+			fi = make_finfo(dentry_name,FALSE);
 			if(fi)
 				finfo = g_list_prepend(finfo,fi);
 			g_free (dentry_name);
@@ -1054,7 +1219,8 @@ create_menu_at (GtkWidget *menu,
 						      applets,
 						      menuitem_name,
 						      pixmap_name,
-						      fake_submenus);
+						      fake_submenus,
+						      FALSE);
 
 			if (!sub) {
 				g_free(filename);
@@ -1101,9 +1267,9 @@ create_menu_at (GtkWidget *menu,
 
 		pixmap = NULL;
 		if (pixmap_name && g_file_exists (pixmap_name)) {
-			pixmap = gnome_pixmap_new_from_file_at_size (pixmap_name,
-								     SMALL_ICON_SIZE,
-								     SMALL_ICON_SIZE);
+			pixmap = gnome_stock_pixmap_widget_at_size (NULL, pixmap_name,
+								    SMALL_ICON_SIZE,
+								    SMALL_ICON_SIZE);
 			if (pixmap)
 				gtk_widget_show (pixmap);
 		}
@@ -1112,6 +1278,9 @@ create_menu_at (GtkWidget *menu,
 		  a submenu or an applet, so that the item can be added*/
  		setup_full_menuitem (menuitem, pixmap, menuitem_name,
 				     (sub||applets)?NULL:item_info);
+		if (applets && !sub)
+			setup_applet_drag (menuitem, item_info);
+
 		if(add_separator) {
 			add_menu_separator(menu);
 			first_item++;
@@ -1145,27 +1314,31 @@ create_menu_at (GtkWidget *menu,
 	if(!applets && items>0) {
 		GtkWidget *pixmap;
 		menuitem = gtk_menu_item_new();
-		gtk_widget_show(menuitem);
 		gtk_menu_insert(GTK_MENU(menu),menuitem,first_item);
 
+		gtk_widget_show(menuitem);
+
 		pixmap = NULL;
 		if (pixmap_name && g_file_exists (pixmap_name)) {
-			pixmap = gnome_pixmap_new_from_file_at_size (pixmap_name,
-								     SMALL_ICON_SIZE,
-								     SMALL_ICON_SIZE);
+			pixmap = gnome_stock_pixmap_widget_at_size (NULL, pixmap_name,
+								    SMALL_ICON_SIZE,
+								    SMALL_ICON_SIZE);
 			if (pixmap)
 				gtk_widget_show (pixmap);
 		} else if (gnome_folder && g_file_exists (gnome_folder)) {
-			pixmap = gnome_pixmap_new_from_file_at_size (gnome_folder,
-								     SMALL_ICON_SIZE,
-								     SMALL_ICON_SIZE);
+			pixmap = gnome_stock_pixmap_widget_at_size (NULL, gnome_folder,
+								    SMALL_ICON_SIZE,
+								    SMALL_ICON_SIZE);
 			if (pixmap)
 				gtk_widget_show (pixmap);
 		}
+
 		menuitem = gtk_menu_item_new();
 		setup_title_menuitem(menuitem,pixmap,
 				     dir_name?dir_name:"Menu",mf);
 		gtk_menu_insert(GTK_MENU(menu),menuitem,first_item);
+
+		setup_directory_drag (menuitem, mf->menudir);
 	}
 
 	if(dir_info)
@@ -1243,7 +1416,7 @@ create_applets_menu(int fake_submenus)
 	}
 
 	applet_menu = create_menu_at(NULL,menudir,TRUE,_("Applets"),NULL,
-				     fake_submenus);
+				     fake_submenus,FALSE);
 	g_free (menudir);
 	return applet_menu;
 }
@@ -1263,6 +1436,7 @@ create_new_panel(GtkWidget *w,gpointer data)
 					   SNAPPED_EXPLICIT_HIDE,
 					   SNAPPED_SHOWN,
 					   TRUE,
+					   TRUE,
 					   PANEL_BACK_NONE,
 					   NULL,
 					   TRUE,
@@ -1274,6 +1448,8 @@ create_new_panel(GtkWidget *w,gpointer data)
 		panel = corner_widget_new(CORNER_NE,
 					  PANEL_HORIZONTAL,
 					  CORNER_SHOWN,
+					  TRUE,
+					  TRUE,
 					  PANEL_BACK_NONE,
 					  NULL,
 					  TRUE,
@@ -1328,7 +1504,7 @@ create_system_menu(GtkWidget *menu, int fake_submenus)
 	g_free (menu_base);
 	if (g_file_exists (menudir)) {
 		menu = create_menu_at (menu,menudir,FALSE,_("System Menus"),
-				       gnome_folder, fake_submenus);
+				       gnome_folder, fake_submenus, FALSE);
 		g_return_val_if_fail(menu,NULL);
 		g_free (menudir);
 	} else
@@ -1338,19 +1514,18 @@ create_system_menu(GtkWidget *menu, int fake_submenus)
 }
 
 static GtkWidget *
-create_user_menu(GtkWidget *menu, int fake_submenus)
+create_user_menu(GtkWidget *menu, int fake_submenus, int force)
 {
 	char *menu_base = gnome_util_home_file ("apps");
 	char *menudir = g_concat_dir_and_file (menu_base, ".");
 	
-	g_free (menu_base);
 	if (!g_file_exists (menudir))
-		mkdir (menudir, 0755);
-	if (g_file_exists (menudir)) {
-		menu = create_menu_at (menu,menudir, FALSE,
-				       _("User Menus"),
-				       gnome_folder,fake_submenus);
-	}
+		mkdir (menu_base, 0755);
+	g_free (menu_base);
+	menu = create_menu_at (menu,menudir, FALSE,
+			       _("User Menus"),
+			       gnome_folder,fake_submenus,
+			       force);
 	g_free (menudir); 
 	return menu;
 }
@@ -1378,7 +1553,7 @@ create_panel_root_menu(GtkWidget *panel)
 	setup_menuitem (menuitem, 0, _("User Menus"));
 	gtk_menu_append (GTK_MENU (panel_menu), menuitem);
 	menu = gtk_menu_new();
-	create_user_menu(menu,TRUE);
+	create_user_menu(menu,TRUE,TRUE);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),menu);
 	gtk_signal_connect(GTK_OBJECT(menuitem),"select",
 			   GTK_SIGNAL_FUNC(submenu_to_display),
@@ -1506,7 +1681,7 @@ make_panel_submenu (GtkWidget *menu, int fake_submenus)
 static void
 panel_lock (GtkWidget *widget, void *data)
 {
-	system ("(xlock&)");
+	system ("(xscreensaver-command -activate&)");
 }
 
 static void
@@ -1580,9 +1755,9 @@ create_root_menu(int fake_submenus, MainMenuType type)
 	uroot_menu = NULL;
 	
 	if(type == MAIN_MENU_BOTH)
-		root_menu = create_user_menu(root_menu, fake_submenus);
+		root_menu = create_user_menu(root_menu, fake_submenus, FALSE);
 	else
-		uroot_menu = create_user_menu(NULL, fake_submenus);
+		uroot_menu = create_user_menu(NULL, fake_submenus, TRUE);
 	
 	if(type == MAIN_MENU_USER) {
 		GtkWidget *menuitem;
@@ -1624,7 +1799,7 @@ add_menu_widget (Menu *menu, GList *menudirl, int main_menu, int fake_subs)
 		for(li=menudirl;li!=NULL;li=g_list_next(li))
 			menu->menu = create_menu_at (menu->menu,li->data,
 						     FALSE, NULL, NULL,
-						     fake_subs);
+						     fake_subs, FALSE);
 	}
 	gtk_signal_connect (GTK_OBJECT (menu->menu), "deactivate",
 			    GTK_SIGNAL_FUNC (menu_deactivate), menu);
@@ -1636,9 +1811,8 @@ menu_button_pressed(GtkWidget *widget, gpointer data)
 	Menu *menu = data;
 	GdkEventButton *bevent = (GdkEventButton*)gtk_get_current_event();
 	GtkWidget *wpanel = get_panel_parent(menu->button);
-	int applet_id =
-		GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(menu->button),
-						    "applet_id"));
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(menu->button),
+					       "applet_info");
 	int main_menu = (strcmp (menu->path, ".") == 0);
 	
 	check_and_reread(menu->menu,menu,main_menu);
@@ -1657,8 +1831,7 @@ menu_button_pressed(GtkWidget *widget, gpointer data)
 	gtk_grab_remove(menu->button);
 
 	gtk_menu_popup(GTK_MENU(menu->menu), 0,0, applet_menu_position,
-		       GINT_TO_POINTER(applet_id),
-		       bevent->button, bevent->time);
+		       info, bevent->button, bevent->time);
 }
 
 static char *
@@ -1812,8 +1985,12 @@ properties_apply_callback(GtkWidget *widget, int page, gpointer data)
 		menu->path = g_strdup(".");
 	} else {
 		g_free(menu->path);
-		s = gtk_entry_get_text(GTK_ENTRY(pathentry));
-		if(!s || !*s)
+		s = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(pathentry),
+						   TRUE);
+		if(!s) {
+			g_warning(_("Can't open directory, using main menu!"));
+			menu->path = g_strdup(".");
+		} else if(!*s)
 			menu->path = g_strdup(".");
 		else
 			menu->path = g_strdup(s);
@@ -1852,20 +2029,11 @@ static int
 properties_close_callback(GtkWidget *widget, gpointer data)
 {
 	Menu *menu = data;
-	GtkWidget *pathentry = gtk_object_get_data(GTK_OBJECT(widget),"path");
 	gtk_object_set_data(GTK_OBJECT(menu->button),
 			    MENU_PROPERTIES,NULL);
-	gtk_signal_disconnect_by_data(GTK_OBJECT(pathentry),widget);
 	return FALSE;
 }
 
-static void
-notify_entry_change (GtkWidget *widget, void *data)
-{
-	GnomePropertyBox *box = GNOME_PROPERTY_BOX (data);
-
-	gnome_property_box_changed (box);
-}
 static void
 toggle_prop(GtkWidget *widget, void *data)
 {
@@ -1919,13 +2087,13 @@ create_properties_dialog(Menu *menu)
 	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
 	
 	vbox = gtk_vbox_new(FALSE,5);
-	gtk_container_border_width(GTK_CONTAINER(vbox),5);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox),5);
 
 	f = gtk_frame_new(_("Menu Type"));
 	gtk_box_pack_start(GTK_BOX(vbox),f,FALSE,FALSE,0);
 	
 	box = gtk_hbox_new(FALSE,5);
-	gtk_container_border_width(GTK_CONTAINER(box),5);
+	gtk_container_set_border_width(GTK_CONTAINER(box),5);
 	gtk_container_add(GTK_CONTAINER(f),box);
 	
 	w = gtk_radio_button_new_with_label (NULL, _("Main Menu"));
@@ -1955,7 +2123,7 @@ create_properties_dialog(Menu *menu)
 	gtk_box_pack_start(GTK_BOX(vbox),f,FALSE,FALSE,0);
 	
 	box = gtk_vbox_new(FALSE,5);
-	gtk_container_border_width(GTK_CONTAINER(box),5);
+	gtk_container_set_border_width(GTK_CONTAINER(box),5);
 	gtk_container_add(GTK_CONTAINER(f),box);
 
 	w = gtk_radio_button_new_with_label (NULL, _("Both System and User menus on the "
@@ -1996,21 +2164,23 @@ create_properties_dialog(Menu *menu)
 	gtk_box_pack_start(GTK_BOX(vbox),f,FALSE,FALSE,0);
 	
 	box = gtk_hbox_new(FALSE,5);
-	gtk_container_border_width(GTK_CONTAINER(box),5);
+	gtk_container_set_border_width(GTK_CONTAINER(box),5);
 	gtk_container_add(GTK_CONTAINER(f),box);
 	
 	w = gtk_label_new(_("Menu path"));
 	gtk_box_pack_start(GTK_BOX(box),w,FALSE,FALSE,0);
 
 	w = gnome_file_entry_new("menu_path",_("Browse"));
+	gnome_file_entry_set_directory(GNOME_FILE_ENTRY(w), TRUE);
+
 	t = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (w));
-	gtk_object_set_data(GTK_OBJECT(dialog),"path",t);
+	gtk_object_set_data(GTK_OBJECT(dialog),"path",w);
 	if (menu->path)
 		gtk_entry_set_text(GTK_ENTRY(t), menu->path);
 	gtk_box_pack_start(GTK_BOX(box),w,TRUE,TRUE,0);
-	gtk_signal_connect (GTK_OBJECT (t), "changed",
-			    GTK_SIGNAL_FUNC(notify_entry_change),
-			    dialog);
+	gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
+					       GTK_SIGNAL_FUNC(gnome_property_box_changed),
+					       GTK_OBJECT(dialog));
 	
 	gtk_notebook_append_page (GTK_NOTEBOOK(GNOME_PROPERTY_BOX (dialog)->notebook),
 				  vbox, gtk_label_new (_("Menu")));
@@ -2056,7 +2226,7 @@ load_menu_applet(char *params, int main_menu_type,
 		register_toy(menu->button,menu,
 			     panel,pos,APPLET_MENU);
 
-		applet_add_callback(applet_count-1,"properties",
+		applet_add_callback(applets_last->data,"properties",
 				    GNOME_STOCK_MENU_PROP,
 				    _("Properties..."));
 	}
@@ -2156,8 +2326,8 @@ panel_menu_position (GtkMenu *menu, int *x, int *y, gpointer data)
 void
 applet_menu_position (GtkMenu *menu, int *x, int *y, gpointer data)
 {
+	AppletInfo *info = data;
 	int wx, wy;
-	AppletInfo *info = get_applet_info(GPOINTER_TO_INT(data));
 	PanelWidget *panel;
 	GtkWidget *w; /*the panel window widget*/
 
