@@ -26,16 +26,9 @@
 #include "logout.h"
 #include "extern.h"
 
-#include "mulapp.h"
-#include "exec-queue.h"
-
-#include "orbit-glue.h"
-
 #include "panel-util.h"
 #include "gdkextra.h"
 
-#include "cookie.h"
-
 /* Gross backward compatibility hack.  */
 #ifndef GPOINTER_TO_INT
 # if SIZEOF_INT == SIZEOF_VOID_P
