@@ -20,6 +20,19 @@ GtkWidget * create_file_entry(GtkWidget *table,
 			      char *label,
 			      char *text,
 			      GtkWidget *w);
+GtkWidget * create_pixmap_entry(GtkWidget *table,
+				char *history_id,
+				int row,
+				char *label,
+				char *text,
+				GtkWidget *w,
+				int pw, int ph /*preview size*/);
+GtkWidget * create_icon_entry(GtkWidget *table,
+			      char *history_id,
+			      int row,
+			      char *label,
+			      char *text,
+			      GtkWidget *w);
 
 GList * my_g_list_swap_next(GList *list, GList *dl);
 GList * my_g_list_swap_prev(GList *list, GList *dl);
