@@ -43,8 +43,6 @@ static void panel_widget_draw		(GtkWidget        *widget,
 					 GdkRectangle     *area);
 
 
-GdkCursor *fleur_cursor;
-
 /*global settings*/
 int pw_explicit_step = 50;
 int pw_drawer_step = 20;
@@ -675,12 +673,13 @@ panel_widget_size_request(GtkWidget *widget, GtkRequisition *requisition)
 
 	panel = PANEL_WIDGET(widget);
 
-	requisition->height = 0;
-	requisition->width = 0;
-	if(panel->orient == PANEL_HORIZONTAL)
+	if(panel->orient == PANEL_HORIZONTAL) {
 		requisition->width = pw_applet_padding;
-	else
+		requisition->height = PANEL_MINIMUM_WIDTH;
+	} else {
 		requisition->height = pw_applet_padding;
+		requisition->width = PANEL_MINIMUM_WIDTH;
+	}
 
 	for(list = panel->applet_list; list!=NULL; list = g_list_next(list)) {
 		AppletData *ad = list->data;
@@ -1078,7 +1077,8 @@ panel_try_to_set_default_back(PanelWidget *panel)
 	g_return_if_fail(panel!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(panel));
 
-	ns = gtk_style_new();
+	ns = gtk_rc_get_style(GTK_WIDGET(panel));
+	if(!ns) ns = gtk_style_new();
 
 	gtk_style_ref(ns);
 	gtk_widget_set_style(GTK_WIDGET(panel), ns);
@@ -1278,7 +1278,6 @@ panel_widget_realize(GtkWidget *w, gpointer data)
 				       panel->pixw,
 				       panel->pixh,
 				       gtk_widget_get_visual(w)->depth);
-
 	panel_widget_draw_all(panel);
 }
 
@@ -1377,9 +1376,6 @@ panel_widget_init (PanelWidget *panel)
 	panel->master_widget = NULL;
 	panel->drop_widget = GTK_WIDGET(panel);
 
-	if(!fleur_cursor)
-		fleur_cursor = gdk_cursor_new(GDK_FLEUR);
-
 	gtk_signal_connect(GTK_OBJECT(panel),
 			   "destroy",
 			   GTK_SIGNAL_FUNC(panel_widget_destroy),
@@ -1497,12 +1493,14 @@ _panel_widget_applet_drag_start(PanelWidget *panel, GtkWidget *applet)
 
 	gtk_grab_add(applet);
 	if(applet->window) {
+		GdkCursor *fleur_cursor = gdk_cursor_new(GDK_FLEUR);
 		gdk_pointer_grab(applet->window,
 				 FALSE,
 				 APPLET_EVENT_MASK,
 				 NULL,
 				 fleur_cursor,
 				 GDK_CURRENT_TIME);
+		gdk_cursor_destroy(fleur_cursor);
 	}
 }
 
@@ -1823,6 +1821,7 @@ panel_widget_applet_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 	switch (event->type) {
 		case GDK_BUTTON_PRESS:
 			bevent = (GdkEventButton *) event;
+			printf("the appwidget %lX\n",(long)widget);
 
 			/* don't propagate this event */
 			if (panel->currently_dragged_applet)
@@ -1852,19 +1851,6 @@ panel_widget_applet_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 	return FALSE;
 }
 
-
-static GtkWidget *
-listening_parent(GtkWidget *widget)
-{
-	g_return_val_if_fail(widget!=NULL,NULL);
-	g_return_val_if_fail(GTK_IS_WIDGET(widget),NULL);
-
-	if (GTK_WIDGET_NO_WINDOW(widget))
-		return listening_parent(widget->parent);
-
-	return widget;
-}
-
 static int
 panel_sub_event_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
@@ -1878,9 +1864,8 @@ panel_sub_event_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 		case GDK_BUTTON_PRESS:
 		case GDK_BUTTON_RELEASE:
 			bevent = (GdkEventButton *)event;
-			if(bevent->button != 1)
-				return gtk_widget_event(
-				    listening_parent(widget->parent), event);
+			if(bevent->button != 1 || panel_applet_in_drag)
+				return gtk_widget_event(data, event);
 
 			break;
 
@@ -1909,11 +1894,11 @@ bind_applet_events(GtkWidget *widget, gpointer data)
 	if (!GTK_WIDGET_NO_WINDOW(widget))
 		gtk_signal_connect(GTK_OBJECT(widget), "event",
 				   (GtkSignalFunc) panel_sub_event_handler,
-				   NULL);
+				   data);
 	
 	if (GTK_IS_CONTAINER(widget))
 		gtk_container_foreach (GTK_CONTAINER (widget),
-				       bind_applet_events, 0);
+				       bind_applet_events, data);
 }
 static int
 panel_widget_applet_destroy(GtkWidget *applet, gpointer data)
@@ -1967,7 +1952,7 @@ bind_top_applet_events(GtkWidget *widget, int bind_lower)
 
 	if (bind_lower && GTK_IS_CONTAINER(widget))
 		gtk_container_foreach (GTK_CONTAINER (widget),
-				       bind_applet_events, 0);
+				       bind_applet_events, widget);
 }
 
 static int
@@ -2119,7 +2104,13 @@ panel_widget_reparent (PanelWidget *old_panel,
 	ad->pos = pos;
 
 	/*reparent applet*/
-	gtk_widget_reparent(applet,GTK_WIDGET(new_panel));
+	if (IS_BUTTON_WIDGET (applet)) {
+		gtk_widget_ref (applet);
+		gtk_container_remove (GTK_CONTAINER (applet->parent), applet);
+		gtk_container_add (GTK_CONTAINER (new_panel), applet);
+		gtk_widget_unref (applet);
+	} else 
+		gtk_widget_reparent(applet,GTK_WIDGET(new_panel));
 
 	return pos;
 }
