@@ -15,6 +15,7 @@
 #include <gnome.h>
 
 #include "panel-include.h"
+#include "gnome-panel.h"
 
 #define PANEL_EVENT_MASK (GDK_BUTTON_PRESS_MASK |		\
 			   GDK_BUTTON_RELEASE_MASK |		\
@@ -37,25 +38,31 @@ extern int panels_to_sync;
 extern int globals_to_sync;
 extern int need_complete_save;
 
-extern GArray *applets;
-extern int applet_count;
-
-extern char *panel_cfg_path;
-extern char *old_panel_cfg_path;
-
 extern GtkTooltips *panel_tooltips;
 
-extern GnomeClient *client;
-
-extern GlobalConfig global_config;
-
 /*???? this might be ugly, but I guess we can safely assume that we can only
   have one menu open and that nothing weird will happen to the panel that
   opened that menu whilethe user is looking over the choices*/
 extern PanelWidget *current_panel;
 
 /*the types of stuff we accept*/
-static char *panel_drop_types[] = {"url:ALL", "application/x-color"};
+
+enum {
+	TARGET_URL,
+	TARGET_DIRECTORY,
+	TARGET_COLOR,
+	TARGET_APPLET
+};
+
+static GtkTargetEntry panel_drop_types[] = {
+	{ "text/uri-list",       0, TARGET_URL },
+	{ "application/x-panel-directory", 0, TARGET_DIRECTORY },
+	{ "application/x-panel-applet", 0, TARGET_APPLET },
+	{ "application/x-color", 0, TARGET_COLOR }
+};
+
+static gint n_panel_drop_types = 
+   sizeof(panel_drop_types) / sizeof(panel_drop_types[0]);
 
 /*get the default panel widget if the panel has more then one or
   just get the that one*/
@@ -205,28 +212,37 @@ change_window_cursor(GdkWindow *window, GdkCursorType cursor_type)
 static void
 panel_realize(GtkWidget *widget, gpointer data)
 {
-	change_window_cursor(widget->window, GDK_ARROW);
+	change_window_cursor(widget->window, GDK_LEFT_PTR);
 	
 	if(IS_SNAPPED_WIDGET(widget))
 		snapped_widget_enable_buttons(SNAPPED_WIDGET(widget));
 	else if(IS_CORNER_WIDGET(widget))
 		corner_widget_enable_buttons(CORNER_WIDGET(widget));
+	/*FIXME: this seems to fix the panel size problems on startup
+	  (from a report) but I don't think it's right*/
+	gtk_widget_queue_resize(GTK_WIDGET(widget));
 }
 
 /*we call this recursively*/
 static void orient_change_foreach(GtkWidget *w, gpointer data);
 
 void
-orientation_change(int applet_id, PanelWidget *panel)
+orientation_change(AppletInfo *info, PanelWidget *panel)
 {
-	AppletInfo *info = get_applet_info(applet_id);
 	if(info->type == APPLET_EXTERN) {
 		Extern *ext = info->data;
 		g_assert(ext);
 		/*ingore this until we get an ior*/
-		if(ext->ior)
-			send_applet_change_orient(ext->ior,info->applet_id,
-						  get_applet_orient(panel));
+		if(ext->applet) {
+			CORBA_Environment ev;
+			CORBA_exception_init(&ev);
+			GNOME_Applet_change_orient(ext->applet,
+						   get_applet_orient(panel),
+						   &ev);
+			if(ev._major)
+				panel_clean_applet(ext->info);
+			CORBA_exception_free(&ev);
+		}
 	} else if(info->type == APPLET_MENU) {
 		Menu *menu = info->data;
 		set_menu_applet_orient(menu,get_applet_orient(panel));
@@ -251,11 +267,10 @@ orientation_change(int applet_id, PanelWidget *panel)
 static void
 orient_change_foreach(GtkWidget *w, gpointer data)
 {
-	int applet_id = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w),
-							    "applet_id"));
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(w), "applet_info");
 	PanelWidget *panel = data;
 	
-	orientation_change(applet_id,panel);
+	orientation_change(info,panel);
 }
 
 
@@ -301,19 +316,30 @@ corner_pos_change(GtkWidget *widget,
 }
 
 void
-back_change(int applet_id,
-	    PanelWidget *panel)
+back_change(AppletInfo *info, PanelWidget *panel)
 {
   
-	AppletInfo *info = get_applet_info(applet_id);
 	if(info->type == APPLET_EXTERN) {
 		Extern *ext = info->data;
 		g_assert(ext);
 		/*ignore until we have a valid IOR*/
-		if(ext->ior)
-			send_applet_change_back(ext->ior, info->applet_id,
-						panel->back_type,panel->back_pixmap,
-						&panel->back_color);
+		if(ext->applet) {
+			GNOME_Panel_BackInfoType backing;
+			CORBA_Environment ev;
+			CORBA_exception_init(&ev);
+			backing._d = panel->back_type;
+			if(panel->back_type == PANEL_BACK_PIXMAP)
+				backing._u.pmap = panel->back_pixmap;
+			else if(panel->back_type == PANEL_BACK_COLOR) {
+				backing._u.c.red = panel->back_color.red;
+				backing._u.c.green = panel->back_color.green;
+				backing._u.c.blue = panel->back_color.blue;
+			}
+			GNOME_Applet_back_change(ext->applet, &backing, &ev);
+			if(ev._major)
+				panel_clean_applet(ext->info);
+			CORBA_exception_free(&ev);
+		}
 	} 
 }
 
@@ -321,11 +347,10 @@ back_change(int applet_id,
 static void
 back_change_foreach(GtkWidget *w, gpointer data)
 {
-	int applet_id = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w),
-							    "applet_id"));
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(w), "applet_info");
 	PanelWidget *panel = data;
 
-	back_change(applet_id,panel);
+	back_change(info,panel);
 }
 
 static void
@@ -346,9 +371,7 @@ static void state_hide_foreach(GtkWidget *w, gpointer data);
 static void
 state_restore_foreach(GtkWidget *w, gpointer data)
 {
-	int applet_id = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w),
-							    "applet_id"));
-	AppletInfo *info = get_applet_info(applet_id);
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(w), "applet_info");
 	
 	if(info->type == APPLET_DRAWER) {
 		Drawer *drawer = info->data;
@@ -370,9 +393,7 @@ state_restore_foreach(GtkWidget *w, gpointer data)
 static void
 state_hide_foreach(GtkWidget *w, gpointer data)
 {
-	int applet_id = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w),
-							    "applet_id"));
-	AppletInfo *info = get_applet_info(applet_id);
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(w), "applet_info");
 
 	if(info->type == APPLET_DRAWER) {
 		Drawer *drawer = info->data;
@@ -458,11 +479,10 @@ panel_size_allocate(GtkWidget *widget, GtkAllocation *alloc, gpointer data)
 static int
 panel_applet_added_idle(gpointer data)
 {
-	int applet_id = GPOINTER_TO_INT(data);
-	AppletInfo *info = get_applet_info(applet_id);
+	AppletInfo *info = data;
 	PanelWidget *panel = PANEL_WIDGET(info->widget->parent);
 
-	orientation_change(applet_id,panel);
+	orientation_change(info,panel);
 
 	return FALSE;
 }
@@ -470,9 +490,8 @@ panel_applet_added_idle(gpointer data)
 static void
 panel_applet_added(GtkWidget *widget, GtkWidget *applet, gpointer data)
 {
-	int applet_id = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(applet),
-							    "applet_id"));
-	AppletInfo *info = get_applet_info(applet_id);
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(applet),
+					       "applet_info");
 	GtkWidget *panelw = gtk_object_get_data(GTK_OBJECT(widget),
 						PANEL_PARENT);
 	
@@ -497,20 +516,18 @@ panel_applet_added(GtkWidget *widget, GtkWidget *applet, gpointer data)
 		snapped_widget_queue_pop_down(SNAPPED_WIDGET(panelw));
 	}
 
-	gtk_idle_add(panel_applet_added_idle,GINT_TO_POINTER(applet_id));
+	gtk_idle_add(panel_applet_added_idle,info);
 
 	/*we will need to save this applet's config now*/
-	if(g_list_find(applets_to_sync, GINT_TO_POINTER(applet_id))==NULL)
-		applets_to_sync = g_list_prepend(applets_to_sync,
-						 GINT_TO_POINTER(applet_id));
+	if(g_list_find(applets_to_sync, info)==NULL)
+		applets_to_sync = g_list_prepend(applets_to_sync,info);
 }
 
 static void
 count_open_drawers(GtkWidget *w, gpointer data)
 {
-	int applet_id = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(w),
-							    "applet_id"));
-	AppletInfo *info = get_applet_info(applet_id);
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(w),
+					       "applet_info");
 	int *count = data;
 	if(info->type == APPLET_DRAWER) {
 		Drawer *drawer = info->data;
@@ -524,8 +541,8 @@ panel_applet_removed(GtkWidget *widget, GtkWidget *applet, gpointer data)
 {
 	GtkWidget *parentw = gtk_object_get_data(GTK_OBJECT(widget),
 						 PANEL_PARENT);
-	int applet_id =
-		GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(applet),"applet_id"));
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(applet),
+					       "applet_info");
 
 	if(IS_SNAPPED_WIDGET(parentw)) {
 		int drawers_open = 0;
@@ -538,9 +555,8 @@ panel_applet_removed(GtkWidget *widget, GtkWidget *applet, gpointer data)
 	}
 
 	/*we will need to save this applet's config now*/
-	if(g_list_find(applets_to_sync, GINT_TO_POINTER(applet_id))==NULL)
-		applets_to_sync = g_list_prepend(applets_to_sync,
-						 GINT_TO_POINTER(applet_id));
+	if(g_list_find(applets_to_sync, info)==NULL)
+		applets_to_sync = g_list_prepend(applets_to_sync,info);
 }
 
 static void
@@ -668,13 +684,11 @@ panel_destroy(GtkWidget *widget, gpointer data)
 	if(IS_DRAWER_WIDGET(widget)) {
 		PanelWidget *panel = PANEL_WIDGET(DRAWER_WIDGET(widget)->panel);
 		if(panel->master_widget) {
-			int applet_id =
-				GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(panel->master_widget),
-								    "applet_id"));
-			AppletInfo *info = get_applet_info(applet_id);
+			AppletInfo *info = gtk_object_get_data(GTK_OBJECT(panel->master_widget),
+							       "applet_info");
 			Drawer *drawer = info->data;
 			drawer->drawer = NULL;
-			panel_clean_applet(applet_id);
+			panel_clean_applet(info);
 		}
 	} else if(IS_SNAPPED_WIDGET(widget) ||
 		  IS_CORNER_WIDGET(widget)) {
@@ -694,18 +708,17 @@ panel_destroy(GtkWidget *widget, gpointer data)
 static void
 panel_applet_move(GtkWidget *panel,GtkWidget *widget, gpointer data)
 {
-	int applet_id = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(widget),"applet_id"));
+	AppletInfo *info = gtk_object_get_data(GTK_OBJECT(widget),
+					       "applet_info");
 	
-	if(g_list_find(applets_to_sync, GINT_TO_POINTER(applet_id))==NULL)
-		applets_to_sync = g_list_prepend(applets_to_sync,
-						 GINT_TO_POINTER(applet_id));
+	if(g_list_find(applets_to_sync, info)==NULL)
+		applets_to_sync = g_list_prepend(applets_to_sync,info);
 }
 
 static int
 panel_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
 	GdkEventButton *bevent;
-
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		bevent = (GdkEventButton *) event;
@@ -820,7 +833,7 @@ panel_sub_event_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 	switch (event->type) {
 		/*pass these to the parent!*/
 		case GDK_BUTTON_PRESS:
-		case GDK_BUTTON_RELEASE:
+	        case GDK_BUTTON_RELEASE:
 			bevent = (GdkEventButton *) event;
 			/*if the widget is a button we want to keep the
 			  button 1 events*/
@@ -837,76 +850,94 @@ panel_sub_event_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 }
 
 
-static void
-bind_panel_events(GtkWidget *widget, gpointer data)
-{
-	if (!GTK_WIDGET_NO_WINDOW(widget))
-		gtk_signal_connect(GTK_OBJECT(widget), "event",
-				   (GtkSignalFunc) panel_sub_event_handler,
-				   NULL);
+/* static void
+bind_panel_events(GtkWidget *widget, gpointer data) 
+{ 
+	if (!GTK_WIDGET_NO_WINDOW(widget)) 
+		gtk_signal_connect(GTK_OBJECT(widget), "event", 
+				   (GtkSignalFunc) panel_sub_event_handler, 
+				   NULL); 
 	
-	if (GTK_IS_CONTAINER(widget))
-		gtk_container_foreach (GTK_CONTAINER (widget),
-				       bind_panel_events, NULL);
-}
+	if (GTK_IS_CONTAINER(widget)) 
+		gtk_container_foreach (GTK_CONTAINER (widget), 
+				       bind_panel_events, NULL); 
+} */
 
 static void
-panel_widget_dnd_drop_internal(GtkWidget *widget,
-			       GdkEventDropDataAvailable *event,
-			       gpointer data)
+panel_widget_dnd_drop_internal (GtkWidget *widget,
+				GdkDragContext   *context,
+				gint              x,
+				gint              y,
+				GtkSelectionData *selection_data,
+				guint             info,
+				guint             time)
 {
 	PanelWidget *panel;
 
 	g_return_if_fail(widget!=NULL);
 	g_return_if_fail(IS_PANEL_WIDGET(widget));
-	g_return_if_fail(event!=NULL);
 
-	panel = PANEL_WIDGET(widget);
+	panel = PANEL_WIDGET (widget);
 
-	/* Test for the type that was dropped */
-	if (strcmp (event->data_type, "url:ALL") == 0) {
-		char *file = event->data;
+	switch (info) {
+	case TARGET_URL: {
+		GList *ltmp, *files;
 		struct stat s;
+		int pos = panel_widget_get_cursorloc(panel);
 
-		/*if we can't stat this file don't even bother loading it*/
-		if(stat(file,&s)==0) {
-			char *p = strrchr(file,'.');
-			int pos = panel_widget_get_cursorloc(panel);
-			if(S_ISDIR(s.st_mode)) /*add a menu*/
-				load_menu_applet(file,MAIN_MENU_BOTH,
-						 panel,pos);
-			else if(p && strcmp(p,".desktop")==0)
-				load_launcher_applet(file,panel,pos);
-			else if(S_IEXEC & s.st_mode) /*executable?*/
-				ask_about_launcher(file,panel,pos);
-			else /*if all else fails it might be a pixmap*/
-				panel_widget_set_back_pixmap (panel, file);
+		files =
+		  gnome_uri_list_extract_filenames(selection_data->data);
+
+		for(ltmp = files; ltmp; ltmp = g_list_next(ltmp)) {
+		  char *mimetype, *p;
+
+		  if(stat(ltmp->data, &s) != 0)
+		    continue;
+
+		  mimetype = gnome_mime_type(ltmp->data);
+		  if(mimetype && !strncmp(mimetype, "image", sizeof("image")-1))
+		    panel_widget_set_back_pixmap (panel, ltmp->data);
+		  else if(mimetype
+			  && !strcmp(mimetype, "application/x-gnome-app-info"))
+		    load_launcher_applet(ltmp->data, panel, pos);
+		  else if(S_ISDIR(s.st_mode))
+		    load_menu_applet(ltmp->data, MAIN_MENU_BOTH,
+				     panel, pos);
+		  else if(S_IEXEC & s.st_mode) /*executable?*/
+		    ask_about_launcher(ltmp->data,panel,pos);
 		}
-	} else if(!strcmp(event->data_type, "application/x-color")) {
-		gdouble *dropped;
+		
+		gnome_uri_list_free_strings (files);
+		break;
+	}
+	case TARGET_COLOR: {
+		guint16 *dropped;
 		GdkColor c;
 
-		dropped = (gdouble *)event->data;
+		dropped = (guint16 *)selection_data->data;
 
-		c.red = (dropped[1]*65535);
-		c.green = (dropped[2]*65535);
-		c.blue = (dropped[3]*65535);
+		c.red = dropped[0];
+		c.green = dropped[1];
+		c.blue = dropped[2];
 		c.pixel = 0;
 
 		panel_widget_set_back_color(panel, &c);
+		break;
 	}
-}
+	case TARGET_DIRECTORY: {
+		int pos = panel_widget_get_cursorloc(panel);
+		load_menu_applet ((char *)selection_data->data, 0,
+				  panel, pos);
+		break;
+	}
+	case TARGET_APPLET: {
+		int pos = panel_widget_get_cursorloc(panel);
+		char *goad_id = (char *)selection_data->data;
 
-static void
-panelw_realize(PanelWidget *panel, gpointer data)
-{
-	g_return_if_fail(panel!=NULL);
-	g_return_if_fail(IS_PANEL_WIDGET(panel));
-
-	gtk_widget_dnd_drop_set (GTK_WIDGET(panel), TRUE,
-				 panel_drop_types,
-				 sizeof(panel_drop_types)/sizeof(char *),
-				 FALSE);
+		load_extern_applet(goad_id,NULL,panel,pos);
+		break;
+	}
+	}
 }
 
 static void
@@ -928,16 +959,17 @@ panel_widget_setup(PanelWidget *panel)
 			   "back_change",
 			   GTK_SIGNAL_FUNC(panel_back_change),
 			   NULL);
-	gtk_signal_connect_after(GTK_OBJECT(panel),
-				 "realize",
-				 GTK_SIGNAL_FUNC(panelw_realize),
-				 NULL);
+	gtk_signal_connect(GTK_OBJECT(panel),
+			   "drag_data_received",
+			   GTK_SIGNAL_FUNC(panel_widget_dnd_drop_internal),
+			   NULL);
 
-	/* Ok, cool hack begins: drop image files on the panel */
-	gtk_signal_connect (GTK_OBJECT (panel),
-			    "drop_data_available_event",
-			    GTK_SIGNAL_FUNC (panel_widget_dnd_drop_internal),
-			    NULL);
+	gtk_drag_dest_set (GTK_WIDGET (panel),
+			   GTK_DEST_DEFAULT_MOTION |
+			   GTK_DEST_DEFAULT_HIGHLIGHT |
+			   GTK_DEST_DEFAULT_DROP,
+			   panel_drop_types, n_panel_drop_types,
+			   GDK_ACTION_COPY);
 }
 
 void
@@ -967,6 +999,7 @@ panel_setup(GtkWidget *panelw)
 	if(IS_DRAWER_WIDGET(panelw)) {
 		PanelWidget *panel = PANEL_WIDGET(DRAWER_WIDGET(panelw)->panel);
 		DrawerWidget *drawer = DRAWER_WIDGET(panelw);
+		GtkObject *button;
 		panel_widget_setup(panel);
 		gtk_signal_connect(GTK_OBJECT(panel),
 				   "orient_change",
@@ -979,15 +1012,31 @@ panel_setup(GtkWidget *panelw)
 		gtk_signal_connect(GTK_OBJECT(drawer->handle_e), "event",
 				   (GtkSignalFunc) panel_sub_event_handler,
 				   panelw);
+		button = gtk_object_get_data(GTK_OBJECT(drawer->handle_e),"button");
+		if (button) gtk_signal_connect(button, "event",
+					       (GtkSignalFunc) panel_sub_event_handler,
+					       panelw);
 		gtk_signal_connect(GTK_OBJECT(drawer->handle_w), "event",
 				   (GtkSignalFunc) panel_sub_event_handler,
 				   panelw);
+		button = gtk_object_get_data(GTK_OBJECT(drawer->handle_w),"button");
+		if (button) gtk_signal_connect(button, "event",
+					       (GtkSignalFunc) panel_sub_event_handler,
+					       panelw);
 		gtk_signal_connect(GTK_OBJECT(drawer->handle_n), "event",
 				   (GtkSignalFunc) panel_sub_event_handler,
 				   panelw);
+		button = gtk_object_get_data(GTK_OBJECT(drawer->handle_n),"button");
+		if (button) gtk_signal_connect(button, "event",
+					       (GtkSignalFunc) panel_sub_event_handler,
+					       panelw);
 		gtk_signal_connect(GTK_OBJECT(drawer->handle_s), "event",
 				   (GtkSignalFunc) panel_sub_event_handler,
 				   panelw);
+		button = gtk_object_get_data(GTK_OBJECT(drawer->handle_s),"button");
+		if (button) gtk_signal_connect(button, "event",
+					       (GtkSignalFunc) panel_sub_event_handler,
+					       panelw);
 	} else if(IS_SNAPPED_WIDGET(panelw)) {
 		PanelWidget *panel =
 			PANEL_WIDGET(SNAPPED_WIDGET(panelw)->panel);
