@@ -8,50 +8,6 @@
 /* Used for all the packing and padding options */
 #define CONFIG_PADDING_SIZE 3
 
-extern GArray *applets;
-extern int applet_count;
-
-extern GlobalConfig global_config;
-
-/* used to temporarily store config values until the 'Apply'
-   button is pressed. */
-typedef struct _PerPanelConfig PerPanelConfig;
-struct _PerPanelConfig {
-	GtkWidget		*panel;
-
-	/*drawer types*/
-	/*nothing!*/
-	
-	/*snapped types*/
-	SnappedPos		snapped_pos;
-	SnappedMode		snapped_mode;
-	int			snapped_hidebuttons;
-	
-	/*corner types*/
-	CornerPos		corner_pos;
-	PanelOrientation	corner_orient;
-
-	int			fit_pixmap_bg;
-	PanelBackType		back_type;
-	char			*back_pixmap;
-	GdkColor		back_color;
-	
-	int			register_changes; /*used for startup*/
-	GtkWidget		*config_window;
-	GtkWidget		*pix_entry;
-	int			pix_ch_signal;
-
-	/*snapped buttons*/
-	GtkWidget		*r_button; /*se*/
-	GtkWidget		*l_button; /*nw*/
-	GtkWidget		*t_button; /*ne*/
-	GtkWidget		*b_button; /*sw*/
-	GtkWidget		*non;
-	GtkWidget		*pix;
-	GtkWidget		*col;
-	GtkWidget		*backsel;
-};
-
 static GList *ppconfigs=NULL;
 
 static PerPanelConfig *
@@ -145,7 +101,7 @@ update_config_back(PanelWidget *pw)
 					    TRUE);
 		break;
 	case PANEL_BACK_PIXMAP:
-		t=gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(ppc->pix_entry));
+		t=gnome_pixmap_entry_gtk_entry(GNOME_PIXMAP_ENTRY(ppc->pix_entry));
 		gtk_entry_set_text(GTK_ENTRY(t),pw->back_pixmap);
 		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ppc->pix),
 					    TRUE);
@@ -157,11 +113,6 @@ static int
 config_destroy(GtkWidget *widget, gpointer data)
 {
 	PerPanelConfig *ppc = data;
-	GtkWidget *t;
-	
-	t=gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(ppc->pix_entry));
-	gtk_signal_disconnect(GTK_OBJECT(t),
-			      ppc->pix_ch_signal);
 	
 	ppconfigs = g_list_remove(ppconfigs,ppc);
 	
@@ -182,6 +133,7 @@ config_apply (GtkWidget *widget, int page, gpointer data)
 					     ppc->snapped_mode,
 					     SNAPPED_WIDGET(ppc->panel)->state,
 					     ppc->snapped_hidebuttons,
+					     ppc->snapped_hidebutton_pixmaps,
 					     ppc->back_type,
 					     ppc->back_pixmap,
 					     ppc->fit_pixmap_bg,
@@ -191,19 +143,23 @@ config_apply (GtkWidget *widget, int page, gpointer data)
 					    ppc->corner_pos,
 					    ppc->corner_orient,
 					    CORNER_WIDGET(ppc->panel)->state,
+					    ppc->corner_hidebuttons,
+					    ppc->corner_hidebutton_pixmaps,
 					    ppc->back_type,
 					    ppc->back_pixmap,
 					    ppc->fit_pixmap_bg,
 					    &ppc->back_color);
 	else if(IS_DRAWER_WIDGET(ppc->panel)) {
-		PanelWidget *pw =
-			PANEL_WIDGET(DRAWER_WIDGET(ppc->panel)->panel);
-		panel_widget_change_params(pw,
-					   pw->orient,
+	        DrawerWidget *dw = DRAWER_WIDGET(ppc->panel);
+		drawer_widget_change_params(dw,
+					   dw->orient,
+					   dw->state, 
 					   ppc->back_type,
 					   ppc->back_pixmap,
 					   ppc->fit_pixmap_bg,
-					   &ppc->back_color);
+					   &ppc->back_color,
+					   ppc->drawer_hidebutton_pixmap,
+					   ppc->drawer_hidebutton);
 	}
 	gtk_widget_queue_draw (ppc->panel);
 }
@@ -249,6 +205,16 @@ config_event(GtkWidget *widget,GdkEvent *event,GtkNotebook *nbook)
 	return FALSE;
 }
 
+static void
+set_toggle_not (GtkWidget *widget, gpointer data)
+{
+	PerPanelConfig *ppc = gtk_object_get_user_data(GTK_OBJECT(widget));
+	int *the_toggle = data;
+
+	*the_toggle = !(GTK_TOGGLE_BUTTON(widget)->active);
+	if (ppc->register_changes)
+		gnome_property_box_changed (GNOME_PROPERTY_BOX (ppc->config_window));
+}
 
 static void
 snapped_set_pos (GtkWidget *widget, gpointer data)
@@ -264,16 +230,6 @@ snapped_set_pos (GtkWidget *widget, gpointer data)
 		gnome_property_box_changed (GNOME_PROPERTY_BOX (ppc->config_window));
 }
 
-static void
-snapped_set_hidebuttons (GtkWidget *widget, gpointer data)
-{
-	PerPanelConfig *ppc = gtk_object_get_user_data(GTK_OBJECT(widget));
-
-	ppc->snapped_hidebuttons = !(GTK_TOGGLE_BUTTON(widget)->active);
-	if (ppc->register_changes)
-		gnome_property_box_changed (GNOME_PROPERTY_BOX (ppc->config_window));
-}
-
 static void
 snapped_set_mode (GtkWidget *widget, gpointer data)
 {
@@ -300,11 +256,11 @@ snapped_notebook_page(PerPanelConfig *ppc)
 	
 	/* main hbox */
 	hbox = gtk_hbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (hbox), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (hbox), CONFIG_PADDING_SIZE);
 	
 	/* Position frame */
 	frame = gtk_frame_new (_("Position"));
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
 	
@@ -312,7 +268,7 @@ snapped_notebook_page(PerPanelConfig *ppc)
 	table = gtk_table_new(3, 3, TRUE);
 	gtk_table_set_row_spacings(GTK_TABLE(table), CONFIG_PADDING_SIZE);
 	gtk_table_set_col_spacings(GTK_TABLE(table), CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER(table), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER(table), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (frame), table);
 	
 	/* Top Position */
@@ -378,13 +334,13 @@ snapped_notebook_page(PerPanelConfig *ppc)
 
 	/* Auto-hide/stayput frame */
 	frame = gtk_frame_new (_("Minimize Options"));
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE,
 			    CONFIG_PADDING_SIZE);
 
 	/* vbox for frame */
 	box = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (frame), box);
 	
 	/* Stay Put */
@@ -411,13 +367,25 @@ snapped_notebook_page(PerPanelConfig *ppc)
 	gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE,
 			    CONFIG_PADDING_SIZE);
 
-	/* Auto-hide */
+	/* Hidebuttons disable */
 	button = gtk_check_button_new_with_label (_("Disable hidebuttons"));
 	gtk_object_set_user_data(GTK_OBJECT(button),ppc);
 	if (!ppc->snapped_hidebuttons)
 		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
 	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
-			    GTK_SIGNAL_FUNC (snapped_set_hidebuttons), NULL);
+			    GTK_SIGNAL_FUNC (set_toggle_not),
+			    &ppc->snapped_hidebuttons);
+	gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE,
+			    CONFIG_PADDING_SIZE);
+
+	/* Arrow disable */
+	button = gtk_check_button_new_with_label (_("Disable hidebutton arrows"));
+	gtk_object_set_user_data(GTK_OBJECT(button),ppc);
+	if (!ppc->snapped_hidebutton_pixmaps)
+		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
+	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
+			    GTK_SIGNAL_FUNC (set_toggle_not),
+			    &ppc->snapped_hidebutton_pixmaps);
 	gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE,
 			    CONFIG_PADDING_SIZE);
 
@@ -452,6 +420,7 @@ corner_set_orient (GtkWidget *widget, gpointer data)
 		gnome_property_box_changed (GNOME_PROPERTY_BOX (ppc->config_window));
 }
 
+
 static GtkWidget *
 corner_notebook_page(PerPanelConfig *ppc)
 {
@@ -459,15 +428,16 @@ corner_notebook_page(PerPanelConfig *ppc)
 	GtkWidget *button;
 	GtkWidget *box;
 	GtkWidget *hbox;
+	GtkWidget *vbox;
         GtkWidget *table;
 	
 	/* main hbox */
 	hbox = gtk_hbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (hbox), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (hbox), CONFIG_PADDING_SIZE);
 	
 	/* Position frame */
 	frame = gtk_frame_new (_("Position"));
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
 	
@@ -475,7 +445,7 @@ corner_notebook_page(PerPanelConfig *ppc)
 	table = gtk_table_new(3, 3, TRUE);
 	gtk_table_set_row_spacings(GTK_TABLE(table), CONFIG_PADDING_SIZE);
 	gtk_table_set_col_spacings(GTK_TABLE(table), CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER(table), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER(table), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (frame), table);
 	
 	/* North East Position */
@@ -539,15 +509,19 @@ corner_notebook_page(PerPanelConfig *ppc)
 		break;
 	}
 
+	vbox = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
+	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE,
+			    CONFIG_PADDING_SIZE);
+
 	/* Orientation frame */
 	frame = gtk_frame_new (_("Orientation"));
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
-	gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE,
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE,
 			    CONFIG_PADDING_SIZE);
 
 	/* vbox for frame */
 	box = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (frame), box);
 	
 	/* Horizontal */
@@ -574,6 +548,40 @@ corner_notebook_page(PerPanelConfig *ppc)
 	gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE,
 			    CONFIG_PADDING_SIZE);
 
+	/* Hidebuttons frame */
+	frame = gtk_frame_new (_("Hidebutton Options"));
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE,
+			    CONFIG_PADDING_SIZE);
+
+	/* vbox for frame */
+	box = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_add (GTK_CONTAINER (frame), box);
+	
+
+	/* Hidebuttons disable */
+	button = gtk_check_button_new_with_label (_("Disable hidebuttons"));
+	gtk_object_set_user_data(GTK_OBJECT(button),ppc);
+	if (!ppc->corner_hidebuttons)
+		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
+	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
+			    GTK_SIGNAL_FUNC (set_toggle_not),
+			    &ppc->corner_hidebuttons);
+	gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE,
+			    CONFIG_PADDING_SIZE);
+
+	/* Arrow disable */
+	button = gtk_check_button_new_with_label (_("Disable hidebutton arrows"));
+	gtk_object_set_user_data(GTK_OBJECT(button),ppc);
+	if (!ppc->corner_hidebutton_pixmaps)
+		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
+	gtk_signal_connect (GTK_OBJECT (button), "toggled", 
+			    GTK_SIGNAL_FUNC (set_toggle_not),
+			    &ppc->corner_hidebutton_pixmaps);
+	gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE,
+			    CONFIG_PADDING_SIZE);
+
 	return (hbox);
 }
 
@@ -584,7 +592,7 @@ value_changed (GtkWidget *w, gpointer data)
 	PerPanelConfig *ppc = data;
 
 	g_free(ppc->back_pixmap);
-	ppc->back_pixmap = g_strdup(gtk_entry_get_text(GTK_ENTRY(w)));
+	ppc->back_pixmap = gnome_pixmap_entry_get_filename(GNOME_PIXMAP_ENTRY(ppc->pix_entry));
 	if (ppc->register_changes)
 		gnome_property_box_changed (GNOME_PROPERTY_BOX (ppc->config_window));
 	return FALSE;
@@ -653,16 +661,16 @@ background_page (PerPanelConfig *ppc)
 	GtkWidget *w;
 
 	vbox = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (vbox), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (vbox), CONFIG_PADDING_SIZE);
 
 	/*selector frame*/
 	f = gtk_frame_new (_("Background"));
-	gtk_container_border_width(GTK_CONTAINER (f), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (f), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (vbox), f, FALSE, FALSE, 
 			    CONFIG_PADDING_SIZE);
 
 	box = gtk_hbox_new (0, 0);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (f), box);
 	
 	/*standard background*/
@@ -697,21 +705,20 @@ background_page (PerPanelConfig *ppc)
 	gtk_object_set_data(GTK_OBJECT(ppc->pix),"pix",f);
 	gtk_object_set_data(GTK_OBJECT(ppc->col),"pix",f);
 	gtk_object_set_data(GTK_OBJECT(ppc->non),"pix",f);
-	gtk_container_border_width(GTK_CONTAINER (f), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (f), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (vbox), f, FALSE, FALSE, 
 			    CONFIG_PADDING_SIZE);
 
 	box = gtk_vbox_new (0, 0);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (f), box);
 
-	ppc->pix_entry = gnome_file_entry_new ("pixmap", _("Browse"));
+	ppc->pix_entry = gnome_pixmap_entry_new ("pixmap", _("Browse"),TRUE);
 
-
-	t = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (ppc->pix_entry));
-	ppc->pix_ch_signal = 
-		gtk_signal_connect (GTK_OBJECT (t), "changed",
-				    GTK_SIGNAL_FUNC (value_changed), ppc);
+	t = gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (ppc->pix_entry));
+	gtk_signal_connect_while_alive (GTK_OBJECT (t), "changed",
+					GTK_SIGNAL_FUNC (value_changed), ppc,
+					GTK_OBJECT (ppc->pix_entry));
 	gtk_box_pack_start (GTK_BOX (box), ppc->pix_entry, FALSE, FALSE, 
 			    CONFIG_PADDING_SIZE);
 	
@@ -741,12 +748,12 @@ background_page (PerPanelConfig *ppc)
 	gtk_object_set_data(GTK_OBJECT(ppc->pix),"col",f);
 	gtk_object_set_data(GTK_OBJECT(ppc->col),"col",f);
 	gtk_object_set_data(GTK_OBJECT(ppc->non),"col",f);
-	gtk_container_border_width(GTK_CONTAINER (f), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (f), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (box), f, FALSE, FALSE, 
 			    CONFIG_PADDING_SIZE);
 
 	box = gtk_vbox_new (0, 0);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (f), box);
 
 	ppc->backsel = gnome_color_picker_new();
@@ -807,6 +814,7 @@ panel_config(GtkWidget *panel)
 		ppc->snapped_pos = snapped->pos;
 		ppc->snapped_mode = snapped->mode;
 		ppc->snapped_hidebuttons = snapped->hidebuttons_enabled;
+		ppc->snapped_hidebutton_pixmaps = snapped->hidebutton_pixmaps_enabled;
 		ppc->fit_pixmap_bg = pw->fit_pixmap_bg;
 		ppc->back_pixmap = g_strdup(pw->back_pixmap);
 		ppc->back_color = pw->back_color;
@@ -816,16 +824,21 @@ panel_config(GtkWidget *panel)
 		PanelWidget *pw = PANEL_WIDGET(CORNER_WIDGET(panel)->panel);
 		ppc->corner_pos = corner->pos;
 		ppc->corner_orient = pw->orient;
+		ppc->corner_hidebuttons = corner->hidebuttons_enabled;
+		ppc->corner_hidebutton_pixmaps = corner->hidebutton_pixmaps_enabled;
 		ppc->fit_pixmap_bg = pw->fit_pixmap_bg;
 		ppc->back_pixmap = g_strdup(pw->back_pixmap);
 		ppc->back_color = pw->back_color;
 		ppc->back_type = pw->back_type;
 	} else if(IS_DRAWER_WIDGET(panel)) {
-		PanelWidget *pw = PANEL_WIDGET(DRAWER_WIDGET(panel)->panel);
+                DrawerWidget *drawer = DRAWER_WIDGET(panel);
+	        PanelWidget *pw = PANEL_WIDGET(DRAWER_WIDGET(panel)->panel);
 		ppc->fit_pixmap_bg = pw->fit_pixmap_bg;
 		ppc->back_pixmap = g_strdup(pw->back_pixmap);
 		ppc->back_color = pw->back_color;
 		ppc->back_type = pw->back_type;
+		ppc->drawer_hidebutton = drawer->hidebutton_enabled;
+		ppc->drawer_hidebutton_pixmap = drawer->hidebutton_pixmap_enabled;
 	}
 	
 	ppc->panel = panel;
@@ -842,7 +855,7 @@ panel_config(GtkWidget *panel)
 			   GTK_SIGNAL_FUNC (config_destroy), ppc);
 	gtk_window_set_title (GTK_WINDOW(ppc->config_window),
 			      _("Panel properties"));
-	gtk_container_border_width (GTK_CONTAINER(ppc->config_window),
+	gtk_container_set_border_width (GTK_CONTAINER(ppc->config_window),
 				    CONFIG_PADDING_SIZE);
 	
 	prop_nbook = GNOME_PROPERTY_BOX (ppc->config_window)->notebook;
@@ -862,12 +875,9 @@ panel_config(GtkWidget *panel)
 	} else if(IS_DRAWER_WIDGET(panel)) {
 		DrawerWidget *dw = DRAWER_WIDGET(panel);
 		GtkWidget *applet = PANEL_WIDGET(dw->panel)->master_widget;
-		int applet_id =
-			GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(applet),
-							    "applet_id"));
-		AppletInfo *info = get_applet_info(applet_id);
-		add_drawer_properties_page(ppc->config_window,
-					   info->data);
+		AppletInfo *info =
+			gtk_object_get_data(GTK_OBJECT(applet), "applet_info");
+		add_drawer_properties_page(ppc, info->data);
 	}
 						
 
