@@ -65,9 +65,9 @@ config_apply (GtkWidget *widget, int page, gpointer data)
 
 	for(i=0;i<LAST_TILE;i++) {
 		global_config.tile_up[i] =
-			g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_up[i])));
+			gnome_icon_entry_get_filename(GNOME_ICON_ENTRY(entry_up[i]));
 		global_config.tile_down[i] =
-			g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_down[i])));
+			gnome_icon_entry_get_filename(GNOME_ICON_ENTRY(entry_down[i]));
 	}
 
 	apply_global_config();
@@ -129,7 +129,8 @@ int_scale_update (GtkAdjustment *adjustment, gpointer data)
 }
 
 static GtkWidget *
-make_int_scale_frame(char *title, int *data, double min, double max)
+make_int_scale_frame(char *title, int *data,
+		     double min, double max, double step)
 {
 	GtkWidget *frame;
 	GtkWidget *box;
@@ -138,17 +139,17 @@ make_int_scale_frame(char *title, int *data, double min, double max)
 
 	/* scale frame */
 	frame = gtk_frame_new (title);
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
 
 	/* vbox for frame */
 	box = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (frame), box);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
 	gtk_widget_show (box);
 
 	/* Animation step_size scale */
 	scale_data = gtk_adjustment_new((double) (*data),
-					min, max, 1.0, 1.0, 0.0);
+					min, max, step, step, 0.0);
 	scale = gtk_hscale_new (GTK_ADJUSTMENT (scale_data));
 	gtk_range_set_update_policy (GTK_RANGE (scale), GTK_UPDATE_DELAYED);
 	gtk_scale_set_digits (GTK_SCALE (scale), 0);
@@ -188,7 +189,7 @@ animation_notebook_page(void)
 	
 	/* main vbox */
 	vbox = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (vbox), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (vbox), CONFIG_PADDING_SIZE);
 
 	/* Animation disable */
 	button = gtk_check_button_new_with_label (_("Disable animations"));
@@ -201,7 +202,7 @@ animation_notebook_page(void)
 	/* AutoHide Animation step_size scale frame */
 	frame = make_int_scale_frame(_("Auto-Hide Animation Speed"),
 				      &(temp_config.auto_hide_step_size),
-				      1.0,100.0);
+				      1.0, 100.0, 1.0);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE,
 			    CONFIG_PADDING_SIZE);
 
@@ -210,7 +211,7 @@ animation_notebook_page(void)
 	/* ExplicitHide Animation step_size scale frame */
 	frame = make_int_scale_frame(_("Explicit-Hide Animation Speed"),
 				      &(temp_config.explicit_hide_step_size),
-				      1.0,100.0);
+				      1.0, 100.0, 1.0);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE,
 			    CONFIG_PADDING_SIZE);
 
@@ -219,7 +220,7 @@ animation_notebook_page(void)
 	/* DrawerHide Animation step_size scale frame */
 	frame = make_int_scale_frame(_("Drawer Animation Speed"),
 				      &(temp_config.drawer_step_size),
-				      1.0,100.0);
+				      1.0, 100.0, 1.0);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE,
 			    CONFIG_PADDING_SIZE);
 
@@ -228,14 +229,14 @@ animation_notebook_page(void)
 	/* Minimize Delay scale frame */
 	frame = make_int_scale_frame(_("Auto-Hide Minimize Delay (ms)"),
 				      &(temp_config.minimize_delay),
-				      30.0,1000.0);
+				      30.0, 10000.0, 10.0);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE,
 			    CONFIG_PADDING_SIZE);
 
 	/* Minimized size scale frame */
 	frame = make_int_scale_frame(_("Auto-Hide Minimized Size (pixels)"),
 				      &(temp_config.minimized_size),
-				      1.0,10.0);
+				      1.0, 10.0, 1.0);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE,
 			    CONFIG_PADDING_SIZE);
 
@@ -264,55 +265,54 @@ set_movement (GtkWidget *widget, gpointer data)
 static int
 set_icon_button_value(GtkWidget *w, gpointer data)
 {
-	int i;
+	int i = GPOINTER_TO_INT(data);
 	int active = GTK_TOGGLE_BUTTON(w)->active;
 
-	for(i=0;i<3;i++) {
-		gtk_widget_set_sensitive(tilefile[i],active);
-		gtk_widget_set_sensitive(tileborder[i],active);
-	}
-	temp_config.tiles_enabled = active;
+	gtk_widget_set_sensitive(tilefile[i],active);
+	gtk_widget_set_sensitive(tileborder[i],active);
+	temp_config.tiles_enabled[i] = active; 
 
 	if(config_window)
 		gnome_property_box_changed (GNOME_PROPERTY_BOX (config_window));
 	return FALSE;
 }
 
-static GtkWidget *
-genicon_notebook_page(void)
-{
-	GtkWidget *frame;
-	GtkWidget *w;
-	GtkWidget *box;
-	GtkWidget *vbox;
-	
-	/* main vbox */
-	vbox = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (vbox), CONFIG_PADDING_SIZE);
-	
-	/* General frame */
-	frame = gtk_frame_new (_("General"));
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE,
-			    CONFIG_PADDING_SIZE);
-	
-	/* vbox for frame */
-	box = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
-	gtk_container_add (GTK_CONTAINER (frame), box);
-	
-	/* Enable tiles frame */
-	w = gtk_check_button_new_with_label (_("Tiles enabled"));
-	if (temp_config.tiles_enabled)
-		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w), TRUE);
-	gtk_signal_connect (GTK_OBJECT (w), "toggled", 
-			    GTK_SIGNAL_FUNC (set_icon_button_value), 
-			    &(temp_config.tiles_enabled));
-	gtk_box_pack_start (GTK_BOX (box), w, FALSE, FALSE,
-			    CONFIG_PADDING_SIZE);
+//static GtkWidget *
+//genicon_notebook_page(void)
+//{
+//	  GtkWidget *frame;
+//	  GtkWidget *w;
+//	  GtkWidget *box;
+//	  GtkWidget *vbox;
+//	  
+//	  /* main vbox */
+//	  vbox = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
+//	  gtk_container_set_border_width(GTK_CONTAINER (vbox), CONFIG_PADDING_SIZE);
+//	  
+//	  /* General frame */
+//	  frame = gtk_frame_new (_("General"));
+//	  gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+//	  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE,
+//			      CONFIG_PADDING_SIZE);
+//	  
+//	  /* vbox for frame */
+//	  box = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
+//	  gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+//	  gtk_container_add (GTK_CONTAINER (frame), box);
+//	  
+//	  /* Enable tiles frame */
+//	  w = gtk_check_button_new_with_label (_("Tiles enabled"));
+//	  if (temp_config.tiles_enabled[0])
+//		  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (w), TRUE);
+//	  gtk_signal_connect (GTK_OBJECT (w), "toggled", 
+//			      GTK_SIGNAL_FUNC (set_icon_button_value), 
+//			      &(temp_config.tiles_enabled[0]));
+//	  gtk_box_pack_start (GTK_BOX (box), w, FALSE, FALSE,
+//			      CONFIG_PADDING_SIZE);
+//
+//	  return (vbox);
+//}
 
-	return (vbox);
-}
 
 static GtkWidget *
 icon_notebook_page(int i, GtkWidget *config_box)
@@ -321,29 +321,40 @@ icon_notebook_page(int i, GtkWidget *config_box)
 	GtkWidget *w;
 	GtkWidget *table;
 	GtkWidget *vbox;
+	GtkWidget *toggle;
 	
 	/* main vbox */
 	vbox = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (vbox), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (vbox), CONFIG_PADDING_SIZE);
+
+	/* toggle button */
+ 	toggle = gtk_check_button_new_with_label (_("Tiles enabled"));
+	if (temp_config.tiles_enabled[i])
+		gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (toggle), TRUE);
+	gtk_signal_connect (GTK_OBJECT (toggle), "toggled", 
+			    GTK_SIGNAL_FUNC (set_icon_button_value), 
+			    GINT_TO_POINTER(i));
+	gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE,
+			    CONFIG_PADDING_SIZE);
 	
 	/* Image frame */
 	tilefile[i] = frame = gtk_frame_new (_("Image files"));
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
-	gtk_widget_set_sensitive(frame,temp_config.tiles_enabled);
+	gtk_widget_set_sensitive(frame,temp_config.tiles_enabled[i]);
 	
 	/* table for frame */
-	table = gtk_table_new(2,2,FALSE);
-	gtk_container_border_width(GTK_CONTAINER (table), CONFIG_PADDING_SIZE);
+	table = gtk_table_new(3,3,FALSE);
+	gtk_container_set_border_width(GTK_CONTAINER (table), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (frame), table);
 	
 	/* image file entry widgets */
-	entry_up[i] = create_file_entry(table,"tile_file",0,
+	entry_up[i] = create_icon_entry(table,"tile_file",1,
 					_("Tile filename (up)"),
 					global_config.tile_up[i],
 					config_box);
-	entry_down[i] = create_file_entry(table,"tile_file",1,
+	entry_down[i] = create_icon_entry(table,"tile_file",2,
 					  _("Tile filename (down)"),
 					  global_config.tile_down[i],
 					  config_box);
@@ -353,15 +364,15 @@ icon_notebook_page(int i, GtkWidget *config_box)
 	/* Minimized size scale frame */
 	tileborder[i] = w = make_int_scale_frame(_("Border width (tile only)"),
 						 &(temp_config.tile_border[i]),
-						 0.0,10.0);
+						 0.0, 10.0, 1.0);
 	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
-	gtk_widget_set_sensitive(w,temp_config.tiles_enabled);
+	gtk_widget_set_sensitive(w,temp_config.tiles_enabled[i]); 
 
 	/* Minimized size scale frame */
 	w = make_int_scale_frame(_("Depth (displacement when pressed)"),
 				 &(temp_config.tile_depth[i]),
-				 0.0,10.0);
+				 0.0, 10.0, 1.0);
 	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
 
@@ -378,17 +389,17 @@ misc_notebook_page(void)
 	
 	/* main vbox */
 	vbox = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (vbox), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (vbox), CONFIG_PADDING_SIZE);
 	
 	/* Tooltips frame */
 	frame = gtk_frame_new (_("Tooltips"));
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
 	
 	/* vbox for frame */
 	box = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (frame), box);
 	
 	/* Tooltips enable */
@@ -403,13 +414,13 @@ misc_notebook_page(void)
 
 	/* Menu frame */
 	frame = gtk_frame_new (_("Menus"));
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
 	
 	/* vbox for frame */
 	box = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (frame), box);
 	
 	/* Small Icons */
@@ -434,13 +445,13 @@ misc_notebook_page(void)
 
 	/* Movement frame */
 	frame = gtk_frame_new (_("Movement"));
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
 	
 	/* vbox for frame */
 	box = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (frame), box);
 
 	/* Switched */
@@ -467,13 +478,13 @@ misc_notebook_page(void)
 	
 	/* Logout frame */
 	frame = gtk_frame_new (_("Log Out"));
-	gtk_container_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (frame), CONFIG_PADDING_SIZE);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
 	
 	/* vbox for frame */
 	box = gtk_vbox_new (FALSE, CONFIG_PADDING_SIZE);
-	gtk_container_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width(GTK_CONTAINER (box), CONFIG_PADDING_SIZE);
 	gtk_container_add (GTK_CONTAINER (frame), box);
 	
 	/* Prompt before log out */
@@ -489,7 +500,7 @@ misc_notebook_page(void)
 	/* Minimize Delay scale frame */
 	frame = make_int_scale_frame(_("Applet Padding"),
 				      &(temp_config.applet_padding),
-				      0.0,10.0);
+				      0.0, 10.0, 1.0);
 	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE,
 			    CONFIG_PADDING_SIZE);
 
@@ -527,7 +538,7 @@ panel_config_global(void)
 			    GTK_SIGNAL_FUNC (config_destroy), NULL);
 	gtk_window_set_title (GTK_WINDOW(box),
 			      _("Global Panel Configuration"));
-	gtk_container_border_width (GTK_CONTAINER(box), CONFIG_PADDING_SIZE);
+	gtk_container_set_border_width (GTK_CONTAINER(box), CONFIG_PADDING_SIZE);
 	
 	prop_nbook = GNOME_PROPERTY_BOX (box)->notebook;
 
@@ -536,11 +547,11 @@ panel_config_global(void)
 	gtk_notebook_append_page (GTK_NOTEBOOK(prop_nbook),
 				  page, gtk_label_new (_("Animation")));
 
-	/* General icon notebook page */
-	page = genicon_notebook_page ();
-	gtk_notebook_append_page (GTK_NOTEBOOK(prop_nbook),
-				  page, gtk_label_new (_("General icon settings")));
-
+//	  /* General icon notebook page */
+//	  page = genicon_notebook_page ();
+//	  gtk_notebook_append_page (GTK_NOTEBOOK(prop_nbook),
+//				    page, gtk_label_new (_("General icon settings")));
+//
 	/* Specific icon notebook pages */
 	for(i = 0; i<LAST_TILE; i++) {
 		page = icon_notebook_page (i,box);
