@@ -20,7 +20,8 @@ int panels_to_sync = FALSE;
 int globals_to_sync = FALSE;
 int need_complete_save = FALSE;
 
-extern GArray *applets;
+extern GList *applets;
+extern GList *applets_list;
 extern int applet_count;
 
 extern GtkTooltips *panel_tooltips;
@@ -39,20 +40,24 @@ extern GList *panel_list;
 static void
 send_tooltips_state(int enabled)
 {
-	AppletInfo *info;
-	int i;
+	GList *li;
 
-	for(i=0,info=(AppletInfo *)applets->data;
-	    i<applet_count;
-	    i++,info++) {
+	for(li = applets; li!=NULL; li = g_list_next(li)) {
+		AppletInfo *info = li->data;
 		if(info->type == APPLET_EXTERN) {
 			Extern *ext = info->data;
 			g_assert(ext);
 			/*if it's not set yet, don't send it, it will be sent
 			  when the ior is discovered anyhow, so this would be
 			  redundant anyway*/
-			if(ext->ior)
-				send_applet_tooltips_state(ext->ior,i,enabled);
+			if(ext->applet) {
+				CORBA_Environment ev;
+				CORBA_exception_init(&ev);
+				GNOME_Applet_set_tooltips_state(ext->applet, enabled, &ev);
+				if(ev._major)
+					panel_clean_applet(ext->info);
+				CORBA_exception_free(&ev);
+			}
 		}
 	}
 }
@@ -76,8 +81,9 @@ apply_global_config(void)
 	set_show_small_icons();
 	set_show_dot_buttons();
 	send_tooltips_state(global_config.tooltips_enabled);
-	button_widget_tile_enable(global_config.tiles_enabled);
+
 	for(i=0;i<LAST_TILE;i++) {
+		button_widget_tile_enable(i, global_config.tiles_enabled[i]);
 		button_widget_load_tile(i, global_config.tile_up[i],
 					global_config.tile_down[i],
 					global_config.tile_border[i],
@@ -133,19 +139,39 @@ gnome_desktop_entry_save_no_sync (GnomeDesktopEntry *dentry)
 	gnome_config_pop_prefix ();
 }
 
+static int
+send_applet_session_save (AppletInfo *info,
+			  CORBA_Object obj,
+			  const char *cfgpath,
+			  const char *globcfgpath)
+{
+  CORBA_short retval;
+  CORBA_Environment ev;
+
+  CORBA_exception_init(&ev);
+  retval = GNOME_Applet_session_save(obj,
+				     (CORBA_char *)cfgpath,
+				     (CORBA_char *)globcfgpath, &ev);
+  if(ev._major)
+    panel_clean_applet(info);
+  CORBA_exception_free(&ev);
+
+  return retval;
+}
+
+
 
 static void
-save_applet_configuration(int num)
+save_applet_configuration(AppletInfo *info)
 {
 	char           path[256];
 	int            panel_num;
 	PanelWidget   *panel;
 	AppletData    *ad;
-	AppletInfo    *info = get_applet_info(num);
 	
 	g_return_if_fail(info!=NULL);
 
-	g_snprintf(path,256, "%sApplet_Config/Applet_%d/", panel_cfg_path, num+1);
+	g_snprintf(path,256, "%sApplet_Config/Applet_%d/", panel_cfg_path, info->applet_id+1);
 	gnome_config_push_prefix(path);
 
 	/*obviously no need for saving*/
@@ -188,16 +214,14 @@ save_applet_configuration(int num)
 			/*this is the file path we pass to the applet for it's
 			  own config, this is a separate file, so that we */
 			g_snprintf(path,256, "%sApplet_%d_Extern/",
-				   panel_cfg_path, num+1);
+				   panel_cfg_path, info->applet_id+1);
 			/*have the applet do it's own session saving*/
-			if(send_applet_session_save(ext->ior,info->applet_id,
+			if(send_applet_session_save(info,ext->applet,
 						    path, globalcfg)) {
 
 				gnome_config_set_string("id", EXTERN_ID);
-				gnome_config_set_string("execpath",
-							ext->path);
-				gnome_config_set_string("parameters",
-							ext->params);
+				gnome_config_set_string("goad_id",
+							ext->goad_id);
 			} else {
 				g_free(globalcfg);
 				gnome_config_set_string("id", EMPTY_ID);
@@ -254,7 +278,8 @@ save_applet_configuration(int num)
 			/*we set the .desktop to be in the panel config
 			  dir*/
 			g_snprintf(path,256, "%s/%sApplet_%d.desktop",
-				   gnome_user_dir,panel_cfg_path, num+1);
+				   gnome_user_dir,panel_cfg_path,
+				   info->applet_id+1);
 			g_free(launcher->dentry->location);
 			launcher->dentry->location = g_strdup(path);
 			gnome_desktop_entry_save_no_sync(launcher->dentry);
@@ -302,6 +327,8 @@ save_panel_configuration(gpointer data, gpointer user_data)
 		gnome_config_set_int("state", snapped->state);
 		gnome_config_set_bool("hidebuttons_enabled",
 				      snapped->hidebuttons_enabled);
+		gnome_config_set_bool("hidebutton_pixmaps_enabled",
+				      snapped->hidebutton_pixmaps_enabled);
 		break;
 		}
 	case CORNER_PANEL:
@@ -310,6 +337,10 @@ save_panel_configuration(gpointer data, gpointer user_data)
 		gnome_config_set_int("pos", corner->pos);
 		gnome_config_set_int("orient",panel->orient);
 		gnome_config_set_int("state", corner->state);
+		gnome_config_set_bool("hidebuttons_enabled",
+				      corner->hidebuttons_enabled);
+		gnome_config_set_bool("hidebutton_pixmaps_enabled",
+				      corner->hidebutton_pixmaps_enabled);
 		break;
 		}
 	case DRAWER_PANEL:
@@ -317,6 +348,10 @@ save_panel_configuration(gpointer data, gpointer user_data)
 		DrawerWidget *drawer = DRAWER_WIDGET(pd->panel);
 		gnome_config_set_int("orient",drawer->orient);
 		gnome_config_set_int("state", drawer->state);
+		gnome_config_set_bool("hidebutton_enabled",
+				      drawer->hidebutton_enabled);
+		gnome_config_set_bool("hidebutton_pixmap_enabled",
+				      drawer->hidebutton_pixmap_enabled);
 		break;
 		}
 	default:
@@ -377,12 +412,13 @@ do_session_save(GnomeClient *client,
 
 	/*DEBUG*/printf("Saving session: 1"); fflush(stdout);
 	if(complete_sync) {
-		for(i=0;i<applet_count;i++)
-			save_applet_configuration(i);
+		GList *li;
+		for(li=applets;li!=NULL;li=g_list_next(li))
+			save_applet_configuration(li->data);
 	} else {
 		GList *li;
 		for(li = sync_applets; li!=NULL; li=g_list_next(li))
-			save_applet_configuration(GPOINTER_TO_INT(li->data));
+			save_applet_configuration(li->data);
 	}
 	/*DEBUG*/printf(" 2"); fflush(stdout);
 
@@ -426,10 +462,10 @@ do_session_save(GnomeClient *client,
 				      global_config.disable_animations);
 		gnome_config_set_int("applet_padding",
 				     global_config.applet_padding);
-		gnome_config_set_bool("tiles_enabled",
-				      global_config.tiles_enabled);
 		for(i=0;i<LAST_TILE;i++) {
 			char buf[256];
+			g_snprintf(buf,256,"tiles_enabled_%d",i);
+			gnome_config_set_bool(buf,global_config.tiles_enabled[i]);
 			g_snprintf(buf,256,"tile_up_%d",i);
 			gnome_config_set_string(buf,global_config.tile_up[i]);
 			g_snprintf(buf,256,"tile_down_%d",i);
@@ -489,17 +525,15 @@ int
 panel_session_die (GnomeClient *client,
 		   gpointer client_data)
 {
-	AppletInfo *info;
-	int i;
+	GList *li;
 
 	gtk_timeout_remove(config_sync_timeout);
   
 	/*don't catch these any more*/
 	signal(SIGCHLD, SIG_DFL);
 	
-	for(i=0,info=(AppletInfo *)applets->data;
-	    i<applet_count;
-	    i++,info++) {
+	for(li=applets; li!=NULL; li=g_list_next(li)) {
+		AppletInfo *info = li->data;
 		if(info->type == APPLET_EXTERN) {
 			gtk_widget_destroy(info->widget);
 		} else if(info->type == APPLET_SWALLOW) {
@@ -513,7 +547,7 @@ panel_session_die (GnomeClient *client,
 	/*clean up corba stuff*/
 	panel_corba_clean_up();
 	
-	panel_corba_gtk_main_quit();
+	gtk_main_quit();
 	return TRUE;
 }
 
@@ -604,8 +638,24 @@ panel_quit(void)
 static void
 load_default_applets(void)
 {
+	char *def_launchers[] =
+	      { "apps/gnome-help.desktop",
+		"apps/System/gnomecc.desktop",
+		"apps/Applications/Netscape.desktop",
+		NULL };
+	int i;
 	load_menu_applet(NULL,0, panels->data, 0);
-	load_extern_applet("gen_util_applet","--clock",NULL,
+
+	for(i=0;def_launchers[i]!=NULL;i++) {
+		char *p = gnome_datadir_file (def_launchers[i]);
+		int center = gdk_screen_width()/2;
+		if(p) {
+			load_launcher_applet(p,panels->data,center+i);
+			g_free(p);
+		}
+	}
+
+	load_extern_applet("gen_util_clock",NULL,
 			   panels->data,INT_MAX/2/*right flush*/);
 	/*we laoded default applets, so we didn't find the config or
 	  something else was wrong, so do complete save when next syncing*/
@@ -659,15 +709,15 @@ init_user_applets(void)
 		pos += gnome_config_get_bool("right_stick=false")?INT_MAX/2:0;
 		
 		if(strcmp(applet_name,EXTERN_ID) == 0) {
-			char *path = gnome_config_get_string("execpath=");
-			char *params = gnome_config_get_string("parameters=");
-			/*this is the config path to be passed to the applet
-			  when it loads*/
-			g_snprintf(buf,256,"%sApplet_%d_Extern/",
-				   old_panel_cfg_path,num);
-			load_extern_applet(path,params,buf,panel,pos);
-			g_free(path);
-			g_free(params);
+			char *goad_id = gnome_config_get_string("goad_id=");
+			if(goad_id) {
+				/*this is the config path to be passed to the
+				  applet when it loads*/
+				g_snprintf(buf,256,"%sApplet_%d_Extern/",
+					   old_panel_cfg_path,num);
+				load_extern_applet(goad_id,buf,panel,pos);
+				g_free(goad_id);
+			}
 		} else if(strcmp(applet_name,LAUNCHER_ID) == 0) { 
 			char *params = gnome_config_get_string("parameters=");
 			load_launcher_applet(params,panel,pos);
@@ -699,6 +749,7 @@ init_user_applets(void)
 		} else
 			g_warning("Unknown applet type!");
 		gnome_config_pop_prefix();
+		g_free(applet_name);
 	}
 }
 
@@ -721,6 +772,7 @@ init_user_panels(void)
 					   SNAPPED_EXPLICIT_HIDE,
 					   SNAPPED_SHOWN,
 					   TRUE,
+					   TRUE,
 					   PANEL_BACK_NONE,
 					   NULL,
 					   TRUE,
@@ -771,6 +823,7 @@ init_user_panels(void)
 				SnappedMode mode;
 				SnappedState state;
 				int hidebuttons_enabled;
+				int hidebutton_pixmaps_enabled;
 
 				g_snprintf(buf,256,"pos=%d", SNAPPED_BOTTOM);
 				pos=gnome_config_get_int(buf);
@@ -783,11 +836,14 @@ init_user_panels(void)
 
 				hidebuttons_enabled =
 					gnome_config_get_bool("hidebuttons_enabled=TRUE");
+				hidebutton_pixmaps_enabled =
+					gnome_config_get_bool("hidebutton_pixmaps_enabled=TRUE");
 
 				panel = snapped_widget_new(pos,
 							   mode,
 							   state,
 							   hidebuttons_enabled,
+							   hidebutton_pixmaps_enabled,
 							   back_type,
 							   back_pixmap,
 							   fit_pixmap_bg,
@@ -798,6 +854,8 @@ init_user_panels(void)
 			{
 				DrawerState state;
 				PanelOrientType orient;
+				int hidebutton_enabled;
+				int hidebutton_pixmap_enabled;
 
 				g_snprintf(buf,256,"state=%d", DRAWER_SHOWN);
 				state=gnome_config_get_int(buf);
@@ -805,12 +863,19 @@ init_user_panels(void)
 				g_snprintf(buf,256,"orient=%d", ORIENT_UP);
 				orient=gnome_config_get_int(buf);
 
+				hidebutton_enabled =
+					gnome_config_get_bool("hidebutton_enabled=TRUE");
+				hidebutton_pixmap_enabled =
+					gnome_config_get_bool("hidebutton_pixmap_enabled=TRUE");
+
 				panel = drawer_widget_new(orient,
 							  state,
 							  back_type,
 							  back_pixmap,
 							  fit_pixmap_bg,
-							  &back_color);
+							  &back_color,
+							  hidebutton_pixmap_enabled,
+							  hidebutton_enabled);
 				break;
 			}
 		case CORNER_PANEL:
@@ -818,6 +883,8 @@ init_user_panels(void)
 				CornerPos pos;
 				PanelOrientation orient;
 				CornerState state;
+				int hidebuttons_enabled;
+				int hidebutton_pixmaps_enabled;
 				
 				g_snprintf(buf,256,"pos=%d", CORNER_NE);
 				pos=gnome_config_get_int(buf);
@@ -828,10 +895,17 @@ init_user_panels(void)
 
 				g_snprintf(buf,256,"state=%d", CORNER_SHOWN);
 				state=gnome_config_get_int(buf);
+
+				hidebuttons_enabled =
+					gnome_config_get_bool("hidebuttons_enabled=TRUE");
+				hidebutton_pixmaps_enabled =
+					gnome_config_get_bool("hidebutton_pixmaps_enabled=TRUE");
 				
 				panel = corner_widget_new(pos,
 							  orient,
 							  state,
+							  hidebuttons_enabled,
+							  hidebutton_pixmaps_enabled,
 							  back_type,
 							  back_pixmap,
 							  fit_pixmap_bg,
@@ -902,10 +976,10 @@ load_up_globals(void)
 
 	global_config.applet_padding=gnome_config_get_int("applet_padding=3");
 
-	global_config.tiles_enabled =
-		gnome_config_get_bool("tiles_enabled=TRUE");
-	
 	for(i=0;i<LAST_TILE;i++) {
+		g_snprintf(buf,256,"tiles_enabled_%d=TRUE",i);
+		global_config.tiles_enabled[i] = gnome_config_get_bool(buf);
+
 		g_free(global_config.tile_up[i]);
 		g_snprintf(buf,256,"tile_up_%d=tiles/tile-%s-up.png",
 			   i, tile_def[i]);
