@@ -21,6 +21,8 @@ static void snapped_widget_size_request	(GtkWidget          *widget,
 static void snapped_widget_size_allocate(GtkWidget          *widget,
 					 GtkAllocation      *allocation);
 
+static GtkWindowClass *parent_class = NULL;
+
 extern GdkCursor *fleur_cursor;
 
 /*global settings*/
@@ -87,13 +89,84 @@ marshal_signal_int (GtkObject * object,
 		  func_data);
 }
 
+static void
+set_frame_colors(SnappedWidget *snapped)
+{
+	if(PANEL_WIDGET(snapped->panel)->back_type == PANEL_BACK_COLOR) {
+		GdkColor n;
+		GtkStyle *ns;
+
+		ns = gtk_style_copy(snapped->panel->style);
+		gtk_style_ref(ns);
+
+		ns->bg[GTK_STATE_NORMAL] =
+			PANEL_WIDGET(snapped->panel)->back_color;
+		ns->base[GTK_STATE_NORMAL] =
+			PANEL_WIDGET(snapped->panel)->back_color;
+
+		n = PANEL_WIDGET(snapped->panel)->back_color;
+		n.red/=3;
+		n.green/=3;
+		n.blue/=3;
+		ns->dark[GTK_STATE_NORMAL] = n;
+
+		n = PANEL_WIDGET(snapped->panel)->back_color;
+		n.red/=2;
+		n.green/=2;
+		n.blue/=2;
+		ns->mid[GTK_STATE_NORMAL] = n;
+
+		n = PANEL_WIDGET(snapped->panel)->back_color;
+		n.red=MIN(65535,n.red*2);
+		n.green=MIN(65535,n.green*2);
+		n.blue=MIN(65535,n.blue*2);
+		ns->light[GTK_STATE_NORMAL] = n;
+
+		gtk_widget_set_style(snapped->frame, ns);
+
+		gtk_style_unref(ns);
+	} else {
+		GtkStyle *ns;
+
+		ns = gtk_rc_get_style(snapped->frame);
+		if(!ns) ns = gtk_style_new();
+
+		gtk_style_ref(ns);
+		gtk_widget_set_style(snapped->frame, ns);
+		gtk_style_unref(ns);
+	}
+}
+
+static void
+snapped_widget_realize(GtkWidget *w)
+{
+	SnappedWidget *snapped = SNAPPED_WIDGET(w);
+	GTK_WIDGET_CLASS(parent_class)->realize(w);
+
+	gnome_win_hints_init();
+	if (gnome_win_hints_wm_exists()) {
+		gnome_win_hints_set_hints(w, 
+					  WIN_HINTS_SKIP_FOCUS |
+					  WIN_HINTS_SKIP_WINLIST |
+					  WIN_HINTS_SKIP_TASKBAR);
+		gnome_win_hints_set_state(w, 
+					  WIN_STATE_STICKY |
+					  WIN_STATE_FIXED_POSITION);
+		gnome_win_hints_set_layer(w, WIN_LAYER_DOCK);
+		gnome_win_hints_set_expanded_size(w, 0, 0, 0, 0);
+		gdk_window_set_decorations(w->window, 0);
+	}    
+	set_frame_colors(snapped);
+}
+
 static void
 snapped_widget_class_init (SnappedWidgetClass *class)
 {
 	GtkObjectClass *object_class = (GtkObjectClass*) class;
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) class;
 
-	snapped_widget_signals[POS_CHANGE_SIGNAL] =
+        parent_class = gtk_type_class (gtk_window_get_type ());
+        snapped_widget_signals[POS_CHANGE_SIGNAL] =
 		gtk_signal_new("pos_change",
 			       GTK_RUN_LAST,
 			       object_class->type,
@@ -122,6 +195,7 @@ snapped_widget_class_init (SnappedWidgetClass *class)
 	
 	widget_class->size_request = snapped_widget_size_request;
 	widget_class->size_allocate = snapped_widget_size_allocate;
+	widget_class->realize = snapped_widget_realize;
 }
 
 /*if this is true the size request will request a 48x48 cube, this is used
@@ -459,11 +533,14 @@ static void
 snapped_widget_pop_show(SnappedWidget *snapped, int fromright)
 {
 	int width, height;
+	static const char *supinfo[] = {"panel", "collapse", NULL};
 
 	if ((snapped->state == SNAPPED_MOVING) ||
 	    (snapped->state == SNAPPED_SHOWN))
 		return;
 
+	gnome_triggers_vdo("", NULL, supinfo);
+
 	snapped->state = SNAPPED_MOVING;
 
 	width   = GTK_WIDGET(snapped)->allocation.width;
@@ -500,10 +577,12 @@ static void
 snapped_widget_pop_hide(SnappedWidget *snapped, int fromright)
 {
 	int width, height;
+	static const char *supinfo[] = {"panel", "collapse", NULL};
 
 	if((snapped->state != SNAPPED_SHOWN))
 		return;
 	
+	gnome_triggers_vdo("", NULL, supinfo);
 
 	if (snapped->leave_notify_timer_tag != 0) {
 		gtk_timeout_remove (snapped->leave_notify_timer_tag);
@@ -585,8 +664,8 @@ snapped_enter_notify(SnappedWidget *snapped,
 		     GdkEventCrossing *event,
 		     gpointer data)
 {
-	/*FIXME: do we want this autoraise piece?*/
-	gdk_window_raise(GTK_WIDGET(snapped)->window);
+  if (!gnome_win_hints_wm_exists())
+    gdk_window_raise(GTK_WIDGET(snapped)->window);
 
 	if ((snapped->mode == SNAPPED_EXPLICIT_HIDE) ||
 	    (event->detail == GDK_NOTIFY_INFERIOR) ||
@@ -645,6 +724,15 @@ snapped_widget_set_hidebuttons(SnappedWidget *snapped)
 		gtk_widget_hide(snapped->hidebutton_e);
 		gtk_widget_hide(snapped->hidebutton_w);
 		gtk_widget_hide(snapped->hidebutton_s);
+		/*in case the panel was hidden, show it, since otherwise
+		  we wouldn't see it anymore*/
+		if(snapped->state == SNAPPED_HIDDEN_RIGHT) {
+			snapped_widget_pop_show(snapped,FALSE);
+			snapped_widget_queue_pop_down(snapped);
+		} else if(snapped->state == SNAPPED_HIDDEN_LEFT) {
+			snapped_widget_pop_show(snapped,TRUE);
+			snapped_widget_queue_pop_down(snapped);
+		}
 	/* horizontal and enabled */
 	} else if(PANEL_WIDGET(snapped->panel)->orient == PANEL_HORIZONTAL) {
 		gtk_widget_hide(snapped->hidebutton_n);
@@ -682,12 +770,38 @@ make_hidebutton(SnappedWidget *snapped,
 	gtk_widget_show(pixmap);
 
 	gtk_container_add(GTK_CONTAINER(w),pixmap);
+	gtk_object_set_user_data(GTK_OBJECT(w), pixmap);
 	gtk_signal_connect(GTK_OBJECT(w), "clicked",
 			   GTK_SIGNAL_FUNC(hidefunc),
 			   snapped);
 	return w;
 }
 
+static void
+show_hidebutton_pixmap(GtkWidget *hidebutton, int show)
+{
+	GtkWidget *pixmap;
+
+	pixmap = gtk_object_get_user_data(GTK_OBJECT(hidebutton));
+
+	if (!pixmap) return;
+
+	if (show)
+		gtk_widget_show(pixmap);
+	else
+		gtk_widget_hide(pixmap);
+}
+
+static void
+snapped_widget_show_hidebutton_pixmaps(SnappedWidget *snapped)
+{
+	int show = snapped->hidebutton_pixmaps_enabled;
+	show_hidebutton_pixmap(snapped->hidebutton_n, show);
+	show_hidebutton_pixmap(snapped->hidebutton_e, show);
+	show_hidebutton_pixmap(snapped->hidebutton_w, show);
+	show_hidebutton_pixmap(snapped->hidebutton_s, show);
+}
+
 static int
 snapped_widget_destroy(GtkWidget *w, gpointer data)
 {
@@ -697,8 +811,12 @@ snapped_widget_destroy(GtkWidget *w, gpointer data)
 static void
 snapped_widget_init (SnappedWidget *snapped)
 {
-	/*if we set the icewm hints it will have to be changed to TOPLEVEL*/
-	GTK_WINDOW(snapped)->type = GTK_WINDOW_POPUP;
+	/*if we set the gnomewm hints it will have to be changed to TOPLEVEL*/
+	gnome_win_hints_init();
+	if (gnome_win_hints_wm_exists())
+		GTK_WINDOW(snapped)->type = GTK_WINDOW_TOPLEVEL;
+	else
+		GTK_WINDOW(snapped)->type = GTK_WINDOW_POPUP;
 	GTK_WINDOW(snapped)->allow_shrink = TRUE;
 	GTK_WINDOW(snapped)->allow_grow = TRUE;
 	GTK_WINDOW(snapped)->auto_shrink = TRUE;
@@ -760,17 +878,29 @@ snapped_widget_init (SnappedWidget *snapped)
 	snapped->mode = SNAPPED_EXPLICIT_HIDE;
 	snapped->state = SNAPPED_SHOWN;
 	snapped->hidebuttons_enabled = TRUE;
+	snapped->hidebutton_pixmaps_enabled = TRUE;
 
 	snapped->leave_notify_timer_tag = 0;
 	snapped->autohide_inhibit = FALSE;
 	snapped->drawers_open = 0;
 }
 
+static void
+s_back_change(PanelWidget *panel,
+	    PanelBackType type,
+	    char *pixmap,
+	    GdkColor *color,
+	    SnappedWidget *snapped)
+{
+	set_frame_colors(snapped);
+}
+
 GtkWidget*
 snapped_widget_new (SnappedPos pos,
 		    SnappedMode mode,
 		    SnappedState state,
 		    int hidebuttons_enabled,
+		    int hidebutton_pixmaps_enabled,
 		    PanelBackType back_type,
 		    char *back_pixmap,
 		    int fit_pixmap_bg,
@@ -795,18 +925,21 @@ snapped_widget_new (SnappedPos pos,
 					  back_pixmap,
 					  fit_pixmap_bg,
 					  back_color);
+	gtk_signal_connect_after(GTK_OBJECT(snapped->panel), "back_change",
+				 GTK_SIGNAL_FUNC(s_back_change),
+				 snapped);
 	gtk_object_set_data(GTK_OBJECT(snapped->panel),PANEL_PARENT,
 			    snapped);
 	PANEL_WIDGET(snapped->panel)->drop_widget = GTK_WIDGET(snapped);
 
 	gtk_widget_show(snapped->panel);
 
-	frame = gtk_frame_new(NULL);
-	gtk_widget_show(frame);
-	gtk_frame_set_shadow_type(GTK_FRAME(frame),GTK_SHADOW_OUT);
-	gtk_container_add(GTK_CONTAINER(frame),snapped->panel);
+	snapped->frame = gtk_frame_new(NULL);
+	gtk_widget_show(snapped->frame);
+	gtk_frame_set_shadow_type(GTK_FRAME(snapped->frame),GTK_SHADOW_OUT);
+	gtk_container_add(GTK_CONTAINER(snapped->frame),snapped->panel);
 
-	gtk_table_attach(GTK_TABLE(snapped->table),frame,1,2,1,2,
+	gtk_table_attach(GTK_TABLE(snapped->table),snapped->frame,1,2,1,2,
 			 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
 			 GTK_FILL|GTK_EXPAND|GTK_SHRINK,
 			 0,0);
@@ -814,6 +947,7 @@ snapped_widget_new (SnappedPos pos,
 	snapped->pos = pos;
 	snapped->mode = mode;
 	snapped->hidebuttons_enabled = hidebuttons_enabled;
+	snapped->hidebutton_pixmaps_enabled = hidebutton_pixmaps_enabled;
 	if(state != SNAPPED_MOVING)
 		snapped->state = state;
 
@@ -825,6 +959,7 @@ snapped_widget_new (SnappedPos pos,
 		snapped->state = SNAPPED_SHOWN;
 
 	snapped_widget_set_hidebuttons(snapped);
+/* */	snapped_widget_show_hidebutton_pixmaps(snapped);
 	snapped_widget_set_initial_pos(snapped);
 
 	if(snapped->mode == SNAPPED_AUTO_HIDE)
@@ -839,6 +974,7 @@ snapped_widget_change_params(SnappedWidget *snapped,
 			     SnappedMode mode,
 			     SnappedState state,
 			     int hidebuttons_enabled,
+			     int hidebutton_pixmaps_enabled,
 			     PanelBackType back_type,
 			     char *pixmap_name,
 			     int fit_pixmap_bg,
@@ -858,6 +994,7 @@ snapped_widget_change_params(SnappedWidget *snapped,
 	snapped->state = state;
 	snapped->mode = mode;
 	snapped->hidebuttons_enabled = hidebuttons_enabled;
+	snapped->hidebutton_pixmaps_enabled = hidebutton_pixmaps_enabled;
 
 	if(snapped->pos == SNAPPED_TOP ||
 	   snapped->pos == SNAPPED_BOTTOM)
@@ -883,6 +1020,7 @@ snapped_widget_change_params(SnappedWidget *snapped,
 				   back_color);
 
 	snapped_widget_set_hidebuttons(snapped);
+/* */	snapped_widget_show_hidebutton_pixmaps(snapped);
 	gtk_widget_queue_resize(GTK_WIDGET(snapped));
 
 	if(snapped->mode == SNAPPED_EXPLICIT_HIDE &&
@@ -913,6 +1051,7 @@ snapped_widget_change_pos(SnappedWidget *snapped,
 				     snapped->mode,
 				     snapped->state,
 				     snapped->hidebuttons_enabled,
+				     snapped->hidebutton_pixmaps_enabled,
 				     panel->back_type,
 				     panel->back_pixmap,
 				     panel->fit_pixmap_bg,
